import { BytesLike, ContractTransaction, Signer } from 'ethers';
import { SublimeConfig } from '../types/sublimeConfig';
import { LoanStatus, PoolGenerateParams } from '../types/poolGenerateParam';
import { PoolInfo } from '../types/poolInfo';
import { TokenManager } from '../tokenManager';
import { Options as Overrides } from '../types/Types';
import { PoolEthUtils } from './utils/poolEthUtils';
/**
 * @todo Pools are not let released
 */
export declare class PoolApi {
    private signer;
    private config;
    private tokenManager;
    private poolFactory;
    constructor(signer: Signer, config: SublimeConfig, tokenManager: TokenManager);
    PoolUtils(): PoolEthUtils;
    createPool(params: PoolGenerateParams, options?: Overrides): Promise<ContractTransaction>;
    generatePoolAddress(creator: string, salt: BytesLike): Promise<string>;
    depositCollateral(poolContract: string, amount: string, transferFromSavingsAccount: boolean, options?: Overrides): Promise<ContractTransaction>;
    interestTillNow(poolContract: string): Promise<string>;
    getCurrentCollateralRatio(poolContract: string): Promise<string>;
    liquidatePool(poolContract: string, fromSavingsAccount: boolean, toSavingsAccount: boolean, recieveLiquidityShare: boolean, options?: Overrides): Promise<ContractTransaction>;
    liquidateLender(poolContract: string, lender: string, fromSavingsAccount: boolean, toSavingsAccount: boolean, recieveLiquidityShare: boolean, options?: Overrides): Promise<ContractTransaction>;
    interestPerPeriod(poolContract: string, amount: string): Promise<string>;
    interestPerSecond(poolContract: string, amount: string): Promise<string>;
    calculateCurrentPeriod(poolContract: string): Promise<string>;
    getMarginCallEndTime(poolContract: string, lender: string): Promise<string>;
    getLoanStatus(poolContract: string): Promise<LoanStatus>;
    getBalanceDetails(poolContract: string, lender: string): Promise<string>;
    getTotalSupply(poolContract: string): Promise<string>;
    getPoolInfo(poolContract: string): Promise<PoolInfo>;
}
