"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PoolApi = void 0;
const bignumber_js_1 = require("bignumber.js");
const poolGenerateParam_1 = require("../types/poolGenerateParam");
const PoolFactory__factory_1 = require("../wrappers/factories/PoolFactory__factory");
const Pool__factory_1 = require("../wrappers/factories/Pool__factory");
const IYield__factory_1 = require("../wrappers/factories/IYield__factory");
const constants_1 = require("../config/constants");
const poolEthUtils_1 = require("./utils/poolEthUtils");
/**
 * @todo Pools are not let released
 */
class PoolApi {
    constructor(signer, config, tokenManager) {
        this.signer = signer;
        this.config = config;
        this.tokenManager = tokenManager;
        this.poolFactory = new PoolFactory__factory_1.PoolFactory__factory(this.signer).attach(config.poolFactoryContractAddress);
    }
    PoolUtils() {
        return new poolEthUtils_1.PoolEthUtils(this.signer, this.config, this.tokenManager);
    }
    createPool(params, options) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.tokenManager.updateTokenDecimals(params.borrowToken);
            const borrowDecimal = this.tokenManager.getTokenDecimals(params.borrowToken);
            yield this.tokenManager.updateTokenDecimals(params.collateralToken);
            const collateralDecimal = this.tokenManager.getTokenDecimals(params.collateralToken);
            const borrowAmountRequests = new bignumber_js_1.BigNumber(params.borrowAmountRequests);
            if (borrowAmountRequests.isNaN() || borrowAmountRequests.isZero() || borrowAmountRequests.isNegative()) {
                throw new Error('borrowAmountRequests should be a valid number');
            }
            const collateralVolatilityThreshold = new bignumber_js_1.BigNumber(params.collateralVolatilityThreshold);
            if (collateralVolatilityThreshold.isNaN() || collateralVolatilityThreshold.isZero() || collateralVolatilityThreshold.isNegative()) {
                throw new Error('collateralVolatilityThreshold should be a valid number');
            }
            const collateralRatio = new bignumber_js_1.BigNumber(params.collateralRatio);
            if (collateralRatio.isNaN() || collateralRatio.isZero() || collateralRatio.isNegative()) {
                throw new Error('collateralRatio should be a valid number');
            }
            const borrowRate = new bignumber_js_1.BigNumber(params.borrowRate);
            if (borrowRate.isNaN() || borrowRate.isZero() || borrowRate.isNegative()) {
                throw new Error('borrowRate should be a valid number');
            }
            const repaymentInterval = new bignumber_js_1.BigNumber(params.repaymentInterval);
            if (repaymentInterval.isNaN() || repaymentInterval.isZero() || repaymentInterval.isNegative()) {
                throw new Error('repaymentInterval should be a valid number');
            }
            const noOfRepaymentIntervals = new bignumber_js_1.BigNumber(params.noOfRepaymentIntervals);
            if (noOfRepaymentIntervals.isNaN() || noOfRepaymentIntervals.isZero() || noOfRepaymentIntervals.isNegative()) {
                throw new Error('noOfRepaymentIntervals should be a valid number');
            }
            const collateralAmount = new bignumber_js_1.BigNumber(params.collateralAmount);
            if (collateralAmount.isNaN() || collateralAmount.isZero() || collateralAmount.isNegative()) {
                throw new Error('collateralAmount should be a valid number');
            }
            return yield this.poolFactory.createPool(borrowAmountRequests.multipliedBy(new bignumber_js_1.BigNumber(10).pow(borrowDecimal)).toFixed(0), borrowRate.multipliedBy(new bignumber_js_1.BigNumber(10).pow(28)).toFixed(0), params.borrowToken, params.collateralToken, collateralRatio.multipliedBy(new bignumber_js_1.BigNumber(10).pow(28)).toFixed(0), repaymentInterval.toFixed(0), noOfRepaymentIntervals.toFixed(0), params.strategy, collateralAmount.multipliedBy(new bignumber_js_1.BigNumber(10).pow(collateralDecimal)).toFixed(0), params.transferFromSavingsAccount, params.salt, this.config.twitterVerifierContractAddress, params.lenderVerifier, Object.assign({}, options));
        });
    }
    generatePoolAddress(creator, salt) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.poolFactory.connect(this.signer).preComputeAddress(creator, salt);
        });
    }
    depositCollateral(poolContract, amount, transferFromSavingsAccount, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const pool = new Pool__factory_1.Pool__factory(this.signer).attach(poolContract);
            const collateralAsset = (yield pool.poolConstants()).collateralAsset;
            yield this.tokenManager.updateTokenDecimals(collateralAsset);
            const collateralDecimal = this.tokenManager.getTokenDecimals(collateralAsset);
            const _amount = new bignumber_js_1.BigNumber(amount);
            if (_amount.isNaN() || _amount.isZero() || _amount.isNegative()) {
                throw new Error('amount should be a valid number');
            }
            return pool.depositCollateral(_amount.multipliedBy(new bignumber_js_1.BigNumber(10).pow(collateralDecimal.toString())).toFixed(0), transferFromSavingsAccount, Object.assign({}, options));
        });
    }
    interestTillNow(poolContract) {
        return __awaiter(this, void 0, void 0, function* () {
            const pool = new Pool__factory_1.Pool__factory(this.signer).attach(poolContract);
            const borrowAsset = (yield pool.poolConstants()).borrowAsset;
            yield this.tokenManager.updateTokenDecimals(borrowAsset);
            const borrowDecimal = this.tokenManager.getTokenDecimals(borrowAsset);
            const interestTillNow = yield (yield pool.interestToPay()).toString();
            return new bignumber_js_1.BigNumber(interestTillNow).div(new bignumber_js_1.BigNumber(10).pow(borrowDecimal.toFixed(0))).toFixed(2);
        });
    }
    getCurrentCollateralRatio(poolContract) {
        return __awaiter(this, void 0, void 0, function* () {
            const pool = new Pool__factory_1.Pool__factory(this.signer).attach(poolContract);
            const ccr = yield (yield pool.callStatic['getCurrentCollateralRatio()']()).toString();
            return new bignumber_js_1.BigNumber(ccr).div(new bignumber_js_1.BigNumber(10).pow(28)).toFixed(2);
        });
    }
    liquidatePool(poolContract, fromSavingsAccount, toSavingsAccount, recieveLiquidityShare, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const pool = new Pool__factory_1.Pool__factory(this.signer).attach(poolContract);
            return yield pool.liquidatePool(fromSavingsAccount, toSavingsAccount, recieveLiquidityShare, Object.assign({}, options));
        });
    }
    liquidateLender(poolContract, lender, fromSavingsAccount, toSavingsAccount, recieveLiquidityShare, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const pool = new Pool__factory_1.Pool__factory(this.signer).attach(poolContract);
            return yield pool.liquidateForLender(lender, fromSavingsAccount, toSavingsAccount, recieveLiquidityShare, Object.assign({}, options));
        });
    }
    interestPerPeriod(poolContract, amount) {
        return __awaiter(this, void 0, void 0, function* () {
            const pool = new Pool__factory_1.Pool__factory(this.signer).attach(poolContract);
            const borrowAsset = (yield pool.poolConstants()).borrowAsset;
            yield this.tokenManager.updateTokenDecimals(borrowAsset);
            const borrowDecimal = this.tokenManager.getTokenDecimals(borrowAsset);
            const _amount = new bignumber_js_1.BigNumber(amount).multipliedBy(new bignumber_js_1.BigNumber(10).pow(borrowDecimal));
            const interestPerPeriod = yield (yield pool.interestPerPeriod(_amount.toFixed(0))).toString();
            const value = new bignumber_js_1.BigNumber(interestPerPeriod).div(new bignumber_js_1.BigNumber(10).pow(borrowDecimal));
            return value.div(new bignumber_js_1.BigNumber(10).pow(30)).toFixed(2);
        });
    }
    interestPerSecond(poolContract, amount) {
        return __awaiter(this, void 0, void 0, function* () {
            const pool = new Pool__factory_1.Pool__factory(this.signer).attach(poolContract);
            const borrowAsset = (yield pool.poolConstants()).borrowAsset;
            yield this.tokenManager.updateTokenDecimals(borrowAsset);
            const borrowDecimal = this.tokenManager.getTokenDecimals(borrowAsset);
            const _amount = new bignumber_js_1.BigNumber(amount).multipliedBy(new bignumber_js_1.BigNumber(10).pow(borrowDecimal));
            const interestPerSecond = yield (yield pool.interestPerSecond(_amount.toFixed(0))).toString();
            const value = new bignumber_js_1.BigNumber(interestPerSecond).div(new bignumber_js_1.BigNumber(10).pow(borrowDecimal));
            return value.div(new bignumber_js_1.BigNumber(10).pow(30)).toFixed(2);
        });
    }
    calculateCurrentPeriod(poolContract) {
        return __awaiter(this, void 0, void 0, function* () {
            const pool = new Pool__factory_1.Pool__factory(this.signer).attach(poolContract);
            return (yield pool.calculateCurrentPeriod()).toString();
        });
    }
    getMarginCallEndTime(poolContract, lender) {
        return __awaiter(this, void 0, void 0, function* () {
            const pool = new Pool__factory_1.Pool__factory(this.signer).attach(poolContract);
            return (yield pool.getMarginCallEndTime(lender)).toString();
        });
    }
    getLoanStatus(poolContract) {
        return __awaiter(this, void 0, void 0, function* () {
            const pool = new Pool__factory_1.Pool__factory(this.signer).attach(poolContract);
            const _status = (yield pool.getLoanStatus()).toString();
            return poolGenerateParam_1.LoanStatus[_status];
        });
    }
    getBalanceDetails(poolContract, lender) {
        return __awaiter(this, void 0, void 0, function* () {
            const pool = new Pool__factory_1.Pool__factory(this.signer).attach(poolContract);
            yield this.tokenManager.updateTokenDecimals(poolContract);
            const _decimal = this.tokenManager.getTokenDecimals(poolContract);
            const details = yield (yield pool.getBalanceDetails(lender)).map((a) => new bignumber_js_1.BigNumber(a.toString()).div(new bignumber_js_1.BigNumber(10).pow(_decimal)));
            return details[0].toFixed(2);
        });
    }
    getTotalSupply(poolContract) {
        return __awaiter(this, void 0, void 0, function* () {
            const pool = new Pool__factory_1.Pool__factory(this.signer).attach(poolContract);
            yield this.tokenManager.updateTokenDecimals(poolContract);
            const _decimal = this.tokenManager.getTokenDecimals(poolContract);
            const totalSupply = yield pool.totalSupply();
            return new bignumber_js_1.BigNumber(totalSupply.toString()).div(new bignumber_js_1.BigNumber(10).pow(_decimal)).toFixed(2);
        });
    }
    getPoolInfo(poolContract) {
        return __awaiter(this, void 0, void 0, function* () {
            const pool = new Pool__factory_1.Pool__factory(this.signer).attach(poolContract);
            const result = {};
            const poolConstants = yield pool.poolConstants();
            let liquidityToken;
            const poolSavingsStrategy = poolConstants.poolSavingsStrategy;
            if (poolSavingsStrategy === constants_1.zeroAddress) {
                liquidityToken = constants_1.zeroAddress;
            }
            else {
                liquidityToken = yield IYield__factory_1.IYield__factory.connect(poolSavingsStrategy, this.signer).liquidityToken(poolConstants.collateralAsset);
            }
            const allTokens = [poolConstants.borrowAsset, poolConstants.collateralAsset, liquidityToken].filter((value, index, array) => array.indexOf(value) === index);
            for (let index = 0; index < allTokens.length; index++) {
                const element = allTokens[index];
                yield this.tokenManager.updateAll(element);
            }
            const minBorrowFraction = yield this.poolFactory.minBorrowFraction();
            result.borrowAmountRequested = new bignumber_js_1.BigNumber(poolConstants.borrowAmountRequested.toString())
                .div(new bignumber_js_1.BigNumber(10).pow(this.tokenManager.getTokenDecimals(poolConstants.borrowAsset.toLowerCase())))
                .toFixed(2);
            result.minborrowAmount = new bignumber_js_1.BigNumber(poolConstants.borrowAmountRequested.toString())
                .multipliedBy(minBorrowFraction.toString())
                .div(new bignumber_js_1.BigNumber(10).pow(28))
                .div(new bignumber_js_1.BigNumber(10).pow(this.tokenManager.getTokenDecimals(poolConstants.borrowAsset.toLowerCase())))
                .toFixed(2);
            result.loanStartTime = poolConstants.loanStartTime.toString();
            result.loanWithdrawalDeadline = poolConstants.loanWithdrawalDeadline.toString();
            result.borrowAsset = {
                address: poolConstants.borrowAsset,
                name: this.tokenManager.getTokenName(poolConstants.borrowAsset.toLowerCase()),
                pricePerAssetInUSD: (yield this.tokenManager.getPricePerAsset(poolConstants.borrowAsset)).toString(),
                logo: this.tokenManager.getLogo(poolConstants.borrowAsset),
                decimals: this.tokenManager.getTokenDecimals(poolConstants.borrowAsset),
            };
            result.idealCollateralRatio = new bignumber_js_1.BigNumber(poolConstants.idealCollateralRatio.toString()).div(new bignumber_js_1.BigNumber(10).pow(28)).toFixed(2);
            result.borrowRate = new bignumber_js_1.BigNumber(poolConstants.borrowRate.toString()).div(new bignumber_js_1.BigNumber(10).pow(28)).toFixed(2);
            result.noOfRepaymentIntervals = poolConstants.noOfRepaymentIntervals.toString();
            result.repaymentInterval = poolConstants.repaymentInterval.toString();
            result.collateralAsset = {
                address: poolConstants.collateralAsset,
                name: this.tokenManager.getTokenName(poolConstants.collateralAsset),
                pricePerAssetInUSD: (yield this.tokenManager.getPricePerAsset(poolConstants.collateralAsset)).toString(),
                logo: this.tokenManager.getLogo(poolConstants.collateralAsset),
                decimals: this.tokenManager.getTokenDecimals(poolConstants.collateralAsset),
            };
            result.poolSavingsStrategy = poolConstants.poolSavingsStrategy;
            const poolVars = yield pool.poolVars();
            result.baseLiquidityShares = new bignumber_js_1.BigNumber(poolVars.baseLiquidityShares.toString())
                .div(new bignumber_js_1.BigNumber(10).pow(this.tokenManager.getTokenDecimals(liquidityToken.toLowerCase())))
                .toFixed(2);
            result.extraLiquidityShares = new bignumber_js_1.BigNumber(poolVars.extraLiquidityShares.toString())
                .div(new bignumber_js_1.BigNumber(10).pow(this.tokenManager.getTokenDecimals(liquidityToken.toLowerCase())))
                .toFixed(2);
            result.loanStatus = poolGenerateParam_1.LoanStatus[poolVars.loanStatus.toString()];
            result.penaltyLiquidityAmount = new bignumber_js_1.BigNumber(poolVars.penaltyLiquidityAmount.toString())
                .div(new bignumber_js_1.BigNumber(10).pow(this.tokenManager.getTokenDecimals(poolConstants.collateralAsset.toLowerCase())))
                .toFixed(2);
            return result;
        });
    }
}
exports.PoolApi = PoolApi;
//# sourceMappingURL=pool.js.map