import { SublimeConfig } from '../types/sublimeConfig';
import { ContractTransaction, Signer } from 'ethers';
import { Balance, Options } from '../types/Types';
/**
 * @class PriceOracle
 */
export declare class PriceOracleApi {
    private priceOracleContract;
    private config;
    constructor(signer: Signer, config: SublimeConfig);
    /**
     * @description Returns the
     * @param num: first token address
     * @param den: second token address
     * @returns price of token1 in terms of token2
     */
    getLatestPrice(num: string, den: string): Promise<string>;
    /**
     *
     * @param token
     * @returns Returns the addresses of chainlink feed to which the tokens prices are linked
     */
    getChainLinkFeedAddress(token: string): Promise<[string, number]>;
    /**
     *
     * @param num
     * @param den
     * @returns Returns true if the both the tokens are mapped against each other in the oracle
     */
    checkIfFeedExists(num: string, den: string): Promise<boolean>;
    /**
     *
     * @param num
     * @param den
     * @returns Returns the price from chainlink
     */
    getUniswapLatestPrice(num: string, den: string): Promise<Balance>;
    /**
     *
     * @param num
     * @param den
     * @returns Returns the price of uniswap
     */
    getChainlinkLatestPrice(num: string, den: string): Promise<Balance>;
    /**
     * @description Can only be called by admin
     * @param token
     * @param feed
     * @param options
     * @returns
     */
    setChainlinkFeedAddress(token: string, feed: string, options?: Options): Promise<ContractTransaction>;
    /**
     * @description Can be only called by admin
     * @param token
     * @param feed
     * @param options
     * @returns
     */
    setUniswapFeedAddress(token1: string, token2: string, feed: string, options?: Options): Promise<ContractTransaction>;
    /**
     * @description Can be only called by admin
     * @param avgPeriod
     * @param options
     * @returns
     */
    setUniswapAveragingPeriod(avgPeriod: string, options?: Options): Promise<ContractTransaction>;
}
