"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PriceOracleApi = void 0;
const PriceOracle__factory_1 = require("../wrappers/factories/PriceOracle__factory");
const bignumber_1 = require("@ethersproject/bignumber");
/**
 * @class PriceOracle
 */
class PriceOracleApi {
    constructor(signer, config) {
        this.priceOracleContract = new PriceOracle__factory_1.PriceOracle__factory(signer).attach(config.priceOracleContractAddress);
        config = this.config;
    }
    /**
     * @description Returns the
     * @param num: first token address
     * @param den: second token address
     * @returns price of token1 in terms of token2
     */
    getLatestPrice(num, den) {
        return __awaiter(this, void 0, void 0, function* () {
            const [ratio, decimals] = yield this.priceOracleContract.getLatestPrice(num, den);
            return ratio.div(bignumber_1.BigNumber.from(10).pow(decimals)).toString();
        });
    }
    /**
     *
     * @param token
     * @returns Returns the addresses of chainlink feed to which the tokens prices are linked
     */
    getChainLinkFeedAddress(token) {
        return __awaiter(this, void 0, void 0, function* () {
            const [address, decimals] = yield this.priceOracleContract.chainlinkFeedAddresses(token);
            return [address, decimals];
        });
    }
    /**
     *
     * @param num
     * @param den
     * @returns Returns true if the both the tokens are mapped against each other in the oracle
     */
    checkIfFeedExists(num, den) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.priceOracleContract.doesFeedExist(num, den);
        });
    }
    /**
     *
     * @param num
     * @param den
     * @returns Returns the price from chainlink
     */
    getUniswapLatestPrice(num, den) {
        return __awaiter(this, void 0, void 0, function* () {
            const [price] = yield this.priceOracleContract.getUniswapLatestPrice(num, den);
            return { value: price.toString(), decimals: 18 };
        });
    }
    /**
     *
     * @param num
     * @param den
     * @returns Returns the price of uniswap
     */
    getChainlinkLatestPrice(num, den) {
        return __awaiter(this, void 0, void 0, function* () {
            const [price] = yield this.priceOracleContract.getChainlinkLatestPrice(num, den);
            return { value: price.toString(), decimals: 18 };
        });
    }
    /**
     * @description Can only be called by admin
     * @param token
     * @param feed
     * @param options
     * @returns
     */
    setChainlinkFeedAddress(token, feed, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.priceOracleContract.setChainlinkFeedAddress(token, feed, Object.assign({}, options));
        });
    }
    /**
     * @description Can be only called by admin
     * @param token
     * @param feed
     * @param options
     * @returns
     */
    setUniswapFeedAddress(token1, token2, feed, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.priceOracleContract.setUniswapFeedAddress(token1, token2, feed, Object.assign({}, options));
        });
    }
    /**
     * @description Can be only called by admin
     * @param avgPeriod
     * @param options
     * @returns
     */
    setUniswapAveragingPeriod(avgPeriod, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.priceOracleContract.setUniswapPriceAveragingPeriod(avgPeriod, Object.assign({}, options));
        });
    }
}
exports.PriceOracleApi = PriceOracleApi;
//# sourceMappingURL=priceOracle.js.map