"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RepaymentApi = void 0;
const Repayments__factory_1 = require("../wrappers/factories/Repayments__factory");
const bignumber_js_1 = require("bignumber.js");
/**
 * @todo Repayment are part of pools and yet to be released
 */
class RepaymentApi {
    constructor(signer, config, tokenManager) {
        this.signer = signer;
        this.repayments = new Repayments__factory_1.Repayments__factory(this.signer).attach(config.repaymentContractAddress);
        this.tokenManager = tokenManager;
    }
    getInterestPerSecond(pool) {
        return __awaiter(this, void 0, void 0, function* () {
            const poolConstants = yield this.repayments.repayInfo(pool);
            yield this.tokenManager.updateTokenDecimals(poolConstants.repayAsset);
            const decimal = this.tokenManager.getTokenDecimals(poolConstants.repayAsset);
            const interestPerPeriod = yield (yield this.repayments.getInterestPerSecond(pool)).toString();
            return new bignumber_js_1.BigNumber(interestPerPeriod).div(new bignumber_js_1.BigNumber(10).pow(decimal)).toFixed(2);
        });
    }
    getInstalmentsCompleted(pool) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield (yield this.repayments.getInstalmentsCompleted(pool)).toString();
        });
    }
    getInterestDueTillInstalmentDeadline(pool) {
        return __awaiter(this, void 0, void 0, function* () {
            const poolConstants = yield this.repayments.repayInfo(pool);
            yield this.tokenManager.updateTokenDecimals(poolConstants.repayAsset);
            const decimal = this.tokenManager.getTokenDecimals(poolConstants.repayAsset);
            const interestDueTillInstalmentDeadline = yield (yield this.repayments.getInterestDueTillInstalmentDeadline(pool)).toString();
            return new bignumber_js_1.BigNumber(interestDueTillInstalmentDeadline).div(new bignumber_js_1.BigNumber(10).pow(decimal)).toFixed(2);
        });
    }
    getNextInstalmentDeadline(pool) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield (yield this.repayments.getNextInstalmentDeadline(pool)).toString();
        });
    }
    getCurrentInstalmentInterval(pool) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield (yield this.repayments.getCurrentInstalmentInterval(pool)).toString();
        });
    }
    getCurrentLoanInterval(pool) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield (yield this.repayments.getCurrentLoanInterval(pool)).toString();
        });
    }
    isGracePenaltyApplicable(pool) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.repayments.isGracePenaltyApplicable(pool);
        });
    }
    didBorrowerDefault(pool) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.repayments.didBorrowerDefault(pool);
        });
    }
    getInterestLeft(pool) {
        return __awaiter(this, void 0, void 0, function* () {
            const poolConstants = yield this.repayments.repayInfo(pool);
            yield this.tokenManager.updateTokenDecimals(poolConstants.repayAsset);
            const decimal = this.tokenManager.getTokenDecimals(poolConstants.repayAsset);
            const interestLeft = yield (yield this.repayments.getInterestLeft(pool)).toString();
            return new bignumber_js_1.BigNumber(interestLeft).div(new bignumber_js_1.BigNumber(10).pow(decimal)).toFixed(2);
        });
    }
    getInterestOverdue(pool) {
        return __awaiter(this, void 0, void 0, function* () {
            const poolConstants = yield this.repayments.repayInfo(pool);
            yield this.tokenManager.updateTokenDecimals(poolConstants.repayAsset);
            const decimal = this.tokenManager.getTokenDecimals(poolConstants.repayAsset);
            const interestOverdue = yield (yield this.repayments.getInterestOverdue(pool)).toString();
            return new bignumber_js_1.BigNumber(interestOverdue).div(new bignumber_js_1.BigNumber(10).pow(decimal)).toFixed(2);
        });
    }
    repayAmount(pool, amount, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const poolConstants = yield this.repayments.repayInfo(pool);
            yield this.tokenManager.updateTokenDecimals(poolConstants.repayAsset);
            const decimal = this.tokenManager.getTokenDecimals(poolConstants.repayAsset);
            const _amount = new bignumber_js_1.BigNumber(amount);
            if (_amount.isNaN() || _amount.isZero() || _amount.isNegative()) {
                throw new Error('amount should be a valid number');
            }
            return this.repayments.repay(pool, _amount.multipliedBy(new bignumber_js_1.BigNumber(10).pow(decimal)).toFixed(0), Object.assign({}, options));
        });
    }
    repayPrincipal(pool, options) {
        return __awaiter(this, void 0, void 0, function* () {
            return this.repayments.repayPrincipal(pool, Object.assign({}, options));
        });
    }
    getTotalRepaidAmount(pool) {
        return __awaiter(this, void 0, void 0, function* () {
            const poolConstants = yield this.repayments.repayInfo(pool);
            yield this.tokenManager.updateTokenDecimals(poolConstants.repayAsset);
            const decimal = this.tokenManager.getTokenDecimals(poolConstants.repayAsset);
            const totalRepaidAmount = yield (yield this.repayments.getTotalRepaidAmount(pool)).toString();
            return new bignumber_js_1.BigNumber(totalRepaidAmount).div(new bignumber_js_1.BigNumber(10).pow(decimal)).toFixed(2);
        });
    }
    getGracePeriodFraction(pool) {
        return __awaiter(this, void 0, void 0, function* () {
            const graceFraction = yield (yield this.repayments.getGracePeriodFraction()).toString();
            return new bignumber_js_1.BigNumber(graceFraction).div(new bignumber_js_1.BigNumber(10).pow(28)).toFixed(2);
        });
    }
}
exports.RepaymentApi = RepaymentApi;
//# sourceMappingURL=repayments.js.map