import { BigNumberish, ContractTransaction, Signer } from 'ethers';
import { TokenManager } from '../tokenManager';
import { Options as Overrides } from '../types/Types';
/**
 * @class TokenApi
 */
export declare class TokenApi {
    /**
     * @description Signer Object
     */
    private signer;
    /**
     * @description Contract instance for the given token
     */
    private tokenContract;
    private minterTokenContract;
    /**
     * @description Instance to update and fetch token metadata
     */
    private tokenManager;
    /**
     * @description if tokenAddress is any invalid token address, the APIs of the class object will fail
     * @param signer
     * @param tokenAddress
     * @param tokenManager
     */
    constructor(signer: Signer, tokenAddress: string, tokenManager: TokenManager);
    /**
     * @description Increase Allowance to a another address
     * @param to
     * @param amount
     * @returns Contract Transaction
     */
    IncreaseAllowance(to: string, amount: string, options?: Overrides): Promise<ContractTransaction>;
    /**
     * @description Approve Allowance to a another address
     * @param to
     * @param amount
     * @returns Contract Transaction
     */
    approve(to: string, amount: string, options?: Overrides): Promise<ContractTransaction>;
    /**
     * @description Returns the allowance of spender by owner
     * @param owner
     * @param spender
     * @param prettified, if true, the result will be in easy readable form
     * @returns number
     */
    allowance(owner: string, spender: string, prettified?: boolean): Promise<string>;
    /**
     * @description return the current signers token balance.
     * @param prettified
     * @returns
     */
    getTokenBalance(prettified?: boolean): Promise<string>;
    /**
     * @description return any user token balance
     * @param user
     * @param prettified
     * @returns
     */
    getUserTokenBalance(user: string, prettified?: boolean): Promise<string>;
    /**
     *
     * @param user address whose balance needs to be fetched
     * @param prettified
     * @returns Balance of the given address
     */
    private _getBalance;
    mintTokens(addressToReceive: string, amount: BigNumberish, options?: Overrides): Promise<ContractTransaction>;
}
