import { SublimeConfig } from '../../types/sublimeConfig';
import { ContractTransaction, Signer, BigNumberish } from 'ethers';
import { TokenManager } from '../../tokenManager';
import { StrategyType, Options as Overrides } from '../../types/Types';
/**
 * @description For handling credit line operations that use ETH
 */
export declare class CreditLineEthUtils {
    private creditLineUtils;
    private tokenManager;
    private config;
    constructor(signer: Signer, config: SublimeConfig, tokenManager: TokenManager);
    /**
     * @description convert ETH to WETH and deposit to credit line
     * @param creditLineNumber credit line ID
     * @param amount amount of ETH to be deposited as collateral
     * @param strategy strategy to which the collateral must be deposited
     * @param options transaction parameters
     * @returns Contract Transaction
     */
    depositEthAsCollateralToTheCreditLine(creditLineNumber: BigNumberish, amount: string, strategy: StrategyType, options?: Overrides): Promise<ContractTransaction>;
    /**
     * @description Convert ETH to WETH and repay to credit line
     * @param creditLineNumber id of the credit line
     * @param amount amount of ETH that needs to be repaid to the credit line
     * @param options transaction parameters
     * @returns Contract Transaction
     */
    repayEthToCreditLine(creditLineNumber: BigNumberish, amount: string, options?: Overrides): Promise<ContractTransaction>;
}
