"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CreditLineEthUtils = void 0;
const bignumber_js_1 = require("bignumber.js");
const CreditLineEthUtils__factory_1 = require("../../wrappers/factories/CreditLineEthUtils__factory");
const Types_1 = require("../../types/Types");
/**
 * @description For handling credit line operations that use ETH
 */
class CreditLineEthUtils {
    constructor(signer, config, tokenManager) {
        if (config.creditLineEthHelperAddress) {
            this.creditLineUtils = new CreditLineEthUtils__factory_1.CreditLineEthUtils__factory(signer).attach(config.creditLineEthHelperAddress);
            this.config = config;
            this.tokenManager = tokenManager;
        }
        else {
            throw new Error('creditLineEthHelperAddress is not defined in the sublime config');
        }
    }
    /**
     * @description convert ETH to WETH and deposit to credit line
     * @param creditLineNumber credit line ID
     * @param amount amount of ETH to be deposited as collateral
     * @param strategy strategy to which the collateral must be deposited
     * @param options transaction parameters
     * @returns Contract Transaction
     */
    depositEthAsCollateralToTheCreditLine(creditLineNumber, amount, strategy, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const decimals = 18;
            let strategyAddress;
            if (strategy == Types_1.StrategyType.NoYield) {
                strategyAddress = this.config.noStrategyAddress;
            }
            else if (strategy == Types_1.StrategyType.CompoundYield) {
                strategyAddress = this.config.compoundStrategyContractAddress;
            }
            else {
                strategyAddress = this.config.yearnStrategyContractAddress;
            }
            const _amount = new bignumber_js_1.BigNumber(amount);
            const value = _amount.multipliedBy(new bignumber_js_1.BigNumber(10).pow(decimals)).toFixed(0);
            return this.creditLineUtils.depositEthAsCollateralToCreditLine(creditLineNumber, Object.assign(Object.assign({}, options), { value }));
        });
    }
    /**
     * @description Convert ETH to WETH and repay to credit line
     * @param creditLineNumber id of the credit line
     * @param amount amount of ETH that needs to be repaid to the credit line
     * @param options transaction parameters
     * @returns Contract Transaction
     */
    repayEthToCreditLine(creditLineNumber, amount, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const decimals = 18;
            const _amount = new bignumber_js_1.BigNumber(amount);
            const value = _amount.multipliedBy(new bignumber_js_1.BigNumber(10).pow(decimals)).toFixed(0);
            return this.creditLineUtils.repayEthToCreditLines(creditLineNumber, Object.assign(Object.assign({}, options), { value }));
        });
    }
}
exports.CreditLineEthUtils = CreditLineEthUtils;
//# sourceMappingURL=creditLineEthUtils.js.map