import { SublimeConfig } from '../../types/sublimeConfig';
import { ContractTransaction, Signer } from 'ethers';
import { TokenManager } from '../../tokenManager';
import { Options as Overrides, StrategyType } from '../../types/Types';
export declare class PoolEthUtils {
    private poolUtils;
    private tokenManager;
    private config;
    constructor(signer: Signer, config: SublimeConfig, tokenManager: TokenManager);
    /**
     * @description Use this function call to deposit ETH directly to a pool.
     * @description The function call will automatically convert ETH to WETH and deposit to the pool
     * @param poolAddress Address of the pool
     * @param amount Amount of ETH to deposit
     * @param options transaction options
     * @returns
     */
    depositEthAsCollateralToPool(poolAddress: string, amount: string, options?: Overrides): Promise<ContractTransaction>;
    /**
     * @description Use this function to all margin call to a lender address when eth is the collateral
     * @param poolAddress Address of the pool
     * @param amount Amount of ETH to deposit
     * @param options transaction options
     * @returns
     */
    addEthCollateralInMarginCall(poolAddress: string, lender: string, amount: string, options?: Overrides): Promise<ContractTransaction>;
    /**
     * @description Use this function to lend ETH to the pool
     * @param poolAddress Address of the pool
     * @param lender Address deposit on behalf of (pass your own address if you want to deposit on behalf of your own address)
     * @param amount Amount of ETH to deposit
     * @param strategy Strategy to which the collateral is deposit
     * @param options transaction options
     * @returns
     */
    ethLend(poolAddress: string, lender: string, amount: string, strategy: StrategyType, options?: Overrides): Promise<ContractTransaction>;
}
