"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PoolEthUtils = void 0;
const bignumber_js_1 = require("bignumber.js");
const PoolEthUtils__factory_1 = require("../../wrappers/factories/PoolEthUtils__factory");
const Types_1 = require("../../types/Types");
class PoolEthUtils {
    constructor(signer, config, tokenManager) {
        if (config.poolUtilsAddress) {
            this.poolUtils = new PoolEthUtils__factory_1.PoolEthUtils__factory(signer).attach(config.poolUtilsAddress);
            this.tokenManager = tokenManager;
            this.config = config;
        }
        else {
            throw new Error('poolUtilsAddress is not defined in the sublime config');
        }
    }
    /**
     * @description Use this function call to deposit ETH directly to a pool.
     * @description The function call will automatically convert ETH to WETH and deposit to the pool
     * @param poolAddress Address of the pool
     * @param amount Amount of ETH to deposit
     * @param options transaction options
     * @returns
     */
    depositEthAsCollateralToPool(poolAddress, amount, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const decimals = 18;
            const _amount = new bignumber_js_1.BigNumber(amount);
            const value = _amount.multipliedBy(new bignumber_js_1.BigNumber(10).pow(decimals)).toFixed(0);
            return this.poolUtils.depositEthAsCollateralToPool(poolAddress, Object.assign(Object.assign({}, options), { value }));
        });
    }
    /**
     * @description Use this function to all margin call to a lender address when eth is the collateral
     * @param poolAddress Address of the pool
     * @param amount Amount of ETH to deposit
     * @param options transaction options
     * @returns
     */
    addEthCollateralInMarginCall(poolAddress, lender, amount, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const decimals = 18;
            const _amount = new bignumber_js_1.BigNumber(amount);
            const value = _amount.multipliedBy(new bignumber_js_1.BigNumber(10).pow(decimals)).toFixed(0);
            return this.poolUtils.addEthCollateralInMarginCall(poolAddress, lender, Object.assign(Object.assign({}, options), { value }));
        });
    }
    /**
     * @description Use this function to lend ETH to the pool
     * @param poolAddress Address of the pool
     * @param lender Address deposit on behalf of (pass your own address if you want to deposit on behalf of your own address)
     * @param amount Amount of ETH to deposit
     * @param strategy Strategy to which the collateral is deposit
     * @param options transaction options
     * @returns
     */
    ethLend(poolAddress, lender, amount, strategy, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const decimals = 18;
            const _amount = new bignumber_js_1.BigNumber(amount);
            const value = _amount.multipliedBy(new bignumber_js_1.BigNumber(10).pow(decimals)).toFixed(0);
            let strategyAddress;
            if (strategy == Types_1.StrategyType.NoYield) {
                strategyAddress = this.config.noStrategyAddress;
            }
            else if (strategy == Types_1.StrategyType.CompoundYield) {
                strategyAddress = this.config.compoundStrategyContractAddress;
            }
            else {
                strategyAddress = this.config.yearnStrategyContractAddress;
            }
            return this.poolUtils.ethLend(poolAddress, lender, strategyAddress, Object.assign(Object.assign({}, options), { value }));
        });
    }
}
exports.PoolEthUtils = PoolEthUtils;
//# sourceMappingURL=poolEthUtils.js.map