"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SavingAccountsEthUtils = void 0;
const bignumber_js_1 = require("bignumber.js");
const SavingsAccountEthUtils__factory_1 = require("../../wrappers/factories/SavingsAccountEthUtils__factory");
const Types_1 = require("../../types/Types");
class SavingAccountsEthUtils {
    /**
     * @param signer
     * @param config
     * @param tokenManager
     */
    constructor(signer, config, tokenManager) {
        if (config.savingsAccountEthUtilsAddress) {
            this.savingsAccountEthUtils = new SavingsAccountEthUtils__factory_1.SavingsAccountEthUtils__factory(signer).attach(config.savingsAccountEthUtilsAddress);
            this.tokenManager = tokenManager;
            this.config = config;
        }
        else {
            throw new Error('savingsAccountEthUtilsAddress is not defined in the sublime config');
        }
    }
    /**
     * @description Use this function to deposit ETH to savings account directly
     * @param strategy strategy
     * @param amount amount of ETH to deposit
     * @param options transaction options
     * @returns
     */
    depositEthToSavingsAccount(strategy, amount, to, options) {
        return __awaiter(this, void 0, void 0, function* () {
            const decimals = 18;
            const _amount = new bignumber_js_1.BigNumber(amount);
            const value = _amount.multipliedBy(new bignumber_js_1.BigNumber(10).pow(decimals)).toFixed(0);
            let strategyAddress;
            if (strategy == Types_1.StrategyType.NoYield) {
                strategyAddress = this.config.noStrategyAddress;
            }
            else if (strategy == Types_1.StrategyType.CompoundYield) {
                strategyAddress = this.config.compoundStrategyContractAddress;
            }
            else {
                strategyAddress = this.config.yearnStrategyContractAddress;
            }
            return this.savingsAccountEthUtils.depositEth(strategyAddress, to, Object.assign(Object.assign({}, options), { value }));
        });
    }
}
exports.SavingAccountsEthUtils = SavingAccountsEthUtils;
//# sourceMappingURL=savingsAccountEthUtils.js.map