import { Signer, ContractTransaction, BigNumberish } from 'ethers';
import { BigNumber } from 'bignumber.js';
import { SublimeConfig } from '../types/sublimeConfig';
import { Strategy, StrategyType, Options as Overrides } from '../types/Types';
import { TokenManager } from '../tokenManager';
/**
 * @class YieldAndStrategyApi
 */
export declare class YieldAndStrategyApi {
    /**
     * @description Signer Object
     */
    private signer;
    private compoundYield;
    /**
     * @description Internal store for all sublime contracts addresses
     */
    private config;
    /**
     * @description Object to update and fetch token meta data
     */
    private tokenManager;
    /**
     * @description Display names of all strategies
     */
    private displayName;
    /**
     * @description Logo for all strategies
     */
    private logo;
    constructor(signer: Signer, config: SublimeConfig, tokenManger: TokenManager);
    /**
     * @param yieldType Type of yield/strategy
     * @param asset Address of the asset/token
     * @param shares Number of shares
     * @param sharesInLowestUnits If true, token decimals for shares are ignored
     * @returns Total number of underlying tokens for a given asset and strategy
     */
    getTokensForShares(yieldType: StrategyType, asset: string, shares: BigNumber): Promise<BigNumber>;
    /**
     * @param yieldType Type of yield/strategy
     * @param asset Address of the asset/token
     * @param amount Number of tokens
     * @returns Number of shares that will be generated for given number of tokens and strategy
     */
    getSharesForTokens(yieldType: StrategyType, asset: string, amount: string): Promise<BigNumber>;
    getApr(yieldType: StrategyType, asset: string): Promise<BigNumber>;
    /**
     * @description returns all strategies supported sublime-sdk
     * @returns
     */
    getStrategies(): Strategy[];
    /**
     * @description Get Strategy address from name
     * @param strategy
     * @returns null if invalid strategy name is given
     */
    getStrategyAddress(strategy: StrategyType): string | undefined;
    /**
     * @description Get Strategy name from given address,
     * @param address
     * @returns null if invalid strategy address is given in input
     */
    getStrategy(address: string): StrategyType | undefined;
    /**
     * @param address yield/strategy address
     * @returns URL of the strategy logo
     */
    getStrategyLogo(address: string): string | undefined;
    /**
     * @param address yield/strategy address
     * @returns Display Name/Info for the strategy
     */
    getStrategyDisplayName(address: string): string | undefined;
    updateLiquidityTokenInCompoundYield(asset: string, cToken: string, options?: Overrides): Promise<ContractTransaction>;
    setDepositLimitInCompoundYield(asset: string, limit: BigNumberish, options?: Overrides): Promise<ContractTransaction>;
}
