"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EmulatedCreditLines = void 0;
const queries_1 = require("../queries");
const CreditLines_1 = require("../emulator/CreditLines");
const tokenManager_1 = require("../tokenManager");
const bignumber_js_1 = __importDefault(require("bignumber.js"));
const yieldAndStrategy_1 = require("../api/yieldAndStrategy");
const Helper_1 = require("./Helper");
const Types_1 = require("types/Types");
class EmulatedCreditLines extends Helper_1.EmulatedHelper {
    constructor(subgraphUrl, priceSubgraphUrl, signer, config, tokenManager) {
        let _tokenManager;
        let yieldApi;
        yieldApi = new yieldAndStrategy_1.YieldAndStrategyApi(signer, config, tokenManager);
        if (tokenManager) {
            _tokenManager = tokenManager;
        }
        else {
            _tokenManager = new tokenManager_1.TokenManager(signer, priceSubgraphUrl);
        }
        super(_tokenManager, yieldApi);
        this.subgraphUrl = subgraphUrl;
    }
    getAllCreditLines(count = 13, skip = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getAllCreditLinesFromSubgraph)(this.subgraphUrl, count, skip);
            const prices = yield this.refreshTokenData(result);
            const collateralPerStrategyToken = yield this.refreshStrategyData(result);
            return this.transformToCreditLineEmulator(result, prices, collateralPerStrategyToken);
        });
    }
    convertToCreditLineStatusEnum(stateInSubgraph) {
        // enum CreditLineStatus {
        //     NOT_CREATED,
        //     REQUESTED,
        //     ACTIVE
        // }
        if (stateInSubgraph == 'NOT_CREATED') {
            return Types_1.CreditLineStatus.NOT_CREATED;
        }
        else if (stateInSubgraph == 'REQUESTED') {
            return Types_1.CreditLineStatus.REQUESTED;
        }
        else if (stateInSubgraph == 'ACTIVE') {
            return Types_1.CreditLineStatus.ACTIVE;
        }
        else {
            return Types_1.CreditLineStatus.NOT_CREATED;
        }
    }
    transformToCreditLineEmulator(data, prices, collateralPerStrategyToken) {
        return data.map((a) => {
            return new CreditLines_1.CreditLineEmulator({
                id: a.id,
                collateralShareInStrategy: new bignumber_js_1.default(1000),
                principal: new bignumber_js_1.default(a.principal),
                lastPrincipalUpdateTime: new bignumber_js_1.default(a.lastPrincipalUpdateTime),
                borrowRate: new bignumber_js_1.default(a.borrowRate),
                interestAccruedTillLastPrincipalUpdate: new bignumber_js_1.default(a.interestAccruedTillLastPrincipalUpdate),
                totalInterestRepaid: new bignumber_js_1.default(a.totalInterestRepaid),
                idealCollateralRatio: new bignumber_js_1.default(a.idealCollateralRatio),
                creditLineStatus: this.convertToCreditLineStatusEnum(a.status),
                borrowLimit: new bignumber_js_1.default(a.borrowLimit),
                createdAt: new bignumber_js_1.default(a.createdAt),
                borrowAsset: a.borrowAsset,
                collateralAsset: a.collateralAsset,
                autoLiquidation: new Boolean(a.autoLiquidation).valueOf(),
                lender: a.lender,
                borrower: a.borrower,
                requestByLender: new Boolean(a.requestByLender).valueOf(),
                borrowAssetStrategy: a.borrowAssetStrategy,
                collateralAssetStrategy: a.collateralAssetStrategy,
            }, {
                collateralPerStrategyToken: new bignumber_js_1.default(collateralPerStrategyToken[a.collateralAssetStrategy][a.collateralAsset]),
                ratioOfPrices: new bignumber_js_1.default(prices[a.borrowAsset]).dividedBy(prices[a.collateralAsset]),
                ratioOfPricesDecimals: 1,
            }, { liquidatorRewardFraction: new bignumber_js_1.default(10).pow(16) });
        });
    }
}
exports.EmulatedCreditLines = EmulatedCreditLines;
// id: '52',
// status: 'REQUESTED',
// lender: '0x808ba745926e6cdef924c9bc0c758ec110474596',
// strategy: '0xf469c04fb301ab0dc5ba425e11645ad6b8c0be64',
// totalInterestRepaid: '0',
// lenderWalletDetails: { wallet: [Object] },
// borrower: '0x3a574461fd1279fcf96043bcf416c53b7e8dcec0',
// borrowerWalletDetails: { wallet: [Object] },
// requestByLender: false,
// principal: '0',
// collateralAsset: '0xe22da380ee6b445bb8273c81944adeb6e8450422',
// borrowLimit: '111000000000000000000',
// borrowRate: '10000000000000000',
// idealCollateralRatio: '10000000000000000',
// borrowAsset: '0x4f96fe3b7a6cf9725f59d353f723c1bdb64ca6aa',
// autoLiquidation: true,
// lastPrincipalUpdateTime: '0',
// createdAt: '1653654940'
//# sourceMappingURL=EmulatedCreditLine.js.map