"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EmulatedPooledCreditLines = void 0;
const tokenManager_1 = require("../tokenManager");
const yieldAndStrategy_1 = require("../api/yieldAndStrategy");
const Helper_1 = require("./Helper");
const queries_1 = require("../queries");
const PooledCreditLines_1 = require("../emulator/PooledCreditLines");
const bignumber_js_1 = __importDefault(require("bignumber.js"));
const Types_1 = require("../types/Types");
class EmulatedPooledCreditLines extends Helper_1.EmulatedHelper {
    constructor(subgraphUrl, priceSubgraphUrl, signer, config, tokenManager) {
        let _tokenManager;
        let yieldApi;
        yieldApi = new yieldAndStrategy_1.YieldAndStrategyApi(signer, config, tokenManager);
        if (tokenManager) {
            _tokenManager = tokenManager;
        }
        else {
            _tokenManager = new tokenManager_1.TokenManager(signer, priceSubgraphUrl);
        }
        super(_tokenManager, yieldApi);
        this.subgraphUrl = subgraphUrl;
    }
    getAllPooledCreditLines(count = 13, skip = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            const lpData = yield (0, queries_1.getPCLandLpTogether)(this.subgraphUrl, count, skip);
            const pclData = lpData.map((a) => a.pooledCreditLine);
            const prices = yield this.refreshTokenData(pclData);
            const collateralPerStrategyToken = yield this.refreshStrategyDataForPcl(pclData);
            const lendersPerPoolData = yield (0, queries_1.getAllLendersPerPool)(this.subgraphUrl, pclData.map((a) => a.id));
            const allPoolIds = pclData.map((a) => a.id);
            const lenderPerPoolData = [];
            for (let index = 0; index < allPoolIds.length; index++) {
                const poolId = allPoolIds[index];
                const requiredLendersPerPool = lendersPerPoolData.filter((a) => a.lenderPool.id == poolId);
                const requiredLenderPerPool = requiredLendersPerPool.map((a) => {
                    return {
                        lenderAddress: a.lenderAddress,
                        amountLent: new bignumber_js_1.default(a.amountLent),
                        amountWithdrawn: new bignumber_js_1.default(a.amountWithdrawn),
                        sharesWithdrawn: new bignumber_js_1.default(a.sharesWithdrawn),
                        interestWithdrawn: new bignumber_js_1.default(a.interestWithdrawn),
                        lenderBalance: new bignumber_js_1.default(a.lenderBalance),
                        borrowerInterestSharesWithdrawn: new bignumber_js_1.default(a.borrowerInterestSharesWithdrawn),
                        yieldInterestWithdrawnShares: new bignumber_js_1.default(a.yieldInterestWithdrawnShares),
                    };
                });
                lenderPerPoolData.push(requiredLenderPerPool);
            }
            return this.transformToPooledCreditLineEmulator(pclData, lpData, lenderPerPoolData, prices, collateralPerStrategyToken);
        });
    }
    convertToPooledCreditLineStatusEnum(stateInSubgraph) {
        //   enum PooledCreditLineStatus {
        //     NOT_CREATED,
        //     REQUESTED,
        //     ACTIVE,
        //     CLOSED,
        //     EXPIRED,
        //     LIQUIDATED,
        //     CANCELLED
        // }
        if (stateInSubgraph == 'NOT_CREATED') {
            return Types_1.CreditLineStatus.NOT_CREATED;
        }
        else if (stateInSubgraph == 'REQUESTED') {
            return Types_1.CreditLineStatus.REQUESTED;
        }
        else if (stateInSubgraph == 'ACTIVE') {
            return Types_1.CreditLineStatus.ACTIVE;
        }
        else if (stateInSubgraph == 'CLOSED') {
            return Types_1.CreditLineStatus.CLOSED;
        }
        else if (stateInSubgraph == 'EXPIRED') {
            return Types_1.CreditLineStatus.EXPIRED;
        }
        else if (stateInSubgraph == 'LIQUIDATED') {
            return Types_1.CreditLineStatus.LIQUIDATED;
        }
        else if (stateInSubgraph == 'CANCELLED') {
            return Types_1.CreditLineStatus.CANCELLED;
        }
        else {
            return Types_1.CreditLineStatus.NOT_CREATED;
        }
    }
    transformToPooledCreditLineEmulator(pclData, lenderPoolData, lendersPerPool, prices, collateralPerStrategyToken) {
        return pclData.map((a, index) => {
            const aLenderPool = lenderPoolData[index];
            const lenderPerPool = lendersPerPool[index];
            return new PooledCreditLines_1.PooledCreditLineEmulator({
                id: a.id,
                pooledCreditLineStatus: this.convertToPooledCreditLineStatusEnum(a.status),
                endsAt: new bignumber_js_1.default(a.endsAt),
                principal: new bignumber_js_1.default(a.principal),
                idealCollateralRatio: new bignumber_js_1.default(a.idealCollateralRatio),
                totalInterestRepaid: new bignumber_js_1.default(a.totalInterestRepaid),
                lastPrincipalUpdateTime: new bignumber_js_1.default(a.lastPrincipalUpdateTime),
                gracePenaltyRate: new bignumber_js_1.default(a.gracePenaltyRate),
                borrowRate: new bignumber_js_1.default(a.borrowRate),
                interestAccruedTillLastPrincipalUpdate: new bignumber_js_1.default(a.interestAccruedTillLastPrincipalUpdate),
                depositedCollateralInShares: new bignumber_js_1.default(a.depositedCollateralInShares),
                borrowLimit: new bignumber_js_1.default(a.borrowLimit),
                defaultsAt: new bignumber_js_1.default(a.defaultsAt),
                borrowerAddress: a.borrowerAddress,
                borrowAsset: a.borrowAsset,
                collateralAsset: a.collateralAsset,
                createdAt: new bignumber_js_1.default(a.createdAt),
                startsAt: new bignumber_js_1.default(a.startsAt),
                lenderStrategy: a.lenderStrategy,
                collateralStrategy: a.collateralStrategy,
                totalLentAmount: new bignumber_js_1.default(a.totalLentAmount),
                minBorrowAmount: new bignumber_js_1.default(a.minBorrowAmount),
                lenderVerifier: a.lenderVerifier.id,
                borrowerVerifier: a.borrowerVerifier.id,
            }, {
                collateralPerStrategyToken: new bignumber_js_1.default(collateralPerStrategyToken[a.lenderStrategy][a.collateralAsset]),
                ratioOfPrices: new bignumber_js_1.default(prices[a.borrowAsset]).dividedBy(prices[a.collateralAsset]),
                ratioOfPricesDecimals: 1,
            }, {
                id: aLenderPool.id,
                sharesHeld: new bignumber_js_1.default(aLenderPool.sharesHeld),
                borrowerInterestShares: new bignumber_js_1.default(aLenderPool.borrowerInterestShares),
                borrowerInterestSharesWithdrawn: new bignumber_js_1.default(aLenderPool.borrowerInterestShares),
                yieldInterestWithdrawnShares: new bignumber_js_1.default(aLenderPool.yieldInterestWithdrawnShares),
                borrowLimit: new bignumber_js_1.default(aLenderPool.borrowLimit),
                startTime: new bignumber_js_1.default(aLenderPool.startTime),
                minBorrowAmount: new bignumber_js_1.default(aLenderPool.minBorrowAmount),
            }, lenderPerPool);
        });
    }
}
exports.EmulatedPooledCreditLines = EmulatedPooledCreditLines;
// {
//     id: '123',
//     borrowerAddress: '0x808ba745926e6cdef924c9bc0c758ec110474596',
//     borrowLimit: '100000000000000000000',
//     borrowRate: '120000000000000000',
//     idealCollateralRatio: '400000000000000000',
//     borrowAsset: '0x4f96fe3b7a6cf9725f59d353f723c1bdb64ca6aa',
//     collateralAsset: '0xe22da380ee6b445bb8273c81944adeb6e8450422',
//     createdAt: '1653581648',
//     startsAt: '1654013648',
//     endsAt: '1661789648',
//     defaultsAt: '1661962448',
//     lenderStrategy: '0x924625da987a8f08f9ff3c370a23a1a1e41920cd',
//     collateralStrategy: '0x924625da987a8f08f9ff3c370a23a1a1e41920cd',
//     gracePenaltyRate: '120000000000000000',
//     status: 'CANCELLED',
//     principal: '0',
//     totalInterestRepaid: '0',
//     lastPrincipalUpdateTime: '0',
//     interestAccruedTillLastPrincipalUpdate: '0',
//     totalLentAmount: '0'
//   }
//# sourceMappingURL=EmulatedPooledCreditLine.js.map