"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EmulatedHelper = void 0;
const bignumber_js_1 = __importDefault(require("bignumber.js"));
const Types_1 = require("../types/Types");
class EmulatedHelper {
    constructor(tokenManager, yieldApi) {
        this.tokenManager = tokenManager;
        this.yieldApi = yieldApi;
    }
    refreshTokenData(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const borrowTokens = data.map((a) => a.collateralAsset);
            const collateralTokens = data.map((a) => a.borrowAsset);
            const allTokens = [...borrowTokens, ...collateralTokens].filter((value, index, array) => array.indexOf(value) === index);
            const prices = {};
            for (let index = 0; index < allTokens.length; index++) {
                const element = allTokens[index];
                yield this.tokenManager.updateAll(element);
                prices[element] = yield this.tokenManager.getPricePerAsset(element);
            }
            return prices;
        });
    }
    refreshStrategyDataForPcl(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const allStrategyAndTokenPairs = data.map((a) => {
                return { strategy: a.lenderStrategy, collateralAsset: a.collateralAsset, id: `${a.lenderStrategy}-${a.collateralAsset}` };
            });
            const allIds = allStrategyAndTokenPairs.map((a) => a.id).filter((value, index, self) => self.indexOf(value) === index);
            const uniquePairs = allIds.map((a) => {
                return { strategy: a.split('-')[0], collateralAsset: a.split('-')[1] };
            });
            const collateralPerStrategyToken = {};
            const valueToCheck = new bignumber_js_1.default(10).pow(18);
            for (let index = 0; index < uniquePairs.length; index++) {
                const element = uniquePairs[index];
                const strategy = this.yieldApi.getStrategy(element.strategy);
                const tokensForShares = yield this.getTokensForShares(strategy, element.collateralAsset, valueToCheck);
                if (!collateralPerStrategyToken[element.strategy]) {
                    collateralPerStrategyToken[element.strategy] = {};
                }
                collateralPerStrategyToken[element.strategy][element.collateralAsset] = tokensForShares.div(valueToCheck);
            }
            return collateralPerStrategyToken;
        });
    }
    refreshStrategyData(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const allStrategyAndTokenPairs = data.map((a) => {
                return { strategy: a.strategy, collateralAsset: a.collateralAsset, id: `${a.strategy}-${a.collateralAsset}` };
            });
            const allIds = allStrategyAndTokenPairs.map((a) => a.id).filter((value, index, self) => self.indexOf(value) === index);
            const uniquePairs = allIds.map((a) => {
                return { strategy: a.split('-')[0], collateralAsset: a.split('-')[1] };
            });
            const collateralPerStrategyToken = {};
            const valueToCheck = new bignumber_js_1.default(10).pow(18);
            for (let index = 0; index < uniquePairs.length; index++) {
                const element = uniquePairs[index];
                const strategy = this.yieldApi.getStrategy(element.strategy);
                const tokensForShares = yield this.getTokensForShares(strategy, element.collateralAsset, valueToCheck);
                if (!collateralPerStrategyToken[element.strategy]) {
                    collateralPerStrategyToken[element.strategy] = {};
                }
                collateralPerStrategyToken[element.strategy][element.collateralAsset] = tokensForShares.div(valueToCheck);
            }
            return collateralPerStrategyToken;
        });
    }
    getTokensForShares(strategy, collateralAsset, amount) {
        return __awaiter(this, void 0, void 0, function* () {
            if (strategy == Types_1.StrategyType.NoYield)
                return new bignumber_js_1.default(amount);
            try {
                return yield this.yieldApi.getTokensForShares(strategy, collateralAsset, amount);
            }
            catch (ex) {
                return new bignumber_js_1.default(0);
            }
        });
    }
    getRandomInt(max) {
        return new bignumber_js_1.default(Math.floor(Math.random() * max));
    }
}
exports.EmulatedHelper = EmulatedHelper;
//# sourceMappingURL=Helper.js.map