"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAllCreditLinesForCount = exports.getCreditLinesOfLenderWithStateNotIn_requestByLender = exports.getCreditLinesOfBorrowerWithStateNotIn_requestByLender = exports.getCreditLinesOfLenderWithState_requestByLender = exports.getCreditLinesOfBorrowerWithState_requestByLender = exports.getCreditLinesOfLenderWithStateNotIn = exports.getCreditLinesOfBorrowerWithStateNotIn = exports.getCreditLinesOfLenderWithState = exports.getCreditLinesOfBorrowerWithState = exports.getPendingCreditLinesRequestedToBorrower = exports.getPendingCreditLinesRequestedByBorrower = exports.getPendingCreditLinesRequestedToLender = exports.getPendingCreditlinesRequestedByLender = exports.getConfirmedCreditLinesOfLender = exports.getConfirmedCreditLinesOfBorrower = exports.getCreditLinesLenderCanLenderTo = exports.getCreditLine = exports.getAllCreditLinesFromSubgraph = exports.getCreditLineTimeline = void 0;
const helpers_1 = require("../helpers");
function getCreditLineTimeline(url, creditLineNumber) {
    return __awaiter(this, void 0, void 0, function* () {
        const data = JSON.stringify({
            query: `{
      creditLines(where:{id:"${creditLineNumber}"}) {
        id
        borrowAsset
        createdAt
        collateralAsset
        borrowAssetStrategy
        collateralAssetStrategy
        totalInterestRepaid
        collateralShareInStrategy
        creditLineTimeline(orderBy:timestamp, orderDirection:desc) {
          id
          timestamp
          creditLineOperation
          amount
          strategy
          liquidator
        }
      }
    }
    `,
        });
        const options = {
            url,
            headers: { 'Content-Type': 'application/json' },
            body: data,
        };
        const result = yield (0, helpers_1.fetchData)(options);
        // console.log(result);
        return result;
    });
}
exports.getCreditLineTimeline = getCreditLineTimeline;
function getAllCreditLinesFromSubgraph(url, count, skip) {
    return __awaiter(this, void 0, void 0, function* () {
        const allData = [];
        const data = JSON.stringify({
            query: `{
        creditLines (orderBy:createdAt, orderDirection:desc, first:${count}, skip:${skip}){
          id
          status
          lender
          borrowAssetStrategy
          collateralAssetStrategy
          totalInterestRepaid
          collateralShareInStrategy
          lenderWalletDetails {
            wallet {
              user {
                userMetadataPerVerifier {
                  metadata
                }
              }
            }
          }
          borrower
          borrowerWalletDetails {
            wallet {
              user {
                userMetadataPerVerifier {
                  metadata
                }
              }
            }
          }
          requestByLender
          principal
          collateralAsset
          borrowLimit
          borrowRate
          idealCollateralRatio
          borrowAsset
          autoLiquidation
          lastPrincipalUpdateTime
          createdAt
          interestAccruedTillLastPrincipalUpdate
        }
    }`,
        });
        const options = {
            url,
            headers: { 'Content-Type': 'application/json' },
            body: data,
        };
        const result = yield (0, helpers_1.fetchData)(options);
        //   console.log(result);
        allData.push(...result.data.creditLines);
        return allData;
    });
}
exports.getAllCreditLinesFromSubgraph = getAllCreditLinesFromSubgraph;
function _getCreditLinesOfBorrower(url, status, requestByLender, borrower, count, skip) {
    return __awaiter(this, void 0, void 0, function* () {
        borrower = borrower.toLowerCase();
        const allData = [];
        const data = JSON.stringify({
            query: `{
        creditLines (orderBy:createdAt, orderDirection:desc, first:${count}, skip:${skip},where:{borrower:"${borrower}",requestByLender_in:[${requestByLender}], status_in:[${status}]}){
          id
          status
          lender
          borrowAssetStrategy
          collateralAssetStrategy
          totalInterestRepaid
          collateralShareInStrategy
          lenderWalletDetails {
            wallet {
              user {
                userMetadataPerVerifier {
                  metadata
                }
              }
            }
          }
          borrower
          borrowerWalletDetails {
            wallet {
              user {
                userMetadataPerVerifier {
                  metadata
                }
              }
            }
          }
          requestByLender
          principal
          collateralAsset
          borrowLimit
          borrowRate
          idealCollateralRatio
          borrowAsset
          autoLiquidation
          lastPrincipalUpdateTime
          createdAt
          interestAccruedTillLastPrincipalUpdate
        }
    }`,
        });
        const options = {
            url,
            headers: { 'Content-Type': 'application/json' },
            body: data,
        };
        const result = yield (0, helpers_1.fetchData)(options);
        //   console.log(result);
        allData.push(...result.data.creditLines);
        return allData;
    });
}
function getCreditLine(url, id) {
    return __awaiter(this, void 0, void 0, function* () {
        const allData = [];
        const data = JSON.stringify({
            query: `{
          creditLines (where:{id:"${id}"}){
            id
            status
            lender
            borrowAssetStrategy
            collateralAssetStrategy
            totalInterestRepaid
            collateralShareInStrategy
            lenderWalletDetails {
              wallet {
                user {
                  userMetadataPerVerifier {
                    metadata
                  }
                }
              }
            }
            borrower
            borrowerWalletDetails {
              wallet {
                user {
                  userMetadataPerVerifier {
                    metadata
                  }
                }
              }
            }
            requestByLender
            principal
            collateralAsset
            borrowLimit
            borrowRate
            idealCollateralRatio
            borrowAsset
            autoLiquidation
            lastPrincipalUpdateTime
            createdAt
            interestAccruedTillLastPrincipalUpdate
          }
      }`,
        });
        const options = {
            url,
            headers: { 'Content-Type': 'application/json' },
            body: data,
        };
        const result = yield (0, helpers_1.fetchData)(options);
        // console.log(result);
        allData.push(...result.data.creditLines);
        return allData;
    });
}
exports.getCreditLine = getCreditLine;
function _getCreditLinesOfLender(url, status, requestByLender, lender, count, skip) {
    return __awaiter(this, void 0, void 0, function* () {
        lender = lender.toLowerCase();
        const allData = [];
        const data = JSON.stringify({
            query: `{
          creditLines (orderBy:createdAt, orderDirection:desc, first:${count}, skip:${skip},where:{lender:"${lender}",requestByLender_in:[${requestByLender}], status_in:[${status}]}){
            id
            status
            lender
            borrowAssetStrategy
            collateralAssetStrategy
            totalInterestRepaid
            collateralShareInStrategy
            lenderWalletDetails {
              wallet {
                user {
                  userMetadataPerVerifier {
                    metadata
                  }
                }
              }
            }
            borrower
            borrowerWalletDetails {
              wallet {
                user {
                  userMetadataPerVerifier {
                    metadata
                  }
                }
              }
            }
            requestByLender
            principal
            collateralAsset
            borrowLimit
            borrowRate
            idealCollateralRatio
            borrowAsset
            autoLiquidation
            lastPrincipalUpdateTime
            createdAt
            interestAccruedTillLastPrincipalUpdate
          }
      }`,
        });
        const options = {
            url,
            headers: { 'Content-Type': 'application/json' },
            body: data,
        };
        const result = yield (0, helpers_1.fetchData)(options);
        // console.log(result);
        allData.push(...result.data.creditLines);
        return allData;
    });
}
function _getCreditLinesNotOfLender(url, status, requestByLender, lender, count, skip) {
    return __awaiter(this, void 0, void 0, function* () {
        lender = lender.toLowerCase();
        const allData = [];
        const data = JSON.stringify({
            query: `{
          creditLines (orderBy:createdAt, orderDirection:desc, first:${count}, skip:${skip},where:{borrower_not:"${lender}",requestByLender_in:[${requestByLender}], status_in:[${status}]}){
            id
            status
            lender
            borrowAssetStrategy
            collateralAssetStrategy
            totalInterestRepaid
            collateralShareInStrategy
            lenderWalletDetails {
              wallet {
                user {
                  userMetadataPerVerifier {
                    metadata
                  }
                }
              }
            }
            borrower
            borrowerWalletDetails {
              wallet {
                user {
                  userMetadataPerVerifier {
                    metadata
                  }
                }
              }
            }
            requestByLender
            principal
            collateralAsset
            borrowLimit
            borrowRate
            idealCollateralRatio
            borrowAsset
            autoLiquidation
            lastPrincipalUpdateTime
            createdAt
            interestAccruedTillLastPrincipalUpdate
          }
      }`,
        });
        const options = {
            url,
            headers: { 'Content-Type': 'application/json' },
            body: data,
        };
        const result = yield (0, helpers_1.fetchData)(options);
        // console.log(result);
        allData.push(...result.data.creditLines);
        return allData;
    });
}
function getCreditLinesLenderCanLenderTo(url, lender, count, skip) {
    return __awaiter(this, void 0, void 0, function* () {
        let data = [];
        const status = ['ACTIVE'];
        const requestByLender = [true, false];
        const lines = yield _getCreditLinesNotOfLender(url, status, requestByLender, lender, count, skip);
        data = [...lines];
        return data;
    });
}
exports.getCreditLinesLenderCanLenderTo = getCreditLinesLenderCanLenderTo;
// creditLineTypes ACTIVE, CLOSED, LIQUIDATED, NOT_CREATED, REQUESTED, CANCELLED
function getConfirmedCreditLinesOfBorrower(url, borrower, count, skip) {
    return __awaiter(this, void 0, void 0, function* () {
        let data = [];
        const status = ['ACTIVE', 'CLOSED', 'LIQUIDATED', 'NOT_CREATED'];
        const requestByLender = [true, false];
        const lines = yield _getCreditLinesOfBorrower(url, status, requestByLender, borrower, count, skip);
        data = [...lines];
        return data;
    });
}
exports.getConfirmedCreditLinesOfBorrower = getConfirmedCreditLinesOfBorrower;
function getConfirmedCreditLinesOfLender(url, lender, count, skip) {
    return __awaiter(this, void 0, void 0, function* () {
        let data = [];
        const status = ['ACTIVE', 'CLOSED', 'LIQUIDATED', 'NOT_CREATED'];
        const requestByLender = [true, false];
        const lines = yield _getCreditLinesOfLender(url, status, requestByLender, lender, count, skip);
        data = [...lines];
        return data;
    });
}
exports.getConfirmedCreditLinesOfLender = getConfirmedCreditLinesOfLender;
function getPendingCreditlinesRequestedByLender(url, lender, count, skip) {
    return __awaiter(this, void 0, void 0, function* () {
        let data = [];
        const status = ['REQUESTED'];
        const requestByLender = [true];
        const lines = yield _getCreditLinesOfLender(url, status, requestByLender, lender, count, skip);
        data = [...lines];
        return data;
    });
}
exports.getPendingCreditlinesRequestedByLender = getPendingCreditlinesRequestedByLender;
function getPendingCreditLinesRequestedToLender(url, lender, count, skip) {
    return __awaiter(this, void 0, void 0, function* () {
        let data = [];
        const status = ['REQUESTED'];
        const requestByLender = [false];
        const lines = yield _getCreditLinesOfLender(url, status, requestByLender, lender, count, skip);
        data = [...lines];
        return data;
    });
}
exports.getPendingCreditLinesRequestedToLender = getPendingCreditLinesRequestedToLender;
function getPendingCreditLinesRequestedByBorrower(url, borrower, count, skip) {
    return __awaiter(this, void 0, void 0, function* () {
        let data = [];
        const status = ['REQUESTED'];
        const requestByLender = [false];
        const lines = yield _getCreditLinesOfBorrower(url, status, requestByLender, borrower, count, skip);
        data = [...lines];
        return data;
    });
}
exports.getPendingCreditLinesRequestedByBorrower = getPendingCreditLinesRequestedByBorrower;
function getPendingCreditLinesRequestedToBorrower(url, borrower, count, skip) {
    return __awaiter(this, void 0, void 0, function* () {
        let data = [];
        const status = ['REQUESTED'];
        const requestByLender = [true];
        const lines = yield _getCreditLinesOfBorrower(url, status, requestByLender, borrower, count, skip);
        data = [...lines];
        return data;
    });
}
exports.getPendingCreditLinesRequestedToBorrower = getPendingCreditLinesRequestedToBorrower;
function getCreditLinesOfBorrowerWithState(url, address, status) {
    return __awaiter(this, void 0, void 0, function* () {
        let skip = 0;
        address = address.toLowerCase();
        const allData = [];
        for (;;) {
            const data = JSON.stringify({
                query: `{
        creditLines(first: ${helpers_1.countPerQuery}, skip:${skip * helpers_1.countPerQuery}, orderBy: createdAt, orderDirection: desc, where:{borrower:"${address}",status_in:[${status}]}){
          id
          status
        }
      }`,
            });
            const options = {
                url,
                headers: { 'Content-Type': 'application/json' },
                body: data,
            };
            const result = yield (0, helpers_1.fetchData)(options);
            if (result.errors) {
                (0, helpers_1.print)(result.errors);
                throw new Error('Error while fetching data from subgraph');
            }
            else if (result.data.creditLines.length == 0) {
                return allData;
            }
            else {
                skip++;
                allData.push(...result.data.creditLines);
            }
        }
    });
}
exports.getCreditLinesOfBorrowerWithState = getCreditLinesOfBorrowerWithState;
function getCreditLinesOfLenderWithState(url, address, status) {
    return __awaiter(this, void 0, void 0, function* () {
        let skip = 0;
        address = address.toLowerCase();
        const allData = [];
        for (;;) {
            const data = JSON.stringify({
                query: `{
        creditLines(first: ${helpers_1.countPerQuery}, skip:${skip * helpers_1.countPerQuery}, orderBy: createdAt, orderDirection: desc, where:{lender:"${address}",status_in:[${status}]}){
          id
          status
        }
      }`,
            });
            const options = {
                url,
                headers: { 'Content-Type': 'application/json' },
                body: data,
            };
            const result = yield (0, helpers_1.fetchData)(options);
            if (result.errors) {
                (0, helpers_1.print)(result.errors);
                throw new Error('Error while fetching data from subgraph');
            }
            else if (result.data.creditLines.length == 0) {
                return allData;
            }
            else {
                skip++;
                allData.push(...result.data.creditLines);
            }
        }
    });
}
exports.getCreditLinesOfLenderWithState = getCreditLinesOfLenderWithState;
function getCreditLinesOfBorrowerWithStateNotIn(url, address, status) {
    return __awaiter(this, void 0, void 0, function* () {
        let skip = 0;
        address = address.toLowerCase();
        const allData = [];
        for (;;) {
            const data = JSON.stringify({
                query: `{
        creditLines(first: ${helpers_1.countPerQuery}, skip:${skip * helpers_1.countPerQuery}, orderBy: createdAt, orderDirection: desc, where:{borrower:"${address}",status_not_in:[${status}]}){
          id
          status
        }
      }`,
            });
            const options = {
                url,
                headers: { 'Content-Type': 'application/json' },
                body: data,
            };
            const result = yield (0, helpers_1.fetchData)(options);
            if (result.errors) {
                (0, helpers_1.print)(result.errors);
                throw new Error('Error while fetching data from subgraph');
            }
            else if (result.data.creditLines.length == 0) {
                return allData;
            }
            else {
                skip++;
                allData.push(...result.data.creditLines);
            }
        }
    });
}
exports.getCreditLinesOfBorrowerWithStateNotIn = getCreditLinesOfBorrowerWithStateNotIn;
function getCreditLinesOfLenderWithStateNotIn(url, address, status) {
    return __awaiter(this, void 0, void 0, function* () {
        let skip = 0;
        address = address.toLowerCase();
        const allData = [];
        for (;;) {
            const data = JSON.stringify({
                query: `{
        creditLines(first: ${helpers_1.countPerQuery}, skip:${skip * helpers_1.countPerQuery}, orderBy: createdAt, orderDirection: desc, where:{lender:"${address}",status_not_in:[${status}]}){
          id
          status
        }
      }`,
            });
            const options = {
                url,
                headers: { 'Content-Type': 'application/json' },
                body: data,
            };
            const result = yield (0, helpers_1.fetchData)(options);
            if (result.errors) {
                (0, helpers_1.print)(result.errors);
                throw new Error('Error while fetching data from subgraph');
            }
            else if (result.data.creditLines.length == 0) {
                return allData;
            }
            else {
                skip++;
                allData.push(...result.data.creditLines);
            }
        }
    });
}
exports.getCreditLinesOfLenderWithStateNotIn = getCreditLinesOfLenderWithStateNotIn;
// /----- request by lender used
function getCreditLinesOfBorrowerWithState_requestByLender(url, address, requestByLender, status) {
    return __awaiter(this, void 0, void 0, function* () {
        let skip = 0;
        address = address.toLowerCase();
        const allData = [];
        for (;;) {
            const data = JSON.stringify({
                query: `{
        creditLines(first: ${helpers_1.countPerQuery}, skip:${skip * helpers_1.countPerQuery}, orderBy: createdAt, orderDirection: desc, where:{requestByLender:${requestByLender}, borrower:"${address}",status_in:[${status}]}){
          id
          status
        }
      }`,
            });
            const options = {
                url,
                headers: { 'Content-Type': 'application/json' },
                body: data,
            };
            const result = yield (0, helpers_1.fetchData)(options);
            if (result.errors) {
                (0, helpers_1.print)(result.errors);
                throw new Error('Error while fetching data from subgraph');
            }
            else if (result.data.creditLines.length == 0) {
                return allData;
            }
            else {
                skip++;
                allData.push(...result.data.creditLines);
            }
        }
    });
}
exports.getCreditLinesOfBorrowerWithState_requestByLender = getCreditLinesOfBorrowerWithState_requestByLender;
function getCreditLinesOfLenderWithState_requestByLender(url, address, requestByLender, status) {
    return __awaiter(this, void 0, void 0, function* () {
        let skip = 0;
        address = address.toLowerCase();
        const allData = [];
        for (;;) {
            const data = JSON.stringify({
                query: `{
        creditLines(first: ${helpers_1.countPerQuery}, skip:${skip * helpers_1.countPerQuery}, orderBy: createdAt, orderDirection: desc, where:{requestByLender:${requestByLender}, lender:"${address}",status_in:[${status}]}){
          id
          status
        }
      }`,
            });
            const options = {
                url,
                headers: { 'Content-Type': 'application/json' },
                body: data,
            };
            const result = yield (0, helpers_1.fetchData)(options);
            if (result.errors) {
                (0, helpers_1.print)(result.errors);
                throw new Error('Error while fetching data from subgraph');
            }
            else if (result.data.creditLines.length == 0) {
                return allData;
            }
            else {
                skip++;
                allData.push(...result.data.creditLines);
            }
        }
    });
}
exports.getCreditLinesOfLenderWithState_requestByLender = getCreditLinesOfLenderWithState_requestByLender;
function getCreditLinesOfBorrowerWithStateNotIn_requestByLender(url, address, requestByLender, status) {
    return __awaiter(this, void 0, void 0, function* () {
        let skip = 0;
        address = address.toLowerCase();
        const allData = [];
        for (;;) {
            const data = JSON.stringify({
                query: `{
        creditLines(first: ${helpers_1.countPerQuery}, skip:${skip * helpers_1.countPerQuery}, orderBy: createdAt, orderDirection: desc, where:{requestByLender:${requestByLender}, borrower:"${address}",status_not_in:[${status}]}){
          id
          status
        }
      }`,
            });
            const options = {
                url,
                headers: { 'Content-Type': 'application/json' },
                body: data,
            };
            const result = yield (0, helpers_1.fetchData)(options);
            if (result.errors) {
                (0, helpers_1.print)(result.errors);
                throw new Error('Error while fetching data from subgraph');
            }
            else if (result.data.creditLines.length == 0) {
                return allData;
            }
            else {
                skip++;
                allData.push(...result.data.creditLines);
            }
        }
    });
}
exports.getCreditLinesOfBorrowerWithStateNotIn_requestByLender = getCreditLinesOfBorrowerWithStateNotIn_requestByLender;
function getCreditLinesOfLenderWithStateNotIn_requestByLender(url, address, requestByLender, status) {
    return __awaiter(this, void 0, void 0, function* () {
        let skip = 0;
        address = address.toLowerCase();
        const allData = [];
        for (;;) {
            const data = JSON.stringify({
                query: `{
        creditLines(first: ${helpers_1.countPerQuery}, skip:${skip * helpers_1.countPerQuery}, orderBy: createdAt, orderDirection: desc, where:{requestByLender:${requestByLender},lender:"${address}",status_not_in:[${status}]}){
          id
          status
        }
      }`,
            });
            const options = {
                url,
                headers: { 'Content-Type': 'application/json' },
                body: data,
            };
            const result = yield (0, helpers_1.fetchData)(options);
            if (result.errors) {
                (0, helpers_1.print)(result.errors);
                throw new Error('Error while fetching data from subgraph');
            }
            else if (result.data.creditLines.length == 0) {
                return allData;
            }
            else {
                skip++;
                allData.push(...result.data.creditLines);
            }
        }
    });
}
exports.getCreditLinesOfLenderWithStateNotIn_requestByLender = getCreditLinesOfLenderWithStateNotIn_requestByLender;
function getAllCreditLinesForCount(url) {
    return __awaiter(this, void 0, void 0, function* () {
        let skip = 0;
        const allData = [];
        for (;;) {
            const data = JSON.stringify({
                query: `{
        creditLines(first: ${helpers_1.countPerQuery}, skip:${skip * helpers_1.countPerQuery}, orderBy: createdAt, orderDirection: desc){
          id
        }
      }`,
            });
            const options = {
                url,
                headers: { 'Content-Type': 'application/json' },
                body: data,
            };
            const result = yield (0, helpers_1.fetchData)(options);
            if (result.errors) {
                (0, helpers_1.print)(result.errors);
                throw new Error('Error while fetching data from subgraph');
            }
            else if (result.data.creditLines.length == 0) {
                return allData;
            }
            else {
                skip++;
                allData.push(...result.data.creditLines);
            }
        }
    });
}
exports.getAllCreditLinesForCount = getAllCreditLinesForCount;
//# sourceMappingURL=getAllCreditLines.js.map