"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAllPoolsByBorrowerByType = exports.getAllPoolsByLenderByType = exports.getAllPoolsByLender = exports.getAllPoolsByBorrower = exports.getAllPoolsByPoolType = exports.getAllPools = void 0;
const helpers_1 = require("../helpers");
function getAllPools(url) {
    return __awaiter(this, void 0, void 0, function* () {
        let skip = 0;
        const allData = [];
        for (;;) {
            const data = JSON.stringify({
                query: `{
        pools(first: ${helpers_1.countPerQuery}, skip:${skip * helpers_1.countPerQuery}) {
          id,
          borrowRate,
          amountBorrowed,
          borrowAmountRequested,
          nextPaymentDeadline,
          borrowAsset,
          collateralAsset,
          poolStatus
        }
      }`,
            });
            const options = {
                url,
                headers: { 'Content-Type': 'application/json' },
                body: data,
            };
            const result = yield (0, helpers_1.fetchData)(options);
            if (result.errors) {
                (0, helpers_1.print)(result.errors);
                throw new Error('Error while fetching data from subgraph');
            }
            else if (result.data.pools.length == 0) {
                return allData;
            }
            else {
                skip++;
                allData.push(...result.data.pools);
            }
        }
    });
}
exports.getAllPools = getAllPools;
function getAllPoolsByPoolType(url, poolType) {
    return __awaiter(this, void 0, void 0, function* () {
        let skip = 0;
        const allData = [];
        for (;;) {
            const data = JSON.stringify({
                query: `{
        pools(first: ${helpers_1.countPerQuery}, skip:${skip * helpers_1.countPerQuery}, where:{poolStatus:"${poolType}"}) {
          id,
          borrowRate,
          amountBorrowed,
          borrowAmountRequested,
          nextPaymentDeadline,
          borrowAsset,
          collateralAsset,
          poolStatus
        }
      }`,
            });
            const options = {
                url,
                headers: { 'Content-Type': 'application/json' },
                body: data,
            };
            const result = yield (0, helpers_1.fetchData)(options);
            if (result.errors) {
                (0, helpers_1.print)(result.errors);
                throw new Error('Error while fetching data from subgraph');
            }
            else if (result.data.pools.length == 0) {
                return allData;
            }
            else {
                skip++;
                allData.push(...result.data.pools);
            }
        }
    });
}
exports.getAllPoolsByPoolType = getAllPoolsByPoolType;
function getAllPoolsByBorrower(url, borrower) {
    return __awaiter(this, void 0, void 0, function* () {
        borrower = borrower.toLowerCase();
        let skip = 0;
        const allData = [];
        for (;;) {
            const data = JSON.stringify({
                query: `{
        users(where:{id:"${borrower}"}) {
          borrowingPools (first: ${helpers_1.countPerQuery}, skip:${skip * helpers_1.countPerQuery}) {
            id,
            borrowRate,
            amountBorrowed,
            borrowAmountRequested,
            nextPaymentDeadline,
            borrowAsset,
            collateralAsset,
            poolStatus
          }
        }
      }`,
            });
            const options = {
                url,
                headers: { 'Content-Type': 'application/json' },
                body: data,
            };
            const result = yield (0, helpers_1.fetchData)(options);
            if (result.errors) {
                (0, helpers_1.print)(result.errors);
                throw new Error('Error while fetching data from subgraph');
            }
            else if (result.data.users.length == 0) {
                return allData;
            }
            else if (result.data.users.length > 0 && result.data.users[0].borrowingPools && result.data.users[0].borrowingPools.length == 0) {
                return allData;
            }
            else {
                skip++;
                allData.push(...result.data.users[0].borrowingPools);
            }
        }
    });
}
exports.getAllPoolsByBorrower = getAllPoolsByBorrower;
function getAllPoolsByLender(url, lender) {
    return __awaiter(this, void 0, void 0, function* () {
        lender = lender.toLowerCase();
        let skip = 0;
        const allData = [];
        for (;;) {
            const data = JSON.stringify({
                query: `{
        users(where:{id:"${lender}"}) {
          lendingPools (first: ${helpers_1.countPerQuery}, skip:${skip * helpers_1.countPerQuery}) {
            id,
            pool {
              id,
              borrowRate,
              amountBorrowed,
              borrowAmountRequested,
              nextPaymentDeadline,
              borrowAsset,
              collateralAsset,
              poolStatus
            }
          }
        }
      }`,
            });
            const options = {
                url,
                headers: { 'Content-Type': 'application/json' },
                body: data,
            };
            const result = yield (0, helpers_1.fetchData)(options);
            if (result.errors) {
                (0, helpers_1.print)(result.errors);
                throw new Error('Error while fetching data from subgraph');
            }
            else if (result.data.users.length == 0) {
                return allData;
            }
            else if (result.data.users.length > 0 && result.data.users[0].lendingPools && result.data.users[0].lendingPools.length == 0) {
                return allData;
            }
            else {
                skip++;
                for (let index = 0; index < result.data.users[0].lendingPools.length; index++) {
                    allData.push(result.data.users[0].lendingPools[index].pool);
                }
            }
        }
    });
}
exports.getAllPoolsByLender = getAllPoolsByLender;
function getAllPoolsByLenderByType(url, lender, poolType) {
    return __awaiter(this, void 0, void 0, function* () {
        lender = lender.toLowerCase();
        let skip = 0;
        const allData = [];
        for (;;) {
            const data = JSON.stringify({
                query: `{
        users(where: {id:"${lender}"}) {
          lendingPools (first: ${helpers_1.countPerQuery}, skip:${skip * helpers_1.countPerQuery}) {
            id,
            pool {
              id,
              borrowRate,
              amountBorrowed,
              borrowAmountRequested,
              nextPaymentDeadline,
              borrowAsset,
              collateralAsset,
              poolStatus
            }
          }
        }
      }`,
            });
            const options = {
                url,
                headers: { 'Content-Type': 'application/json' },
                body: data,
            };
            const result = yield (0, helpers_1.fetchData)(options);
            if (result.errors) {
                (0, helpers_1.print)(result.errors);
                throw new Error('Error while fetching data from subgraph');
            }
            else if (result.data.users.length == 0) {
                return allData.filter((a) => a.poolStatus === poolType);
            }
            else if (result.data.users.length > 0 && result.data.users[0].lendingPools && result.data.users[0].lendingPools.length == 0) {
                return allData.filter((a) => a.poolStatus === poolType);
            }
            else {
                skip++;
                for (let index = 0; index < result.data.users[0].lendingPools.length; index++) {
                    allData.push(result.data.users[0].lendingPools[index].pool);
                }
            }
        }
    });
}
exports.getAllPoolsByLenderByType = getAllPoolsByLenderByType;
function getAllPoolsByBorrowerByType(url, borrower, poolType) {
    return __awaiter(this, void 0, void 0, function* () {
        borrower = borrower.toLowerCase();
        let skip = 0;
        const allData = [];
        for (;;) {
            const data = JSON.stringify({
                query: `{
        users(where:{id:"${borrower}"}) {
          borrowingPools (first: ${helpers_1.countPerQuery}, skip:${skip * helpers_1.countPerQuery}, where:{poolStatus:"${poolType}"}) {
            id,
            borrowRate,
            amountBorrowed,
            borrowAmountRequested,
            nextPaymentDeadline,
            borrowAsset,
            collateralAsset,
            poolStatus
          }
        }
      }`,
            });
            const options = {
                url,
                headers: { 'Content-Type': 'application/json' },
                body: data,
            };
            const result = yield (0, helpers_1.fetchData)(options);
            if (result.errors) {
                (0, helpers_1.print)(result.errors);
                throw new Error('Error while fetching data from subgraph');
            }
            else if (result.data.users.length == 0) {
                return allData.filter((a) => a.poolStatus === poolType);
            }
            else if (result.data.users.length > 0 && result.data.users[0].borrowingPools && result.data.users[0].borrowingPools.length == 0) {
                return allData.filter((a) => a.poolStatus === poolType);
            }
            else {
                skip++;
                allData.push(...result.data.users[0].borrowingPools);
            }
        }
    });
}
exports.getAllPoolsByBorrowerByType = getAllPoolsByBorrowerByType;
//# sourceMappingURL=getAllPools.js.map