"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAllLendersPerPool = exports.getPCLandLpTogether = exports.getLendersOfPooledCreditLines = exports.getAllPooledCreditLinesOfBorrower = exports.getAllPooledCreditLinesOfBorrowerWithStateNotIn = exports.getAllPooledCreditLinesOfBorrowerWithState = exports.getAllPooledCreditLinesOfLenderWithStateNotIn = exports.getAllPooledCreditLinesOfLender = exports.getAllPooledCreditLinesOfLenderWithState = exports.getAllPooledCreditLinesForCountWithStateNotIn = exports.getAllPooledCreditLinesForCountWithState = exports.getAllPooledCreditLinesForCount = exports.getLenderPerPool = exports.getAllPooledCreditLinesWithNotState = exports.getAllPooledCreditLinesWithState = exports.getAllPooledCreditLines = exports.getPooledCreditLineById = exports.getPooledCreditLinesOfLenderCanLendTo = exports.getPooledCreditLinesOfBorrowerWithNotState = exports.getPooledCreditLinesOfBorrowerWithState = exports.getPooledCreditLinesOfBorrower = exports.getPooledCreditLinesForLenderById = exports.getPooledCreditLinesOfLender = exports.getPooledCreditLineTimeline = void 0;
const helpers_1 = require("../helpers");
function getPooledCreditLineTimeline(url, pooledCreditLineId, count, skip) {
    return __awaiter(this, void 0, void 0, function* () {
        const allData = [];
        const data = JSON.stringify({
            query: `{
        pooledCreditLineTimeLines(first:${count}, skip:${skip}, orderBy: timestamp, orderDirection: desc, where:{pooledCreditLine:"${pooledCreditLineId}"}){
        id
        pooledCreditLineOperation
        timestamp
        amount
        strategy
      }
    }`,
        });
        const options = {
            url,
            headers: { 'Content-Type': 'application/json' },
            body: data,
        };
        const result = yield (0, helpers_1.fetchData)(options);
        // print({result});
        allData.push(...result.data.pooledCreditLineTimeLines);
        // console.log({allData})
        return allData;
    });
}
exports.getPooledCreditLineTimeline = getPooledCreditLineTimeline;
function getPooledCreditLinesOfLender(url, lenderAddress, count, skip) {
    return __awaiter(this, void 0, void 0, function* () {
        lenderAddress = lenderAddress.toLowerCase();
        const allData = [];
        const poolContributions = [];
        const data = JSON.stringify({
            query: `{
        lenderPerLenderPools(first:${count}, skip:${skip}, where:{lenderAddress:"${lenderAddress}"}){
          lenderAddress
          amountLent
          amountWithdrawn
          sharesWithdrawn
          interestWithdrawn
          lenderPool{
            pooledCreditLine{
              id
              status
              endsAt
              principal
              idealCollateralRatio
              totalInterestRepaid
              lastPrincipalUpdateTime
              gracePenaltyRate
              borrowRate
              interestAccruedTillLastPrincipalUpdate
              borrowLimit
              borrowAsset
              collateralAsset
              lenderStrategy
              depositedCollateralInShares
              borrowerAddress
              createdAt
              startsAt
              defaultsAt
              collateralStrategy
              totalLentAmount
              status
              minBorrowAmount
              lenderVerifier {
                id
              }
              borrowerVerifier {
                id
              }
              lenderPool{
                id
                sharesHeld
                borrowerInterestShares
                borrowerInterestSharesWithdrawn
                yieldInterestWithdrawnShares
                borrowLimit
                startTime
                minBorrowAmount
              }
            }
          }
        }
      }`,
        });
        const options = {
            url,
            headers: { 'Content-Type': 'application/json' },
            body: data,
        };
        const result = yield (0, helpers_1.fetchData)(options);
        // print({result});
        allData.push(...result.data.lenderPerLenderPools.map((a) => a.lenderPool.pooledCreditLine));
        poolContributions.push(...result.data.lenderPerLenderPools.map((a) => {
            return {
                amountLent: a.amountLent,
                amountWithdrawn: a.amountWithdrawn,
                sharesWithdrawn: a.sharesWithdrawn,
                interestWithdrawn: a.interestWithdrawn,
            };
        }));
        // console.log({allData})
        return [allData, poolContributions];
    });
}
exports.getPooledCreditLinesOfLender = getPooledCreditLinesOfLender;
function getPooledCreditLinesForLenderById(url, lenderAddress, id) {
    return __awaiter(this, void 0, void 0, function* () {
        lenderAddress = lenderAddress.toLowerCase();
        id = id.toLowerCase();
        const allData = [];
        const poolContributions = [];
        const data = JSON.stringify({
            query: `{
        lenderPerLenderPools(where:{lenderPool:"${id}", lenderAddress:"${lenderAddress}"}){
          lenderAddress
          amountLent
          amountWithdrawn
          sharesWithdrawn
          interestWithdrawn
          lenderPool{
            pooledCreditLine{
              id
              status
              endsAt
              principal
              idealCollateralRatio
              totalInterestRepaid
              lastPrincipalUpdateTime
              gracePenaltyRate
              borrowRate
              interestAccruedTillLastPrincipalUpdate
              borrowLimit
              borrowAsset
              collateralAsset
              lenderStrategy
              depositedCollateralInShares
              borrowerAddress
              createdAt
              startsAt
              defaultsAt
              collateralStrategy
              totalLentAmount
              status
              minBorrowAmount
              lenderVerifier {
                id
              }
              borrowerVerifier {
                id
              }
              lenderPool{
                id
                sharesHeld
                borrowerInterestShares
                borrowerInterestSharesWithdrawn
                yieldInterestWithdrawnShares
                borrowLimit
                startTime
                minBorrowAmount
              }
            }
          }
        }
      }`,
        });
        const options = {
            url,
            headers: { 'Content-Type': 'application/json' },
            body: data,
        };
        const result = yield (0, helpers_1.fetchData)(options);
        // print({result});
        allData.push(...result.data.lenderPerLenderPools.map((a) => a.lenderPool.pooledCreditLine));
        poolContributions.push(...result.data.lenderPerLenderPools.map((a) => {
            return {
                amountLent: a.amountLent,
                amountWithdrawn: a.amountWithdrawn,
                sharesWithdrawn: a.sharesWithdrawn,
                interestWithdrawn: a.interestWithdrawn,
            };
        }));
        // console.log({allData})
        return [allData, poolContributions];
    });
}
exports.getPooledCreditLinesForLenderById = getPooledCreditLinesForLenderById;
function getPooledCreditLinesOfBorrower(url, borrower, count, skip) {
    return __awaiter(this, void 0, void 0, function* () {
        borrower = borrower.toLowerCase();
        const allData = [];
        const data = JSON.stringify({
            query: `{
        pooledCreditLines(first:${count}, skip:${skip}, orderBy:createdAt, orderDirection:desc, where:{borrowerAddress:"${borrower}"}){
          id
          status
          endsAt
          principal
          idealCollateralRatio
          totalInterestRepaid
          lastPrincipalUpdateTime
          gracePenaltyRate
          borrowRate
          interestAccruedTillLastPrincipalUpdate
          borrowLimit
          borrowAsset
          collateralAsset
          lenderStrategy
          depositedCollateralInShares
          borrowerAddress
          createdAt
          startsAt
          defaultsAt
          collateralStrategy
          totalLentAmount
          status
          minBorrowAmount
          lenderVerifier {
            id
          }
          borrowerVerifier {
            id
          }
          lenderPool{
            id
            sharesHeld
            borrowerInterestShares
            borrowerInterestSharesWithdrawn
            yieldInterestWithdrawnShares
            borrowLimit
            startTime
            minBorrowAmount
          }
        }
      }`,
        });
        const options = {
            url,
            headers: { 'Content-Type': 'application/json' },
            body: data,
        };
        const result = yield (0, helpers_1.fetchData)(options);
        // print({result});
        allData.push(...result.data.pooledCreditLines);
        // console.log({allData})
        return allData;
    });
}
exports.getPooledCreditLinesOfBorrower = getPooledCreditLinesOfBorrower;
function getPooledCreditLinesOfBorrowerWithState(url, borrower, count, skip, status) {
    return __awaiter(this, void 0, void 0, function* () {
        borrower = borrower.toLowerCase();
        const allData = [];
        const data = JSON.stringify({
            query: `{
        pooledCreditLines(first:${count}, skip:${skip}, orderBy:createdAt, orderDirection:desc, where:{borrowerAddress:"${borrower}",status_in:[${status}]}){
          id
          status
          endsAt
          principal
          idealCollateralRatio
          totalInterestRepaid
          lastPrincipalUpdateTime
          gracePenaltyRate
          borrowRate
          interestAccruedTillLastPrincipalUpdate
          borrowLimit
          borrowAsset
          collateralAsset
          lenderStrategy
          depositedCollateralInShares
          borrowerAddress
          createdAt
          startsAt
          defaultsAt
          collateralStrategy
          totalLentAmount
          status
          minBorrowAmount
          lenderVerifier {
            id
          }
          borrowerVerifier {
            id
          }
          lenderPool{
            id
            sharesHeld
            borrowerInterestShares
            borrowerInterestSharesWithdrawn
            yieldInterestWithdrawnShares
            borrowLimit
            startTime
            minBorrowAmount
          }
        }
      }`,
        });
        const options = {
            url,
            headers: { 'Content-Type': 'application/json' },
            body: data,
        };
        const result = yield (0, helpers_1.fetchData)(options);
        allData.push(...result.data.pooledCreditLines);
        // console.log({allData})
        return allData;
    });
}
exports.getPooledCreditLinesOfBorrowerWithState = getPooledCreditLinesOfBorrowerWithState;
function getPooledCreditLinesOfBorrowerWithNotState(url, borrower, count, skip, status) {
    return __awaiter(this, void 0, void 0, function* () {
        borrower = borrower.toLowerCase();
        const allData = [];
        const data = JSON.stringify({
            query: `{
        pooledCreditLines(first:${count}, skip:${skip}, orderBy:createdAt, orderDirection:desc, where:{borrowerAddress:"${borrower}",status_not_in:[${status}]}){
          id
          status
          endsAt
          principal
          idealCollateralRatio
          totalInterestRepaid
          lastPrincipalUpdateTime
          gracePenaltyRate
          borrowRate
          interestAccruedTillLastPrincipalUpdate
          borrowLimit
          borrowAsset
          collateralAsset
          lenderStrategy
          depositedCollateralInShares
          borrowerAddress
          createdAt
          startsAt
          defaultsAt
          collateralStrategy
          totalLentAmount
          status
          minBorrowAmount
          lenderVerifier {
            id
          }
          borrowerVerifier {
            id
          }
          lenderPool{
            id
            sharesHeld
            borrowerInterestShares
            borrowerInterestSharesWithdrawn
            yieldInterestWithdrawnShares
            borrowLimit
            startTime
            minBorrowAmount
          }
        }
      }`,
        });
        const options = {
            url,
            headers: { 'Content-Type': 'application/json' },
            body: data,
        };
        const result = yield (0, helpers_1.fetchData)(options);
        // print({result});
        allData.push(...result.data.pooledCreditLines);
        // console.log({allData})
        return allData;
    });
}
exports.getPooledCreditLinesOfBorrowerWithNotState = getPooledCreditLinesOfBorrowerWithNotState;
function getPooledCreditLinesOfLenderCanLendTo(url, lender, count, skip) {
    return __awaiter(this, void 0, void 0, function* () {
        lender = lender.toLowerCase();
        const allData = [];
        const data = JSON.stringify({
            query: `{
        pooledCreditLines(first:${count}, skip:${skip}, orderBy:createdAt, orderDirection:desc, where:{borrowerAddress_not:"${lender}"}){
          id
          status
          endsAt
          principal
          idealCollateralRatio
          totalInterestRepaid
          lastPrincipalUpdateTime
          gracePenaltyRate
          borrowRate
          interestAccruedTillLastPrincipalUpdate
          borrowLimit
          borrowAsset
          collateralAsset
          lenderStrategy
          depositedCollateralInShares
          borrowerAddress
          createdAt
          startsAt
          defaultsAt
          collateralStrategy
          totalLentAmount
          status
          minBorrowAmount
          lenderVerifier {
            id
          }
          borrowerVerifier {
            id
          }
          lenderPool{
            id
            sharesHeld
            borrowerInterestShares
            borrowerInterestSharesWithdrawn
            yieldInterestWithdrawnShares
            borrowLimit
            startTime
            minBorrowAmount
          }
        }
      }`,
        });
        const options = {
            url,
            headers: { 'Content-Type': 'application/json' },
            body: data,
        };
        const result = yield (0, helpers_1.fetchData)(options);
        // print({result});
        allData.push(...result.data.pooledCreditLines);
        // console.log({allData})
        return allData;
    });
}
exports.getPooledCreditLinesOfLenderCanLendTo = getPooledCreditLinesOfLenderCanLendTo;
function getPooledCreditLineById(url, id) {
    return __awaiter(this, void 0, void 0, function* () {
        const allData = [];
        const data = JSON.stringify({
            query: `{
        pooledCreditLines(where:{id:"${id}"}){
          id
          status
          endsAt
          principal
          idealCollateralRatio
          totalInterestRepaid
          lastPrincipalUpdateTime
          gracePenaltyRate
          borrowRate
          interestAccruedTillLastPrincipalUpdate
          borrowLimit
          borrowAsset
          collateralAsset
          lenderStrategy
          depositedCollateralInShares
          borrowerAddress
          createdAt
          startsAt
          defaultsAt
          collateralStrategy
          totalLentAmount
          status
          minBorrowAmount
          lenderVerifier {
            id
          }
          borrowerVerifier {
            id
          }
          lenderPool{
            id
            sharesHeld
            borrowerInterestShares
            borrowerInterestSharesWithdrawn
            yieldInterestWithdrawnShares
            borrowLimit
            startTime
            minBorrowAmount
          }
        }
      }`,
        });
        const options = {
            url,
            headers: { 'Content-Type': 'application/json' },
            body: data,
        };
        const result = yield (0, helpers_1.fetchData)(options);
        // print({result});
        allData.push(...result.data.pooledCreditLines);
        // console.log({allData})
        return allData;
    });
}
exports.getPooledCreditLineById = getPooledCreditLineById;
function getAllPooledCreditLines(url, count, skip) {
    return __awaiter(this, void 0, void 0, function* () {
        const allData = [];
        const data = JSON.stringify({
            query: `{
        pooledCreditLines(orderBy:createdAt, orderDirection:desc, first:${count}, skip:${skip}){
          id
          status
          endsAt
          principal
          idealCollateralRatio
          totalInterestRepaid
          lastPrincipalUpdateTime
          gracePenaltyRate
          borrowRate
          interestAccruedTillLastPrincipalUpdate
          borrowLimit
          borrowAsset
          collateralAsset
          lenderStrategy
          depositedCollateralInShares
          borrowerAddress
          createdAt
          startsAt
          defaultsAt
          collateralStrategy
          totalLentAmount
          status
          minBorrowAmount
          lenderVerifier {
            id
          }
          borrowerVerifier {
            id
          }
          lenderPool{
            id
            sharesHeld
            borrowerInterestShares
            borrowerInterestSharesWithdrawn
            yieldInterestWithdrawnShares
            borrowLimit
            startTime
            minBorrowAmount
          }
        }
      }`,
        });
        const options = {
            url,
            headers: { 'Content-Type': 'application/json' },
            body: data,
        };
        const result = yield (0, helpers_1.fetchData)(options);
        // print({result});
        allData.push(...result.data.pooledCreditLines);
        // console.log({allData})
        return allData;
    });
}
exports.getAllPooledCreditLines = getAllPooledCreditLines;
function getAllPooledCreditLinesWithState(url, count, skip, status) {
    return __awaiter(this, void 0, void 0, function* () {
        const allData = [];
        const data = JSON.stringify({
            query: `{
        pooledCreditLines(orderBy:createdAt, orderDirection:desc, first:${count}, skip:${skip}, where:{status_in:[${status}]}){
          id
          status
          endsAt
          principal
          idealCollateralRatio
          totalInterestRepaid
          lastPrincipalUpdateTime
          gracePenaltyRate
          borrowRate
          interestAccruedTillLastPrincipalUpdate
          borrowLimit
          borrowAsset
          collateralAsset
          lenderStrategy
          depositedCollateralInShares
          borrowerAddress
          createdAt
          startsAt
          defaultsAt
          collateralStrategy
          totalLentAmount
          status
          minBorrowAmount
          lenderVerifier {
            id
          }
          borrowerVerifier {
            id
          }
          lenderPool{
            id
            sharesHeld
            borrowerInterestShares
            borrowerInterestSharesWithdrawn
            yieldInterestWithdrawnShares
            borrowLimit
            startTime
            minBorrowAmount
          }
        }
      }`,
        });
        const options = {
            url,
            headers: { 'Content-Type': 'application/json' },
            body: data,
        };
        const result = yield (0, helpers_1.fetchData)(options);
        // print({result});
        allData.push(...result.data.pooledCreditLines);
        // console.log({allData})
        return allData;
    });
}
exports.getAllPooledCreditLinesWithState = getAllPooledCreditLinesWithState;
function getAllPooledCreditLinesWithNotState(url, count, skip, status) {
    return __awaiter(this, void 0, void 0, function* () {
        const allData = [];
        const data = JSON.stringify({
            query: `{
        pooledCreditLines(orderBy:createdAt, orderDirection:desc, first:${count}, skip:${skip}, where:{status_not_in:[${status}]}){
          id
          status
          endsAt
          principal
          idealCollateralRatio
          totalInterestRepaid
          lastPrincipalUpdateTime
          gracePenaltyRate
          borrowRate
          interestAccruedTillLastPrincipalUpdate
          borrowLimit
          borrowAsset
          collateralAsset
          lenderStrategy
          depositedCollateralInShares
          borrowerAddress
          createdAt
          startsAt
          defaultsAt
          collateralStrategy
          totalLentAmount
          status
          minBorrowAmount
          lenderVerifier {
            id
          }
          borrowerVerifier {
            id
          }
          lenderPool{
            id
            sharesHeld
            borrowerInterestShares
            borrowerInterestSharesWithdrawn
            yieldInterestWithdrawnShares
            borrowLimit
            startTime
            minBorrowAmount
          }
        }
      }`,
        });
        const options = {
            url,
            headers: { 'Content-Type': 'application/json' },
            body: data,
        };
        const result = yield (0, helpers_1.fetchData)(options);
        // print({result});
        allData.push(...result.data.pooledCreditLines);
        // console.log({allData})
        return allData;
    });
}
exports.getAllPooledCreditLinesWithNotState = getAllPooledCreditLinesWithNotState;
function getLenderPerPool(url, id) {
    return __awaiter(this, void 0, void 0, function* () {
        const allData = [];
        const data = JSON.stringify({
            query: `{
      lenderPools(where:{id:"${id}"}){
        id
        startTime
        borrowAsset
        collateralAsset
        borrowLimit
        minBorrowAmount
        borrowAssetStrategy
        sharesHeld
        borrowerInterestShares
        yieldInterestWithdrawnShares
        collateralHeld
        areTokensTransferable
        verifier{
          id
          status
          usersVerified{
            metadata
          }
        }
        lender{
          lenderAddress
          amountLent
          amountWithdrawn
          sharesWithdrawn
          strategy
        }
      }
    }`,
        });
        const options = {
            url,
            headers: { 'Content-Type': 'application/json' },
            body: data,
        };
        const result = yield (0, helpers_1.fetchData)(options);
        // print({result});
        allData.push(...result.data.lenderPools);
        // console.log({allData})
        return allData;
    });
}
exports.getLenderPerPool = getLenderPerPool;
function getAllPooledCreditLinesForCount(url) {
    return __awaiter(this, void 0, void 0, function* () {
        let skip = 0;
        const allData = [];
        for (;;) {
            const data = JSON.stringify({
                query: `{
        pooledCreditLines(first: ${helpers_1.countPerQuery}, skip:${skip * helpers_1.countPerQuery}, orderBy: createdAt, orderDirection: desc){
          id
        }
      }`,
            });
            const options = {
                url,
                headers: { 'Content-Type': 'application/json' },
                body: data,
            };
            const result = yield (0, helpers_1.fetchData)(options);
            if (result.errors) {
                (0, helpers_1.print)(result.errors);
                throw new Error('Error while fetching data from subgraph');
            }
            else if (result.data.pooledCreditLines.length == 0) {
                return allData;
            }
            else {
                skip++;
                allData.push(...result.data.pooledCreditLines);
            }
        }
    });
}
exports.getAllPooledCreditLinesForCount = getAllPooledCreditLinesForCount;
function getAllPooledCreditLinesForCountWithState(url, status) {
    return __awaiter(this, void 0, void 0, function* () {
        let skip = 0;
        const allData = [];
        for (;;) {
            const data = JSON.stringify({
                query: `{
        pooledCreditLines(first: ${helpers_1.countPerQuery}, skip:${skip * helpers_1.countPerQuery}, orderBy: createdAt, orderDirection: desc, where:{status_in:[${status}]}){
          id
          status
        }
      }`,
            });
            const options = {
                url,
                headers: { 'Content-Type': 'application/json' },
                body: data,
            };
            const result = yield (0, helpers_1.fetchData)(options);
            if (result.errors) {
                (0, helpers_1.print)(result.errors);
                throw new Error('Error while fetching data from subgraph');
            }
            else if (result.data.pooledCreditLines.length == 0) {
                return allData;
            }
            else {
                skip++;
                allData.push(...result.data.pooledCreditLines);
            }
        }
    });
}
exports.getAllPooledCreditLinesForCountWithState = getAllPooledCreditLinesForCountWithState;
function getAllPooledCreditLinesForCountWithStateNotIn(url, status) {
    return __awaiter(this, void 0, void 0, function* () {
        let skip = 0;
        const allData = [];
        for (;;) {
            const data = JSON.stringify({
                query: `{
        pooledCreditLines(first: ${helpers_1.countPerQuery}, skip:${skip * helpers_1.countPerQuery}, orderBy: createdAt, orderDirection: desc, where:{status_not_in:[${status}]}){
          id
          status
        }
      }`,
            });
            const options = {
                url,
                headers: { 'Content-Type': 'application/json' },
                body: data,
            };
            const result = yield (0, helpers_1.fetchData)(options);
            if (result.errors) {
                (0, helpers_1.print)(result.errors);
                throw new Error('Error while fetching data from subgraph');
            }
            else if (result.data.pooledCreditLines.length == 0) {
                return allData;
            }
            else {
                skip++;
                allData.push(...result.data.pooledCreditLines);
            }
        }
    });
}
exports.getAllPooledCreditLinesForCountWithStateNotIn = getAllPooledCreditLinesForCountWithStateNotIn;
function getAllPooledCreditLinesOfLenderWithState(url, lenderAddress, status) {
    return __awaiter(this, void 0, void 0, function* () {
        let skip = 0;
        lenderAddress = lenderAddress.toLowerCase();
        const allData = [];
        for (;;) {
            const data = JSON.stringify({
                query: `{
        lenderSharesBalancePCLs(where:{user:"${lenderAddress}"}){
          user
          pooledCreditLines(first: ${helpers_1.countPerQuery}, skip: ${helpers_1.countPerQuery * skip}, where:{status_in:[${status}]}){
            id
            status
          }
        }
      }`,
            });
            const options = {
                url,
                headers: { 'Content-Type': 'application/json' },
                body: data,
            };
            const result = yield (0, helpers_1.fetchData)(options);
            // print({ result });
            if (result.errors) {
                (0, helpers_1.print)(result.errors);
                throw new Error('Error while fetching data from subgraph');
            }
            else if (result.data.lenderSharesBalancePCLs.length == 0) {
                return allData;
            }
            else {
                skip++;
                const linesInThisIteration = [];
                for (let index = 0; index < result.data.lenderSharesBalancePCLs.length; index++) {
                    const element = result.data.lenderSharesBalancePCLs[index];
                    allData.push(...element.pooledCreditLines);
                    linesInThisIteration.push(...element.pooledCreditLines);
                }
                if (linesInThisIteration.length == 0) {
                    return allData;
                }
            }
        }
    });
}
exports.getAllPooledCreditLinesOfLenderWithState = getAllPooledCreditLinesOfLenderWithState;
function getAllPooledCreditLinesOfLender(url, lenderAddress) {
    return __awaiter(this, void 0, void 0, function* () {
        let skip = 0;
        lenderAddress = lenderAddress.toLowerCase();
        const allData = [];
        for (;;) {
            const data = JSON.stringify({
                query: `{
        lenderSharesBalancePCLs(where:{user:"${lenderAddress}"}){
          user
          pooledCreditLines(first: ${helpers_1.countPerQuery}, skip: ${helpers_1.countPerQuery * skip}){
            id
            status
          }
        }
      }`,
            });
            const options = {
                url,
                headers: { 'Content-Type': 'application/json' },
                body: data,
            };
            const result = yield (0, helpers_1.fetchData)(options);
            // print({ result });
            if (result.errors) {
                (0, helpers_1.print)(result.errors);
                throw new Error('Error while fetching data from subgraph');
            }
            else if (result.data.lenderSharesBalancePCLs.length == 0) {
                return allData;
            }
            else {
                skip++;
                const linesInThisIteration = [];
                for (let index = 0; index < result.data.lenderSharesBalancePCLs.length; index++) {
                    const element = result.data.lenderSharesBalancePCLs[index];
                    allData.push(...element.pooledCreditLines);
                    linesInThisIteration.push(...element.pooledCreditLines);
                }
                if (linesInThisIteration.length == 0) {
                    return allData;
                }
            }
        }
    });
}
exports.getAllPooledCreditLinesOfLender = getAllPooledCreditLinesOfLender;
function getAllPooledCreditLinesOfLenderWithStateNotIn(url, lenderAddress, status) {
    return __awaiter(this, void 0, void 0, function* () {
        let skip = 0;
        lenderAddress = lenderAddress.toLowerCase();
        const allData = [];
        for (;;) {
            const data = JSON.stringify({
                query: `{
        lenderSharesBalancePCLs(where:{user:"${lenderAddress}"}){
          user
          pooledCreditLines(first: ${helpers_1.countPerQuery}, skip: ${helpers_1.countPerQuery * skip}, where:{status_not_in:[${status}]}){
            id
            status
          }
        }
      }`,
            });
            const options = {
                url,
                headers: { 'Content-Type': 'application/json' },
                body: data,
            };
            const result = yield (0, helpers_1.fetchData)(options);
            // print({ result });
            if (result.errors) {
                (0, helpers_1.print)(result.errors);
                throw new Error('Error while fetching data from subgraph');
            }
            else if (result.data.lenderSharesBalancePCLs.length == 0) {
                return allData;
            }
            else {
                skip++;
                const linesInThisIteration = [];
                for (let index = 0; index < result.data.lenderSharesBalancePCLs.length; index++) {
                    const element = result.data.lenderSharesBalancePCLs[index];
                    allData.push(...element.pooledCreditLines);
                    linesInThisIteration.push(...element.pooledCreditLines);
                }
                if (linesInThisIteration.length == 0) {
                    return allData;
                }
            }
        }
    });
}
exports.getAllPooledCreditLinesOfLenderWithStateNotIn = getAllPooledCreditLinesOfLenderWithStateNotIn;
function getAllPooledCreditLinesOfBorrowerWithState(url, borrowerAddress, status) {
    return __awaiter(this, void 0, void 0, function* () {
        let skip = 0;
        borrowerAddress = borrowerAddress.toLowerCase();
        const allData = [];
        for (;;) {
            const data = JSON.stringify({
                query: `{
        pooledCreditLines(first: ${helpers_1.countPerQuery}, skip:${skip * helpers_1.countPerQuery}, orderBy: createdAt, orderDirection: desc, where:{borrowerAddress:"${borrowerAddress}",status_in:[${status}]}){
          id
          status
          endsAt
          defaultsAt
        }
      }`,
            });
            const options = {
                url,
                headers: { 'Content-Type': 'application/json' },
                body: data,
            };
            const result = yield (0, helpers_1.fetchData)(options);
            if (result.errors) {
                (0, helpers_1.print)(result.errors);
                throw new Error('Error while fetching data from subgraph');
            }
            else if (result.data.pooledCreditLines.length == 0) {
                return allData;
            }
            else {
                skip++;
                allData.push(...result.data.pooledCreditLines);
            }
        }
    });
}
exports.getAllPooledCreditLinesOfBorrowerWithState = getAllPooledCreditLinesOfBorrowerWithState;
function getAllPooledCreditLinesOfBorrowerWithStateNotIn(url, borrowerAddress, status) {
    return __awaiter(this, void 0, void 0, function* () {
        let skip = 0;
        borrowerAddress = borrowerAddress.toLowerCase();
        const allData = [];
        for (;;) {
            const data = JSON.stringify({
                query: `{
        pooledCreditLines(first: ${helpers_1.countPerQuery}, skip:${skip * helpers_1.countPerQuery}, orderBy: createdAt, orderDirection: desc, where:{borrowerAddress:"${borrowerAddress}",status_not_in:[${status}]}){
          id
          status
        }
      }`,
            });
            const options = {
                url,
                headers: { 'Content-Type': 'application/json' },
                body: data,
            };
            const result = yield (0, helpers_1.fetchData)(options);
            if (result.errors) {
                (0, helpers_1.print)(result.errors);
                throw new Error('Error while fetching data from subgraph');
            }
            else if (result.data.pooledCreditLines.length == 0) {
                return allData;
            }
            else {
                skip++;
                allData.push(...result.data.pooledCreditLines);
            }
        }
    });
}
exports.getAllPooledCreditLinesOfBorrowerWithStateNotIn = getAllPooledCreditLinesOfBorrowerWithStateNotIn;
function getAllPooledCreditLinesOfBorrower(url, borrowerAddress) {
    return __awaiter(this, void 0, void 0, function* () {
        let skip = 0;
        borrowerAddress = borrowerAddress.toLowerCase();
        const allData = [];
        for (;;) {
            const data = JSON.stringify({
                query: `{
        pooledCreditLines(first: ${helpers_1.countPerQuery}, skip:${skip * helpers_1.countPerQuery}, orderBy: createdAt, orderDirection: desc, where:{borrowerAddress:"${borrowerAddress}"}){
          id
          status
        }
      }`,
            });
            const options = {
                url,
                headers: { 'Content-Type': 'application/json' },
                body: data,
            };
            const result = yield (0, helpers_1.fetchData)(options);
            if (result.errors) {
                (0, helpers_1.print)(result.errors);
                throw new Error('Error while fetching data from subgraph');
            }
            else if (result.data.pooledCreditLines.length == 0) {
                return allData;
            }
            else {
                skip++;
                allData.push(...result.data.pooledCreditLines);
            }
        }
    });
}
exports.getAllPooledCreditLinesOfBorrower = getAllPooledCreditLinesOfBorrower;
function getLendersOfPooledCreditLines(url, id) {
    return __awaiter(this, void 0, void 0, function* () {
        let skip = 0;
        const allData = [];
        for (;;) {
            const data = JSON.stringify({
                query: `{
        lenderPools(where:{id:${id}}){
          id
          lender(first: ${helpers_1.countPerQuery}, skip: ${helpers_1.countPerQuery * skip}){
            id
          }
        }
      }`,
            });
            const options = {
                url,
                headers: { 'Content-Type': 'application/json' },
                body: data,
            };
            const result = yield (0, helpers_1.fetchData)(options);
            // print({result});
            if (result.errors) {
                (0, helpers_1.print)(result.errors);
                throw new Error('Error while fetching data from subgraph');
            }
            else if (result.data.lenderPools.length == 0) {
                return allData;
            }
            else if (result.data.lenderPools[0].lender.length == 0) {
                return allData;
            }
            else {
                skip++;
                allData.push(...result.data.lenderPools[0].lender);
            }
        }
    });
}
exports.getLendersOfPooledCreditLines = getLendersOfPooledCreditLines;
function getPCLandLpTogether(url, count, skip) {
    return __awaiter(this, void 0, void 0, function* () {
        const allData = [];
        const data = JSON.stringify({
            query: `{
      lenderPools(first:${count}, skip:${skip}){
        id,
        sharesHeld,
        borrowerInterestShares
        borrowerInterestSharesWithdrawn
        yieldInterestWithdrawnShares
        borrowLimit
        startTime
        minBorrowAmount
        pooledCreditLine{
          id
          status
          endsAt
          principal
          idealCollateralRatio
          totalInterestRepaid
          lastPrincipalUpdateTime
          gracePenaltyRate
          borrowRate
          interestAccruedTillLastPrincipalUpdate
          borrowLimit
          borrowAsset
          collateralAsset
          lenderStrategy
          depositedCollateralInShares
          borrowerAddress
          collateralAsset
          createdAt
          startsAt
          collateralStrategy
          totalLentAmount
          lenderVerifier {
            id
          }
          borrowerVerifier {
            id
          }
        }
      }
    }`,
        });
        const options = {
            url,
            headers: { 'Content-Type': 'application/json' },
            body: data,
        };
        const result = yield (0, helpers_1.fetchData)(options);
        // print({result});
        allData.push(...result.data.lenderPools);
        // console.log({allData})
        return allData;
    });
}
exports.getPCLandLpTogether = getPCLandLpTogether;
function getAllLendersPerPool(url, poolIds) {
    return __awaiter(this, void 0, void 0, function* () {
        poolIds = poolIds.map((a) => `"${a}"`);
        let skip = 0;
        const allData = [];
        for (;;) {
            const data = JSON.stringify({
                query: `{
        lenderPerLenderPools(first:${helpers_1.countPerQuery}, skip: ${helpers_1.countPerQuery * skip}, where:{lenderPool_in:[${poolIds}]}){
          id
          lenderAddress
          amountLent
          amountWithdrawn
          sharesWithdrawn
          interestWithdrawn
          strategy
          borrowerInterestSharesWithdrawn
          yieldInterestWithdrawnShares
          lenderBalance
          lenderPool {id}
        }
      }`,
            });
            const options = {
                url,
                headers: { 'Content-Type': 'application/json' },
                body: data,
            };
            const result = yield (0, helpers_1.fetchData)(options);
            if (result.errors) {
                (0, helpers_1.print)(result.errors);
                throw new Error('Error while fetching data from subgraph');
            }
            else if (result.data.lenderPerLenderPools.length == 0) {
                return allData;
            }
            else {
                skip++;
                allData.push(...result.data.lenderPerLenderPools);
            }
        }
    });
}
exports.getAllLendersPerPool = getAllLendersPerPool;
//# sourceMappingURL=getPooledCreditLines.js.map