"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getLenderPooledCreditLineCollective = exports.getBorrowerPooledCreditLineCollective = void 0;
const helpers_1 = require("../helpers");
function getBorrowerPooledCreditLineCollective(url, address) {
    return __awaiter(this, void 0, void 0, function* () {
        let skip = 0;
        address = address.toLowerCase();
        const allData = [];
        for (;;) {
            const data = JSON.stringify({
                query: `{
        borrowerSharesBalancePCLs(first: ${helpers_1.countPerQuery}, skip:${skip * helpers_1.countPerQuery}, where:{user:"${address}"}){
          id
          token
          user
          strategy
          pooledCreditLines{
            id
          }
          collateralDeposited
          amountRepaid
          amountBorrowed
        }
      }`,
            });
            const options = {
                url,
                headers: { 'Content-Type': 'application/json' },
                body: data,
            };
            const result = yield (0, helpers_1.fetchData)(options);
            if (result.errors) {
                (0, helpers_1.print)(result.errors);
                throw new Error('Error while fetching data from subgraph');
            }
            else if (result.data.borrowerSharesBalancePCLs.length == 0) {
                return allData;
            }
            else {
                skip++;
                allData.push(...result.data.borrowerSharesBalancePCLs);
            }
        }
    });
}
exports.getBorrowerPooledCreditLineCollective = getBorrowerPooledCreditLineCollective;
function getLenderPooledCreditLineCollective(url, address) {
    return __awaiter(this, void 0, void 0, function* () {
        let skip = 0;
        address = address.toLowerCase();
        const allData = [];
        for (;;) {
            const data = JSON.stringify({
                query: `{
        lenderSharesBalancePCLs(first: ${helpers_1.countPerQuery}, skip:${skip * helpers_1.countPerQuery}, where:{user:"${address}"}){
          id
          token
          user
          strategy
          pooledCreditLines{
            id
            status
            endsAt
            principal
            idealCollateralRatio
            totalInterestRepaid
            lastPrincipalUpdateTime
            gracePenaltyRate
            borrowRate
            interestAccruedTillLastPrincipalUpdate
            borrowLimit
            borrowAsset
            collateralAsset
            lenderStrategy
            depositedCollateralInShares
            borrowerAddress
            createdAt
            startsAt
            defaultsAt
            collateralStrategy
            totalLentAmount
            status
            minBorrowAmount
            lenderVerifier {
              id
            }
            borrowerVerifier {
              id
            }
            lenderPool{
              id
              sharesHeld
              borrowerInterestShares
              borrowerInterestSharesWithdrawn
              yieldInterestWithdrawnShares
              borrowLimit
              startTime
              minBorrowAmount
            }
          }
          amountLent
          amountWithdrawn
          sharesWithdrawn
          interestWithdrawn
        }
      }`,
            });
            const options = {
                url,
                headers: { 'Content-Type': 'application/json' },
                body: data,
            };
            const result = yield (0, helpers_1.fetchData)(options);
            if (result.errors) {
                (0, helpers_1.print)(result.errors);
                throw new Error('Error while fetching data from subgraph');
            }
            else if (result.data.lenderSharesBalancePCLs.length == 0) {
                return allData;
            }
            else {
                skip++;
                allData.push(...result.data.lenderSharesBalancePCLs);
            }
        }
    });
}
exports.getLenderPooledCreditLineCollective = getLenderPooledCreditLineCollective;
//# sourceMappingURL=pclCollective.js.map