"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getUniswapPrice = void 0;
const bignumber_js_1 = require("bignumber.js");
const helpers_1 = require("../helpers");
function getUniswapPrice(url, address) {
    return __awaiter(this, void 0, void 0, function* () {
        address = address.toLowerCase();
        const data = JSON.stringify({
            query: `
      {
        price0: pairs(where:{
          token0: "${address}",
          token1_in : [
            "0xa0b86991c6218b36c1d19d4a2e9eb0ce3606eb48",
            "0xdac17f958d2ee523a2206206994597c13d831ec7",
            "0x6b175474e89094c44da98b954eedeac495271d0f"
          ]
        }, orderBy: volumeUSD, orderDirection: desc, first: 1) {
          token1Price
        }
        price1: pairs(where:{
          token1: "${address}",
          token0_in : [
            "0xa0b86991c6218b36c1d19d4a2e9eb0ce3606eb48",
            "0xdac17f958d2ee523a2206206994597c13d831ec7",
            "0x6b175474e89094c44da98b954eedeac495271d0f"
          ]
        }, orderBy: volumeUSD, orderDirection: desc, first: 1) {
          token0Price
        }
      }
    `,
        });
        const options = {
            url,
            headers: { 'Content-Type': 'application/json' },
            body: data,
        };
        const result = yield (0, helpers_1.fetchData)(options);
        if (!result) {
            throw new Error('Error while fetching data from subgraph');
        }
        if (result.errors) {
            (0, helpers_1.print)(result.errors);
            throw new Error('Error while fetching data from subgraph');
        }
        let price = new bignumber_js_1.BigNumber(0);
        if (result.data.price0 && result.data.price0.length != 0) {
            price = result.data.price0[0].token1Price;
        }
        if (result.data.price1 && result.data.price1.length != 0) {
            price = result.data.price1[0].token0Price;
        }
        if (result.data.price0 && result.data.price1 && result.data.price0.length != 0 && result.data.price1.length != 0) {
            const volume0 = new bignumber_js_1.BigNumber(result.data.price0[0].volumeUSD);
            const volume1 = new bignumber_js_1.BigNumber(result.data.price1[0].volumeUSD);
            if (volume0.gt(volume1)) {
                price = result.data.price0[0].token1Price;
            }
        }
        return price;
    });
}
exports.getUniswapPrice = getUniswapPrice;
//# sourceMappingURL=prices.js.map