"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAllowances = exports.getSavingsAccountTokenDetails = exports.getBalances = void 0;
const subgraph_1 = require("../utils/subgraph");
const helpers_1 = require("../helpers");
function getBalances(url, address) {
    return __awaiter(this, void 0, void 0, function* () {
        // TODO: Update queries to use graphql variables instead
        // TODO: Support pagination for the case when number of tokens goes above 1000 😅
        address = address.toLowerCase();
        const queryData = JSON.stringify({
            query: `{
      balances(where: {
        user: "${address}"
      }, first: ${helpers_1.countPerQuery}) {
        token
        strategy {
          address
        }
        shares
      }
    }`,
        });
        const data = yield (0, subgraph_1.query)(url, queryData);
        return data.balances;
    });
}
exports.getBalances = getBalances;
function getSavingsAccountTokenDetails(url, address) {
    return __awaiter(this, void 0, void 0, function* () {
        address = address.toLowerCase();
        let skip = 0;
        const allData = [];
        for (;;) {
            const data = JSON.stringify({
                query: `{
        savingAccounts(where:{id:"${address}"}) {
          id,
          deposits (first: ${helpers_1.countPerQuery}, skip:${skip * helpers_1.countPerQuery}){
            id,
            asset
          },
        }
      }
    `,
            });
            const options = {
                url,
                headers: { 'Content-Type': 'application/json' },
                body: data,
            };
            const result = yield (0, helpers_1.fetchData)(options);
            // console.log({result, skip});
            if (result.errors) {
                (0, helpers_1.print)(result.errors);
                throw new Error('Error while fetching data from subgraph');
            }
            else if (result.data.savingAccounts.length == 0) {
                return allData;
            }
            else if (result.data.savingAccounts.length > 0 &&
                result.data.savingAccounts[0].deposits &&
                result.data.savingAccounts[0].deposits == 0) {
                return allData;
            }
            else {
                skip++;
                allData.push(...result.data.savingAccounts[0].deposits);
            }
        }
    });
}
exports.getSavingsAccountTokenDetails = getSavingsAccountTokenDetails;
function getAllowances(url, address, to) {
    return __awaiter(this, void 0, void 0, function* () {
        address = address.toLowerCase();
        to = to.toLowerCase();
        let skip = 0;
        const allData = [];
        for (;;) {
            const data = JSON.stringify({
                query: `{
        allowances(where:{from:"${address}", to:"${to}"} , first: ${helpers_1.countPerQuery}, skip:${skip * helpers_1.countPerQuery}) {
          amount
          from
          to
          token
        }
      }
    `,
            });
            const options = {
                url,
                headers: { 'Content-Type': 'application/json' },
                body: data,
            };
            const result = yield (0, helpers_1.fetchData)(options);
            // console.log({result});
            // console.log({result, skip});
            if (result.errors) {
                (0, helpers_1.print)(result.errors);
                throw new Error('Error while fetching data from subgraph');
            }
            else if (result.data.allowances.length == 0) {
                return allData;
            }
            else {
                skip++;
                allData.push(...result.data.allowances);
            }
        }
    });
}
exports.getAllowances = getAllowances;
//# sourceMappingURL=savingsAccount.js.map