"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getTwitterDetails = exports.getUserMetadata = exports.getTwitterId = void 0;
const subgraph_1 = require("../utils/subgraph");
const axios_1 = __importDefault(require("axios"));
const sublimeVerifierUrl = 'https://sublime-twitter-fetch-userinfobyid.saxenism.workers.dev/?twitterID=';
function getTwitterId(url, address) {
    return __awaiter(this, void 0, void 0, function* () {
        // TODO: Update queries to use graphql variables instead
        // TODO: Support pagination for the case when number of tokens goes above 1000 😅
        address = address.toLowerCase();
        const queryData = JSON.stringify({
            query: `{
  userProfiles(where : {id : "${address}"}) {
    userMetadataPerVerifier {
      id
      metadata
    }
  }
}
`,
        });
        const data = yield (0, subgraph_1.query)(url, queryData);
        // return data.userProfiles;
        if (data.userProfiles.length > 0) {
            return data.userProfiles[0].userMetadataPerVerifier;
        }
        else {
            return [];
        }
    });
}
exports.getTwitterId = getTwitterId;
function getUserMetadata(url, address) {
    return __awaiter(this, void 0, void 0, function* () {
        // TODO: Update queries to use graphql variables instead
        // TODO: Support pagination for the case when number of tokens goes above 1000 😅
        address = address.toLowerCase();
        const queryData = JSON.stringify({
            query: `{
  userProfiles(where : {id : "${address}"}) {
    userMetadataPerVerifier {
      id,
      verifier,
      metadata,
      verifiedBy{
        id
        status
      }
    }
  }
}
`,
        });
        const data = yield (0, subgraph_1.query)(url, queryData);
        // return data.userProfiles;
        if (data.userProfiles.length > 0) {
            return data.userProfiles[0].userMetadataPerVerifier;
        }
        else {
            return [];
        }
    });
}
exports.getUserMetadata = getUserMetadata;
/**
 *
 * @param twtID This is twitter id
 * @returns
 */
function getTwitterDetails(twtID) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const { data, status } = yield axios_1.default.get(sublimeVerifierUrl + twtID);
            if (status >= 200 && status < 400) {
                return {
                    profilePic: data.account_dp_url,
                    name: data.account_name,
                    twitterId: data.account_id,
                    userName: data.account_username,
                };
            }
            else {
                throw new Error('invalid status');
            }
        }
        catch (ex) {
            return {
                profilePic: undefined,
                name: undefined,
                twitterId: twtID,
                userName: undefined,
            };
        }
    });
}
exports.getTwitterDetails = getTwitterDetails;
//# sourceMappingURL=verification.js.map