import { Signer } from '@ethersproject/abstract-signer';
import { TokenManager } from '../tokenManager';
import { SublimeConfig } from '../types/sublimeConfig';
import { YieldAndStrategyApi } from '../api/yieldAndStrategy';
import { DashboardOverview, StrategyType } from '../types/Types';
import BigNumber from 'bignumber.js';
import { VerificationAPI } from '../api/verification';
export declare class Base {
    /**
     * @description subgraph url for fetching the sublime data
     */
    protected subgraphUrl: string;
    /**
     * @description signer used to sign
     */
    protected signer: Signer;
    /**
     * @description instance of token manager class that caches and retreives token data
     */
    protected tokenManager: TokenManager;
    protected yieldApi: YieldAndStrategyApi;
    protected verificationApi: VerificationAPI;
    /**
     * @description sublime config. (Contains all addresses relevant to sublime ecosystem)
     */
    protected sublimeAddresses: SublimeConfig;
    constructor(url: string, signer: Signer, tokenManager: TokenManager, config: SublimeConfig);
    /**
     * @param max
     * @description Function to generate a random decimals in range (0, max)
     */
    protected getRandomInt(max: any): number;
    /**
     * @param address: address of the user to query
     * @description Return the dashboard view type for a user
     */
    getDashboardOverview(address: string): Promise<DashboardOverview>;
    protected refreshTokenData(data: any[]): Promise<Record<string, BigNumber>>;
    protected refreshStrategyDataForPcl(data: any[]): Promise<Record<string, Record<string, BigNumber>>>;
    protected refreshStrategyData(data: any[]): Promise<Record<string, Record<string, BigNumber>>>;
    protected getTokensForShares(strategy: StrategyType, collateralAsset: string, amount: BigNumber): Promise<BigNumber>;
}
