"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Base = void 0;
const yieldAndStrategy_1 = require("../api/yieldAndStrategy");
const Types_1 = require("../types/Types");
const bignumber_js_1 = __importDefault(require("bignumber.js"));
const verification_1 = require("../api/verification");
class Base {
    constructor(url, signer, tokenManager, config) {
        this.subgraphUrl = url;
        this.signer = signer;
        this.tokenManager = tokenManager;
        this.sublimeAddresses = config;
        this.yieldApi = new yieldAndStrategy_1.YieldAndStrategyApi(this.signer, config, this.tokenManager);
        this.verificationApi = new verification_1.VerificationAPI(this.signer, config);
    }
    /**
     * @param max
     * @description Function to generate a random decimals in range (0, max)
     */
    getRandomInt(max) {
        return Math.floor(Math.random() * max);
    }
    /**
     * @param address: address of the user to query
     * @description Return the dashboard view type for a user
     */
    getDashboardOverview(address) {
        return __awaiter(this, void 0, void 0, function* () {
            return {
                totalBorrowedAmount: { value: new bignumber_js_1.default(this.getRandomInt(100000000)).div(100).toFixed(2), decimals: 2 },
                totalLentAmount: { value: new bignumber_js_1.default(this.getRandomInt(10000000)).div(100).toFixed(2), decimals: 2 },
                totalBorrowRate: new bignumber_js_1.default(this.getRandomInt(1000)).div(100).toFixed(2),
                totalLentRate: new bignumber_js_1.default(this.getRandomInt(1000)).div(100).toFixed(2),
            };
        });
    }
    refreshTokenData(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const borrowTokens = data.map((a) => a.collateralAsset);
            const collateralTokens = data.map((a) => a.borrowAsset);
            const allTokens = [...borrowTokens, ...collateralTokens].filter((value, index, array) => array.indexOf(value) === index);
            const prices = {};
            for (let index = 0; index < allTokens.length; index++) {
                const element = allTokens[index];
                yield this.tokenManager.updateAll(element);
                prices[element] = yield this.tokenManager.getPricePerAsset(element);
            }
            return prices;
        });
    }
    refreshStrategyDataForPcl(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const allStrategyAndTokenPairs = data.map((a) => {
                return { strategy: a.lenderStrategy, collateralAsset: a.collateralAsset, id: `${a.lenderStrategy}-${a.collateralAsset}` };
            });
            const allIds = allStrategyAndTokenPairs.map((a) => a.id).filter((value, index, self) => self.indexOf(value) === index);
            const uniquePairs = allIds.map((a) => {
                return { strategy: a.split('-')[0], collateralAsset: a.split('-')[1] };
            });
            const collateralPerStrategyToken = {};
            const valueToCheck = new bignumber_js_1.default(10).pow(18);
            for (let index = 0; index < uniquePairs.length; index++) {
                const element = uniquePairs[index];
                const strategy = this.yieldApi.getStrategy(element.strategy);
                const tokensForShares = yield this.getTokensForShares(strategy, element.collateralAsset, valueToCheck);
                if (!collateralPerStrategyToken[element.strategy]) {
                    collateralPerStrategyToken[element.strategy] = {};
                }
                collateralPerStrategyToken[element.strategy][element.collateralAsset] = tokensForShares.div(valueToCheck);
            }
            return collateralPerStrategyToken;
        });
    }
    refreshStrategyData(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const allStrategyAndTokenPairs = data.map((a) => {
                return {
                    strategy: a.collateralAssetStrategy,
                    collateralAsset: a.collateralAsset,
                    id: `${a.collateralAssetStrategy}-${a.collateralAsset}`,
                };
            });
            const allIds = allStrategyAndTokenPairs.map((a) => a.id).filter((value, index, self) => self.indexOf(value) === index);
            const uniquePairs = allIds.map((a) => {
                return { strategy: a.split('-')[0], collateralAsset: a.split('-')[1] };
            });
            const collateralPerStrategyToken = {};
            const valueToCheck = new bignumber_js_1.default(10).pow(18);
            for (let index = 0; index < uniquePairs.length; index++) {
                const element = uniquePairs[index];
                const strategy = this.yieldApi.getStrategy(element.strategy);
                const tokensForShares = yield this.getTokensForShares(strategy, element.collateralAsset, valueToCheck);
                if (!collateralPerStrategyToken[element.strategy]) {
                    collateralPerStrategyToken[element.strategy] = {};
                }
                collateralPerStrategyToken[element.strategy][element.collateralAsset] = tokensForShares.div(valueToCheck);
            }
            return collateralPerStrategyToken;
        });
    }
    getTokensForShares(strategy, collateralAsset, amount) {
        return __awaiter(this, void 0, void 0, function* () {
            if (strategy == Types_1.StrategyType.NoYield)
                return new bignumber_js_1.default(amount);
            try {
                const tokensInCompoundYield = yield this.yieldApi.getTokensForShares(strategy, collateralAsset, amount);
                return tokensInCompoundYield;
            }
            catch (ex) {
                return new bignumber_js_1.default(0);
            }
        });
    }
}
exports.Base = Base;
//# sourceMappingURL=Base.js.map