"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CreditLineInterestCalculator = void 0;
const PooledCreditLinesLenderOverview_1 = require("./PooledCreditLinesLenderOverview");
const queries_1 = require("../queries");
const bignumber_js_1 = __importDefault(require("bignumber.js"));
const exponent = new bignumber_js_1.default(10).pow(18);
const oneYear = new bignumber_js_1.default(365).multipliedBy(86400);
class CreditLineInterestCalculator extends PooledCreditLinesLenderOverview_1.PooledCreditLinesLenderOverview {
    constructor(url, signer, tokenManager, config) {
        super(url, signer, tokenManager, config);
    }
    getBorrowerInterestDueForAllCreditLines(borrower) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = [];
            const count = 999;
            let skip = 0;
            for (;;) {
                const _temp = yield (0, queries_1.getConfirmedCreditLinesOfBorrower)(this.subgraphUrl, borrower, count, skip);
                skip += count;
                if (_temp.length == 0) {
                    break;
                }
                else {
                    result.push(..._temp);
                }
            }
            return this.transformToInterestDueDetail(result);
        });
    }
    getBorrowerInterestDueForSpecificCreditLines(borrower, creditLines) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = [];
            const count = 999;
            let skip = 0;
            for (;;) {
                const _temp = yield (0, queries_1.getConfirmedCreditLinesOfBorrower)(this.subgraphUrl, borrower, count, skip);
                skip += count;
                if (_temp.length == 0) {
                    break;
                }
                else {
                    result.push(..._temp);
                }
            }
            const relevant = result.filter((a) => creditLines.includes(a.id));
            return this.transformToInterestDueDetail(relevant);
        });
    }
    transformToInterestDueDetail(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const borrowTokens = data.map((a) => a.borrowAsset);
            const collateralTokens = data.map((a) => a.collateralAsset);
            const allTokens = [...borrowTokens, ...collateralTokens].filter((value, index, self) => self.indexOf(value) === index);
            const prices = {};
            for (let index = 0; index < allTokens.length; index++) {
                const element = allTokens[index];
                yield this.tokenManager.updateAll(element);
                prices[element] = yield this.tokenManager.getPricePerAsset(element);
            }
            const interestDuePerTokenPerStrategy = yield this.transformToInterestDuePerTokenPerStrategy(data, prices);
            const interestDuePerStrategy = yield this.transformToInterestDuePerStrategy(interestDuePerTokenPerStrategy, prices);
            const interestDuePerToken = yield this.transformToInterestDuePerToken(interestDuePerTokenPerStrategy);
            const interestDue = yield this.transformToInterestDue(interestDuePerTokenPerStrategy, prices);
            return { interestDue, interestDuePerStrategy, interestDuePerToken, interestDuePerTokenPerStrategy };
        });
    }
    transformToInterestDuePerStrategy(interestDuePerTokenPerStrategy, prices) {
        return __awaiter(this, void 0, void 0, function* () {
            const interestDuePerStrategy = [];
            const strategies = interestDuePerTokenPerStrategy
                .map((a) => a.strategy.address)
                .filter((value, index, self) => self.indexOf(value) === index);
            for (let index = 0; index < strategies.length; index++) {
                const element = strategies[index];
                const requiredElements = interestDuePerTokenPerStrategy.filter((a) => a.strategy.address === element);
                if (requiredElements.length > 0) {
                    const totalInterest = requiredElements.reduce((total, current) => total.plus(new bignumber_js_1.default(current.interest.value.toString())
                        .multipliedBy(prices[current.token.address])
                        .dividedBy(new bignumber_js_1.default(10).pow(this.tokenManager.getTokenDecimals(current.token.address)))), new bignumber_js_1.default(0));
                    interestDuePerStrategy.push({
                        id: 'to-do',
                        interest: { value: totalInterest.toString(), decimals: 0 },
                        strategy: requiredElements[0].strategy,
                    });
                }
            }
            return interestDuePerStrategy;
        });
    }
    transformToInterestDuePerToken(interestDuePerTokenPerStrategy) {
        return __awaiter(this, void 0, void 0, function* () {
            const interestDuePerToken = [];
            const uniqueTokens = interestDuePerTokenPerStrategy
                .map((a) => a.token.address)
                .filter((value, index, self) => self.indexOf(value) === index);
            for (let index = 0; index < uniqueTokens.length; index++) {
                const element = uniqueTokens[index];
                const requiredElements = interestDuePerTokenPerStrategy.filter((a) => a.token.address === element);
                if (requiredElements.length > 0) {
                    const totalInterest = requiredElements.reduce((total, current) => total.plus(current.interest.value.toString()), new bignumber_js_1.default(0));
                    interestDuePerToken.push({
                        interest: { value: totalInterest.toString(), decimals: this.tokenManager.getTokenDecimals(requiredElements[0].token.address) },
                        token: requiredElements[0].token,
                        id: 'to-do',
                    });
                }
            }
            return interestDuePerToken;
        });
    }
    transformToInterestDue(interestDuePerTokenPerStrategy, prices) {
        return __awaiter(this, void 0, void 0, function* () {
            const totalInterest = interestDuePerTokenPerStrategy.reduce((total, current) => total.plus(new bignumber_js_1.default(current.interest.value.toString())
                .multipliedBy(prices[current.token.address])
                .dividedBy(new bignumber_js_1.default(10).pow(this.tokenManager.getTokenDecimals(current.token.address)))), new bignumber_js_1.default(0));
            return { id: 'to-do', interest: { value: totalInterest.toString(), decimals: 0 } };
        });
    }
    transformToInterestDuePerTokenPerStrategy(data, prices) {
        return __awaiter(this, void 0, void 0, function* () {
            return data.map((a) => {
                return {
                    id: a.id,
                    token: {
                        address: a.borrowAsset,
                        name: this.tokenManager.getTokenName(a.borrowAsset),
                        logo: this.tokenManager.getLogo(a.borrowAsset),
                        pricePerAssetInUSD: prices[a.borrowAsset].toString(),
                        decimals: this.tokenManager.getTokenDecimals(a.borrowAsset),
                    },
                    strategy: {
                        address: a.strategy,
                        type: this.yieldApi.getStrategy(a.strategy),
                        displayName: this.yieldApi.getStrategyDisplayName(a.strategy),
                        logo: this.yieldApi.getStrategyLogo(a.strategy),
                    },
                    interest: {
                        value: this.calculateInterest(a.principal, a.borrowRate, a.lastPrincipalUpdateTime).toString(),
                        decimals: this.tokenManager.getTokenDecimals(a.borrowAsset),
                    },
                };
            });
        });
    }
    calculateInterest(principal, borrowRate, lastPrincipalUpdateTime) {
        const now = new bignumber_js_1.default(new Date().valueOf()).div(1000);
        if (now.gt(lastPrincipalUpdateTime)) {
            return new bignumber_js_1.default(principal)
                .multipliedBy(now.minus(lastPrincipalUpdateTime))
                .multipliedBy(borrowRate)
                .dividedBy(exponent)
                .dividedBy(oneYear);
        }
        else {
            return new bignumber_js_1.default(0);
        }
    }
}
exports.CreditLineInterestCalculator = CreditLineInterestCalculator;
// {
//     id: '39',
//     status: 'ACTIVE',
//     lender: '0xe33896558027811799165c5a85b7c9c318a0e7c4',
//     createdAt: '1652613432',
//     strategy: '0xf469c04fb301ab0dc5ba425e11645ad6b8c0be64',
//     totalInterestRepaid: '0',
//     lenderWalletDetails: { wallet: null },
//     borrower: '0x808ba745926e6cdef924c9bc0c758ec110474596',
//     borrowerWalletDetails: { wallet: null },
//     requestByLender: false,
//     principal: '9999999999734520925',
//     collateralAsset: '0xe22da380ee6b445bb8273c81944adeb6e8450422',
//     borrowLimit: '20000000000000000000',
//     borrowRate: '2628000000000000000000',
//     idealCollateralRatio: '0',
//     borrowAsset: '0x4f96fe3b7a6cf9725f59d353f723c1bdb64ca6aa',
//     autoLiquidation: true,
//     lastPrincipalUpdateTime: '1652615400'
//   }
//# sourceMappingURL=CreditLineInterestCalculator.js.map