"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CreditLinesOverviewCall = void 0;
const UserMeta_1 = require("./UserMeta");
const Types_1 = require("../types/Types");
const queries_1 = require("../queries");
const bignumber_js_1 = __importDefault(require("bignumber.js"));
class CreditLinesOverviewCall extends UserMeta_1.UserMetaCalls {
    constructor(url, signer, tokenManager, config) {
        super(url, signer, tokenManager, config);
    }
    /**
     * @description Total Amount put by a lender in all credit lines
     * @param user
     * @returns
     */
    getLenderCreditLineCollective(user) {
        return __awaiter(this, void 0, void 0, function* () {
            const data = yield (0, queries_1.getLenderCreditLineCollective)(this.subgraphUrl, user);
            const lenderCollectivePerTokenPerStrategy = yield this.tranformToLenderCollectivePerTokenPerStrategy(data);
            return {
                lenderCollective: this.TransformToLenderCollective(user, lenderCollectivePerTokenPerStrategy),
                lenderCollectivePerStrategy: this.transformToLenderCollectivePerStrategy(user, lenderCollectivePerTokenPerStrategy),
                lenderCollectivePerToken: this.transformToLenderCollectivePerToken(user, lenderCollectivePerTokenPerStrategy),
                lenderCollectivePerTokenPerStrategy: lenderCollectivePerTokenPerStrategy,
            };
        });
    }
    /**
     * @description Amount borrowed and repaid by a borrower accross all credit lines
     * @summary Time based interest calculation is pending. (amountRepaid + outstandingInterest)
     * @param user
     * @returns
     */
    getBorrowerCreditLineCollective(user) {
        return __awaiter(this, void 0, void 0, function* () {
            const data = yield (0, queries_1.getBorrowerCreditLineCollective)(this.subgraphUrl, user);
            const borrowerCollectivePerTokenPerStrategy = yield this.tranformToBorrowerCollectivePerTokenPerStrategy(data);
            return {
                borrowerCollective: this.TransformToBorrowerCollective(user, borrowerCollectivePerTokenPerStrategy),
                borrowerCollectivePerStrategy: this.transformToBorrowerCollectivePerStrategy(user, borrowerCollectivePerTokenPerStrategy),
                borrowerCollectivePerToken: this.transformToBorrowerCollectivePerToken(user, borrowerCollectivePerTokenPerStrategy),
                borrowerCollectivePerTokenPerStrategy: borrowerCollectivePerTokenPerStrategy,
            };
        });
    }
    getTotalAmountCommitedByLenderToCreditLines(lender) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = [];
            const count = 999;
            let skip = 0;
            for (;;) {
                const _temp = yield (0, queries_1.getConfirmedCreditLinesOfLender)(this.subgraphUrl, lender, count, skip);
                skip += count;
                if (_temp.length == 0) {
                    break;
                }
                else {
                    result.push(..._temp);
                }
            }
            const allBorrowAsset = result.map((a) => a.borrowAsset);
            for (let index = 0; index < allBorrowAsset.length; index++) {
                const element = allBorrowAsset[index];
                yield this.tokenManager.updateAll(element);
            }
            return this.transformToLenderTotalCredit(result);
        });
    }
    getInterestCollectedByLenderFromCreditLines(lender) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = [];
            const count = 999;
            let skip = 0;
            for (;;) {
                const _temp = yield (0, queries_1.getConfirmedCreditLinesOfLender)(this.subgraphUrl, lender, count, skip);
                skip += count;
                if (_temp.length == 0) {
                    break;
                }
                else {
                    result.push(..._temp);
                }
            }
            const allBorrowAsset = result.map((a) => a.borrowAsset);
            for (let index = 0; index < allBorrowAsset.length; index++) {
                const element = allBorrowAsset[index];
                yield this.tokenManager.updateAll(element);
            }
            return this.transformToInterestCollectedByLender(result);
        });
    }
    getAvaialbleBorrowLimitOfBorrowerOfCreditLines(borrower) {
        return __awaiter(this, void 0, void 0, function* () {
            let result = [];
            const count = 999;
            let skip = 0;
            for (;;) {
                // const _temp = await getConfirmedCreditLinesOfBorrower(this.subgraphUrl, borrower, count, skip);
                const _temp = yield this.getConfirmedCreditLinesOfBorrower(borrower, count, skip);
                skip += count;
                if (_temp.length == 0) {
                    break;
                }
                else {
                    result.push(..._temp);
                }
            }
            result = result.filter((a) => a.emulator.getStatus() === Types_1.CreditLineStatus.ACTIVE);
            // console.log(result.map(a => {return {
            //   id: a.emulator.getId(),
            //   status: a.emulator.getStatus()
            // }}))
            const allBorrowAsset = result.map((a) => a.emulator.borrowAsset());
            for (let index = 0; index < allBorrowAsset.length; index++) {
                const element = allBorrowAsset[index];
                yield this.tokenManager.updateAll(element);
            }
            return this.transformToBorrowerAvailableCredit(result);
        });
    }
    transformToInterestCollectedByLender(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const prices = {};
            const allTokens = data.map((a) => a.borrowAsset);
            for (let index = 0; index < allTokens.length; index++) {
                prices[allTokens[index]] = yield this.tokenManager.getPricePerAsset(allTokens[index]);
            }
            const total = data.reduce((total, current) => total.plus(new bignumber_js_1.default(current.totalInterestRepaid)
                .multipliedBy(prices[current.borrowAsset])
                .dividedBy(new bignumber_js_1.default(10).pow(this.tokenManager.getTokenDecimals(current.borrowAsset)))), new bignumber_js_1.default(0));
            return {
                amount: { value: total.toString(), decimals: 0 },
                interestCollectedByLenderPerToken: yield this.transformToInterestCollectedByLenderPerToken(data),
            };
        });
    }
    transformToInterestCollectedByLenderPerToken(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const interestCollectedByLenderPerToken = [];
            const allTokens = data.map((a) => a.borrowAsset);
            const uniqueTokens = allTokens.filter((value, index, self) => self.indexOf(value) === index);
            for (let index = 0; index < uniqueTokens.length; index++) {
                const element = uniqueTokens[index];
                const requiredEntries = data.filter((a) => a.borrowAsset === element);
                if (requiredEntries.length > 0) {
                    const totalInterestCollectedPerToken = requiredEntries.reduce((total, current) => total.plus(current.totalInterestRepaid), new bignumber_js_1.default(0));
                    interestCollectedByLenderPerToken.push({
                        amount: {
                            value: totalInterestCollectedPerToken.toString(),
                            decimals: this.tokenManager.getTokenDecimals(requiredEntries[0].borrowAsset),
                        },
                        token: yield this.tokenManager.getAssetMeta(requiredEntries[0].borrowAsset),
                    });
                }
            }
            return interestCollectedByLenderPerToken;
        });
    }
    transformToLenderTotalCredit(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const prices = {};
            const allTokens = data.map((a) => a.borrowAsset);
            for (let index = 0; index < allTokens.length; index++) {
                prices[allTokens[index]] = yield this.tokenManager.getPricePerAsset(allTokens[index]);
            }
            const lenderTotalCredit = data.reduce((total, current) => total.plus(new bignumber_js_1.default(current.borrowLimit)
                .multipliedBy(prices[current.borrowAsset])
                .dividedBy(new bignumber_js_1.default(10).pow(this.tokenManager.getTokenDecimals(current.borrowAsset)))), new bignumber_js_1.default(0));
            const lenderTotalCreditPerStrategy = yield this.transformToLenderTotalCreditPerStrategy(data);
            const lenderTotalCreditPerToken = yield this.transformToLenderTotalCreditPerToken(data);
            return {
                lenderTotalCredit: { amount: { value: lenderTotalCredit.toString(), decimals: 0 } },
                lenderTotalCreditPerStrategy,
                lenderTotalCreditCommittedPerToken: lenderTotalCreditPerToken,
            };
        });
    }
    transformToLenderTotalCreditPerToken(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const prices = {};
            const allTokens = data.map((a) => a.borrowAsset);
            for (let index = 0; index < allTokens.length; index++) {
                prices[allTokens[index]] = yield this.tokenManager.getPricePerAsset(allTokens[index]);
            }
            const uniqueTokens = allTokens.filter((value, index, self) => self.indexOf(value) === index);
            const lenderTotalCreditPerToken = [];
            for (let index = 0; index < uniqueTokens.length; index++) {
                const element = uniqueTokens[index];
                const requiredEntries = data.filter((a) => a.borrowAsset === element);
                if (requiredEntries.length > 0) {
                    lenderTotalCreditPerToken.push({
                        amount: {
                            value: requiredEntries.reduce((total, current) => total.plus(current.borrowLimit), new bignumber_js_1.default(0)),
                            decimals: this.tokenManager.getTokenDecimals(requiredEntries[0].borrowAsset),
                        },
                        token: yield this.tokenManager.getAssetMeta(requiredEntries[0].borrowAsset),
                    });
                }
            }
            return lenderTotalCreditPerToken;
        });
    }
    transformToLenderTotalCreditPerStrategy(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const strategyAddresses = data.map((a) => a.strategy).filter((value, index, self) => self.indexOf(value) === index);
            const prices = {};
            const allTokens = data.map((a) => a.borrowAsset);
            for (let index = 0; index < allTokens.length; index++) {
                prices[allTokens[index]] = yield this.tokenManager.getPricePerAsset(allTokens[index]);
            }
            const lenderTotalCreditPerStrategy = [];
            for (let index = 0; index < strategyAddresses.length; index++) {
                const element = strategyAddresses[index];
                const requiredEntries = data.filter((a) => a.strategy === element);
                if (requiredEntries.length > 0) {
                    const totalCredit = requiredEntries.reduce((total, current) => total.plus(new bignumber_js_1.default(current.borrowLimit)
                        .multipliedBy(prices[current.borrowAsset])
                        .dividedBy(new bignumber_js_1.default(10).pow(this.tokenManager.getTokenDecimals(current.borrowAsset)))), new bignumber_js_1.default(0));
                    lenderTotalCreditPerStrategy.push({
                        amount: { value: totalCredit.toString(), decimals: 0 },
                        strategy: {
                            address: requiredEntries[0].strategy,
                            type: this.yieldApi.getStrategy(requiredEntries[0].strategy),
                            displayName: this.yieldApi.getStrategyDisplayName(requiredEntries[0].strategy),
                            logo: this.yieldApi.getStrategyLogo(requiredEntries[0].strategy),
                        },
                    });
                }
            }
            return lenderTotalCreditPerStrategy;
        });
    }
    transformToLenderCollectivePerToken(user, lenderCollectivePerTokenPerStrategy) {
        const lenderCollectivePerToken = [];
        const allTokens = lenderCollectivePerTokenPerStrategy
            .map((a) => a.token.address)
            .filter((value, index, self) => self.indexOf(value) === index);
        for (let index = 0; index < allTokens.length; index++) {
            const element = allTokens[index];
            const requiredTokenEntries = lenderCollectivePerTokenPerStrategy.filter((a) => a.token.address === element);
            if (requiredTokenEntries.length > 0) {
                const entry = requiredTokenEntries[0];
                const totalAmountInAllStrategies = requiredTokenEntries.reduce((total, currentValue) => total.plus(currentValue.amountLent.value.toString()), new bignumber_js_1.default(0));
                const totalInterestInAllStrategies = requiredTokenEntries.reduce((total, currentValue) => total.plus(currentValue.interestReceived.value.toString()), new bignumber_js_1.default(0));
                lenderCollectivePerToken.push({
                    id: `${user}#${entry.token.address}`,
                    token: entry.token,
                    amountLent: { value: totalAmountInAllStrategies.toPrecision(), decimals: entry.amountLent.decimals },
                    interestReceived: { value: totalInterestInAllStrategies.toString(), decimals: entry.interestReceived.decimals },
                });
            }
        }
        return lenderCollectivePerToken;
    }
    transformToBorrowerCollectivePerToken(user, borrowerCollectivePerTokenPerStrategy) {
        const borrowerCollectivePerToken = [];
        const allTokens = borrowerCollectivePerTokenPerStrategy
            .map((a) => a.token.address)
            .filter((value, index, self) => self.indexOf(value) === index);
        for (let index = 0; index < allTokens.length; index++) {
            const element = allTokens[index];
            const requiredTokenEntries = borrowerCollectivePerTokenPerStrategy.filter((a) => a.token.address === element);
            if (requiredTokenEntries.length > 0) {
                const entry = requiredTokenEntries[0];
                const totalAmountInAllStrategies = requiredTokenEntries.reduce((total, currentValue) => total.plus(currentValue.amountBorrowed.value.toString()), new bignumber_js_1.default(0));
                const totalRepaidInAllStrategies = requiredTokenEntries.reduce((total, currentValue) => total.plus(currentValue.amountRepaid.value.toString()), new bignumber_js_1.default(0));
                borrowerCollectivePerToken.push({
                    id: `${user}#${entry.token.address}`,
                    token: entry.token,
                    amountBorrowed: { value: totalAmountInAllStrategies.toPrecision(), decimals: entry.amountBorrowed.decimals },
                    amountRepaid: { value: totalRepaidInAllStrategies.toString(), decimals: entry.amountRepaid.decimals },
                });
            }
        }
        return borrowerCollectivePerToken;
    }
    transformToLenderCollectivePerStrategy(user, lenderCollectivePerTokenPerStrategy) {
        const lenderCollectivePerStrategy = [];
        const allStrategies = lenderCollectivePerTokenPerStrategy
            .map((a) => a.strategy.type)
            .filter((value, index, self) => self.indexOf(value) === index);
        for (let index = 0; index < allStrategies.length; index++) {
            const element = allStrategies[index];
            const requiredTokenEntries = lenderCollectivePerTokenPerStrategy.filter((a) => a.strategy.type === element);
            if (requiredTokenEntries.length > 0) {
                const entry = requiredTokenEntries[0];
                const totalTokenInAll = requiredTokenEntries.reduce((total, currentValue) => total.plus(new bignumber_js_1.default(currentValue.amountLent.value.toString())
                    .multipliedBy(currentValue.token.pricePerAssetInUSD)
                    .dividedBy(new bignumber_js_1.default(10).pow(currentValue.amountLent.decimals))), new bignumber_js_1.default(0));
                const totalInterestInAll = requiredTokenEntries.reduce((total, currentValue) => total.plus(new bignumber_js_1.default(currentValue.interestReceived.value.toString())
                    .multipliedBy(currentValue.token.pricePerAssetInUSD)
                    .dividedBy(new bignumber_js_1.default(10).pow(currentValue.interestReceived.decimals))), new bignumber_js_1.default(0));
                lenderCollectivePerStrategy.push({
                    id: `${user}#${entry.strategy.address}`,
                    strategy: entry.strategy,
                    amountLent: { value: totalTokenInAll.toString(), decimals: 0 },
                    interestReceived: { value: totalInterestInAll.toString(), decimals: 0 },
                });
            }
        }
        return lenderCollectivePerStrategy;
    }
    transformToBorrowerCollectivePerStrategy(user, borrowerCollectivePerTokenPerStrategy) {
        const borrowerCollectivePerStrategy = [];
        const allStrategies = borrowerCollectivePerTokenPerStrategy
            .map((a) => a.strategy.type)
            .filter((value, index, self) => self.indexOf(value) === index);
        for (let index = 0; index < allStrategies.length; index++) {
            const element = allStrategies[index];
            const requiredTokenEntries = borrowerCollectivePerTokenPerStrategy.filter((a) => a.strategy.type === element);
            if (requiredTokenEntries.length > 0) {
                const entry = requiredTokenEntries[0];
                const totalTokenInAll = requiredTokenEntries.reduce((total, currentValue) => total.plus(new bignumber_js_1.default(currentValue.amountBorrowed.value.toString())
                    .multipliedBy(currentValue.token.pricePerAssetInUSD)
                    .dividedBy(new bignumber_js_1.default(10).pow(currentValue.amountBorrowed.decimals))), new bignumber_js_1.default(0));
                const totalRepaidInAll = requiredTokenEntries.reduce((total, currentValue) => total.plus(new bignumber_js_1.default(currentValue.amountRepaid.value.toString())
                    .multipliedBy(currentValue.token.pricePerAssetInUSD)
                    .dividedBy(new bignumber_js_1.default(10).pow(currentValue.amountRepaid.decimals))), new bignumber_js_1.default(0));
                borrowerCollectivePerStrategy.push({
                    id: `${user}#${entry.strategy.address}`,
                    strategy: entry.strategy,
                    amountBorrowed: { value: totalTokenInAll.toString(), decimals: 0 },
                    amountRepaid: { value: totalRepaidInAll.toString(), decimals: 0 },
                });
            }
        }
        return borrowerCollectivePerStrategy;
    }
    TransformToLenderCollective(user, lenderCollectivePerTokenPerStrategy) {
        const totallTokenInAll = lenderCollectivePerTokenPerStrategy.reduce((total, current) => total.plus(new bignumber_js_1.default(current.amountLent.value.toString())
            .multipliedBy(current.token.pricePerAssetInUSD)
            .dividedBy(new bignumber_js_1.default(10).pow(current.amountLent.decimals))), new bignumber_js_1.default(0));
        const totalInterestInAll = lenderCollectivePerTokenPerStrategy.reduce((total, current) => total.plus(new bignumber_js_1.default(current.interestReceived.value.toString())
            .multipliedBy(current.token.pricePerAssetInUSD)
            .dividedBy(new bignumber_js_1.default(10).pow(current.interestReceived.decimals))), new bignumber_js_1.default(0));
        return {
            id: user,
            amountLent: { value: totallTokenInAll.toString(), decimals: 0 },
            interestReceived: { value: totalInterestInAll.toString(), decimals: 0 },
        };
    }
    TransformToBorrowerCollective(user, borrowerCollectivePerTokenPerStrategy) {
        const totallTokenInAll = borrowerCollectivePerTokenPerStrategy.reduce((total, current) => total
            .plus(current.amountBorrowed.value.toString())
            .multipliedBy(current.token.pricePerAssetInUSD)
            .dividedBy(new bignumber_js_1.default(10).pow(current.amountRepaid.decimals)), new bignumber_js_1.default(0));
        const totalRepaidInAll = borrowerCollectivePerTokenPerStrategy.reduce((total, current) => total.plus(new bignumber_js_1.default(current.amountBorrowed.value.toString())
            .multipliedBy(current.token.pricePerAssetInUSD)
            .dividedBy(new bignumber_js_1.default(10).pow(current.amountRepaid.decimals))), new bignumber_js_1.default(0));
        return {
            id: user,
            amountBorrowed: { value: totallTokenInAll.toString(), decimals: 0 },
            amountRepaid: { value: totalRepaidInAll.toString(), decimals: 0 },
        };
    }
    tranformToLenderCollectivePerTokenPerStrategy(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const tokens = data.map((a) => a.token);
            const prices = {};
            for (let index = 0; index < tokens.length; index++) {
                const element = tokens[index];
                yield this.tokenManager.updateAll(element);
                prices[element] = yield this.tokenManager.getPricePerAsset(element);
            }
            return data.map((a) => {
                return {
                    id: a.id,
                    token: {
                        name: this.tokenManager.getTokenName(a.token),
                        pricePerAssetInUSD: prices[a.token],
                        logo: this.tokenManager.getLogo(a.token),
                        address: a.token,
                        decimals: this.tokenManager.getTokenDecimals(a.token),
                    },
                    strategy: {
                        type: this.yieldApi.getStrategy(a.strategy),
                        address: a.strategy,
                        displayName: this.yieldApi.getStrategyDisplayName(a.strategy),
                        logo: this.yieldApi.getStrategyLogo(a.strategy),
                    },
                    amountLent: { value: a.amountLent, decimals: this.tokenManager.getTokenDecimals(a.token) },
                    interestReceived: { value: a.interestReceived, decimals: this.tokenManager.getTokenDecimals(a.token) },
                };
            });
        });
    }
    tranformToBorrowerCollectivePerTokenPerStrategy(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const tokens = data.map((a) => a.token);
            const prices = {};
            for (let index = 0; index < tokens.length; index++) {
                const element = tokens[index];
                yield this.tokenManager.updateAll(element);
                prices[element] = yield this.tokenManager.getPricePerAsset(element);
            }
            return data.map((a) => {
                console.log(a);
                return {
                    id: a.id,
                    token: {
                        name: this.tokenManager.getTokenName(a.token),
                        pricePerAssetInUSD: prices[a.token],
                        logo: this.tokenManager.getLogo(a.token),
                        address: a.token,
                        decimals: this.tokenManager.getTokenDecimals(a.token),
                    },
                    strategy: {
                        type: this.yieldApi.getStrategy(a.strategy),
                        address: a.strategy,
                        displayName: this.yieldApi.getStrategy(a.strategy),
                        logo: this.yieldApi.getStrategyLogo(a.strategy),
                    },
                    amountBorrowed: { value: a.amountBorrowed, decimals: this.tokenManager.getTokenDecimals(a.token) },
                    amountRepaid: { value: a.amountRepaid, decimals: this.tokenManager.getTokenDecimals(a.token) },
                };
            });
        });
    }
    transformToBorrowerAvailableCredit(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const prices = {};
            const allTokens = data.map((a) => a.emulator.borrowAsset());
            for (let index = 0; index < allTokens.length; index++) {
                prices[allTokens[index]] = yield this.tokenManager.getPricePerAsset(allTokens[index]);
            }
            const total = data.reduce((total, current) => total.plus(new bignumber_js_1.default(current.emulator.getCreditLimit())
                .minus(current.emulator.getPrincipal())
                .multipliedBy(prices[current.emulator.borrowAsset()])
                .dividedBy(new bignumber_js_1.default(10).pow(this.tokenManager.getTokenDecimals(current.emulator.borrowAsset())))), new bignumber_js_1.default(0));
            return {
                amount: { value: total.toString(), decimals: 0 },
                borrowerAvailableCreditPerToken: yield this.transformToBorrowerAvailableCreditPerToken(data),
            };
        });
    }
    transformToBorrowerAvailableCreditPerToken(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const allTokens = data.map((a) => a.emulator.borrowAsset());
            const uniqueTokens = allTokens.filter((value, index, self) => self.indexOf(value) === index);
            const borrowerAvailableCreditPerToken = [];
            for (let index = 0; index < uniqueTokens.length; index++) {
                const element = uniqueTokens[index];
                const requiredEntries = data.filter((a) => a.emulator.borrowAsset() === element);
                if (requiredEntries.length > 0) {
                    borrowerAvailableCreditPerToken.push({
                        amount: {
                            value: requiredEntries
                                .reduce((total, current) => total.plus(new bignumber_js_1.default(current.emulator.getCreditLimit()).minus(current.emulator.getPrincipal())), new bignumber_js_1.default(0))
                                .toString(),
                            decimals: this.tokenManager.getTokenDecimals(requiredEntries[0].emulator.borrowAsset()),
                        },
                        token: yield this.tokenManager.getAssetMeta(requiredEntries[0].emulator.borrowAsset()),
                    });
                }
            }
            return borrowerAvailableCreditPerToken;
        });
    }
}
exports.CreditLinesOverviewCall = CreditLinesOverviewCall;
//# sourceMappingURL=CreditLineOverview.js.map