"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PooledCreditLineCalls = void 0;
const CreditLine_1 = require("./CreditLine");
const queries_1 = require("../queries");
const Types_1 = require("../types/Types");
const bignumber_js_1 = __importDefault(require("bignumber.js"));
const PooledCreditLines_1 = require("../emulator/PooledCreditLines");
class PooledCreditLineCalls extends CreditLine_1.CreditLineCalls {
    constructor(url, signer, tokenManager, config) {
        super(url, signer, tokenManager, config);
    }
    /**
     * @description Count all the pooled credit lines
     * @returns
     */
    countAllPooledCreditLines() {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getAllPooledCreditLinesForCount)(this.subgraphUrl);
            return result.length;
        });
    }
    /**
     * @description Count all pooled credit lines of a borrower
     * @param borrower
     * @returns
     */
    countAllPooledCreditLinesOfBorrower(borrower) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getAllPooledCreditLinesOfBorrower)(this.subgraphUrl, borrower);
            return result.length;
        });
    }
    /**
     * @description Count all Pooled Credit Lines of a lender
     * @param lender
     * @returns
     */
    countAllPooledCreditLinesOfLender(lender) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getAllPooledCreditLinesOfLender)(this.subgraphUrl, lender);
            return result.length;
        });
    }
    /**
     * @description Returns pooled credit ordered by latest created
     * @param count
     * @param skip
     * @returns
     */
    getAllPooledCreditLines(count = 13, skip = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getAllPooledCreditLines)(this.subgraphUrl, count, skip);
            const lpData = result.map((a) => a.lenderPool);
            const pclData = result;
            const prices = yield this.refreshTokenData(pclData);
            const collateralPerStrategyToken = yield this.refreshStrategyDataForPcl(pclData);
            const lendersPerPoolData = yield (0, queries_1.getAllLendersPerPool)(this.subgraphUrl, pclData.map((a) => a.id));
            const allPoolIds = pclData.map((a) => a.id);
            const lenderPerPoolData = [];
            for (let index = 0; index < allPoolIds.length; index++) {
                const poolId = allPoolIds[index];
                const requiredLendersPerPool = lendersPerPoolData.filter((a) => a.lenderPool.id == poolId);
                const requiredLenderPerPool = requiredLendersPerPool.map((a) => {
                    return {
                        lenderAddress: a.lenderAddress,
                        amountLent: new bignumber_js_1.default(a.amountLent),
                        amountWithdrawn: new bignumber_js_1.default(a.amountWithdrawn),
                        sharesWithdrawn: new bignumber_js_1.default(a.sharesWithdrawn),
                        interestWithdrawn: new bignumber_js_1.default(a.interestWithdrawn),
                        lenderBalance: new bignumber_js_1.default(a.lenderBalance),
                        borrowerInterestSharesWithdrawn: new bignumber_js_1.default(a.borrowerInterestSharesWithdrawn),
                        yieldInterestWithdrawnShares: new bignumber_js_1.default(a.yieldInterestWithdrawnShares),
                    };
                });
                lenderPerPoolData.push(requiredLenderPerPool);
            }
            const emulatorResult = yield this.transformToPooledCreditLineEmulator(pclData, lpData, lenderPerPoolData, prices, collateralPerStrategyToken);
            return this.transformToPooledCreditLine(emulatorResult, prices);
        });
    }
    /**
     * @description Returns all PCL which have state
     * @param state
     * @param count
     * @param skip
     * @returns
     */
    getAllPoolCreditLinesWithStateIn(state, count = 13, skip = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getAllPooledCreditLinesWithState)(this.subgraphUrl, count, skip, state.map((a) => a.toString()));
            const lpData = result.map((a) => a.lenderPool);
            const pclData = result;
            const prices = yield this.refreshTokenData(pclData);
            const collateralPerStrategyToken = yield this.refreshStrategyDataForPcl(pclData);
            const lendersPerPoolData = yield (0, queries_1.getAllLendersPerPool)(this.subgraphUrl, pclData.map((a) => a.id));
            const allPoolIds = pclData.map((a) => a.id);
            const lenderPerPoolData = [];
            for (let index = 0; index < allPoolIds.length; index++) {
                const poolId = allPoolIds[index];
                const requiredLendersPerPool = lendersPerPoolData.filter((a) => a.lenderPool.id == poolId);
                const requiredLenderPerPool = requiredLendersPerPool.map((a) => {
                    return {
                        lenderAddress: a.lenderAddress,
                        amountLent: new bignumber_js_1.default(a.amountLent),
                        amountWithdrawn: new bignumber_js_1.default(a.amountWithdrawn),
                        sharesWithdrawn: new bignumber_js_1.default(a.sharesWithdrawn),
                        interestWithdrawn: new bignumber_js_1.default(a.interestWithdrawn),
                        lenderBalance: new bignumber_js_1.default(a.lenderBalance),
                        borrowerInterestSharesWithdrawn: new bignumber_js_1.default(a.borrowerInterestSharesWithdrawn),
                        yieldInterestWithdrawnShares: new bignumber_js_1.default(a.yieldInterestWithdrawnShares),
                    };
                });
                lenderPerPoolData.push(requiredLenderPerPool);
            }
            const emulatorResult = yield this.transformToPooledCreditLineEmulator(pclData, lpData, lenderPerPoolData, prices, collateralPerStrategyToken);
            return this.transformToPooledCreditLine(emulatorResult, prices);
        });
    }
    /**
     * @description Returns all PCL which do not have state
     * @param state
     * @param count
     * @param skip
     * @returns
     */
    getAllPoolCreditLinesWithStateNotIn(state, count = 13, skip = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getAllPooledCreditLinesWithNotState)(this.subgraphUrl, count, skip, state.map((a) => a.toString()));
            const lpData = result.map((a) => a.lenderPool);
            const pclData = result;
            const prices = yield this.refreshTokenData(pclData);
            const collateralPerStrategyToken = yield this.refreshStrategyDataForPcl(pclData);
            const lendersPerPoolData = yield (0, queries_1.getAllLendersPerPool)(this.subgraphUrl, pclData.map((a) => a.id));
            const allPoolIds = pclData.map((a) => a.id);
            const lenderPerPoolData = [];
            for (let index = 0; index < allPoolIds.length; index++) {
                const poolId = allPoolIds[index];
                const requiredLendersPerPool = lendersPerPoolData.filter((a) => a.lenderPool.id == poolId);
                const requiredLenderPerPool = requiredLendersPerPool.map((a) => {
                    return {
                        lenderAddress: a.lenderAddress,
                        amountLent: new bignumber_js_1.default(a.amountLent),
                        amountWithdrawn: new bignumber_js_1.default(a.amountWithdrawn),
                        sharesWithdrawn: new bignumber_js_1.default(a.sharesWithdrawn),
                        interestWithdrawn: new bignumber_js_1.default(a.interestWithdrawn),
                        lenderBalance: new bignumber_js_1.default(a.lenderBalance),
                        borrowerInterestSharesWithdrawn: new bignumber_js_1.default(a.borrowerInterestSharesWithdrawn),
                        yieldInterestWithdrawnShares: new bignumber_js_1.default(a.yieldInterestWithdrawnShares),
                    };
                });
                lenderPerPoolData.push(requiredLenderPerPool);
            }
            const emulatorResult = yield this.transformToPooledCreditLineEmulator(pclData, lpData, lenderPerPoolData, prices, collateralPerStrategyToken);
            return this.transformToPooledCreditLine(emulatorResult, prices);
        });
    }
    /**
     * @description Returns specific PCL details
     * @param id
     * @returns
     */
    getPooledCreditLineById(id) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getPooledCreditLineById)(this.subgraphUrl, new bignumber_js_1.default(id).toNumber());
            const lpData = result.map((a) => a.lenderPool);
            const pclData = result;
            const prices = yield this.refreshTokenData(pclData);
            const collateralPerStrategyToken = yield this.refreshStrategyDataForPcl(pclData);
            const lendersPerPoolData = yield (0, queries_1.getAllLendersPerPool)(this.subgraphUrl, pclData.map((a) => a.id));
            const allPoolIds = pclData.map((a) => a.id);
            const lenderPerPoolData = [];
            for (let index = 0; index < allPoolIds.length; index++) {
                const poolId = allPoolIds[index];
                const requiredLendersPerPool = lendersPerPoolData.filter((a) => a.lenderPool.id == poolId);
                const requiredLenderPerPool = requiredLendersPerPool.map((a) => {
                    return {
                        lenderAddress: a.lenderAddress,
                        amountLent: new bignumber_js_1.default(a.amountLent),
                        amountWithdrawn: new bignumber_js_1.default(a.amountWithdrawn),
                        sharesWithdrawn: new bignumber_js_1.default(a.sharesWithdrawn),
                        interestWithdrawn: new bignumber_js_1.default(a.interestWithdrawn),
                        lenderBalance: new bignumber_js_1.default(a.lenderBalance),
                        borrowerInterestSharesWithdrawn: new bignumber_js_1.default(a.borrowerInterestSharesWithdrawn),
                        yieldInterestWithdrawnShares: new bignumber_js_1.default(a.yieldInterestWithdrawnShares),
                    };
                });
                lenderPerPoolData.push(requiredLenderPerPool);
            }
            const emulatorResult = yield this.transformToPooledCreditLineEmulator(pclData, lpData, lenderPerPoolData, prices, collateralPerStrategyToken);
            return this.transformToPooledCreditLine(emulatorResult, prices);
        });
    }
    /**
     * @description Returns all PCL of borrower
     * @param address
     * @param count
     * @param skip
     * @returns
     */
    getAllPooledCreditLinesOfBorrower(address, count = 13, skip = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getPooledCreditLinesOfBorrower)(this.subgraphUrl, address, count, skip);
            const lpData = result.map((a) => a.lenderPool);
            const pclData = result;
            const prices = yield this.refreshTokenData(pclData);
            const collateralPerStrategyToken = yield this.refreshStrategyDataForPcl(pclData);
            const lendersPerPoolData = yield (0, queries_1.getAllLendersPerPool)(this.subgraphUrl, pclData.map((a) => a.id));
            const allPoolIds = pclData.map((a) => a.id);
            const lenderPerPoolData = [];
            for (let index = 0; index < allPoolIds.length; index++) {
                const poolId = allPoolIds[index];
                const requiredLendersPerPool = lendersPerPoolData.filter((a) => a.lenderPool.id == poolId);
                const requiredLenderPerPool = requiredLendersPerPool.map((a) => {
                    return {
                        lenderAddress: a.lenderAddress,
                        amountLent: new bignumber_js_1.default(a.amountLent),
                        amountWithdrawn: new bignumber_js_1.default(a.amountWithdrawn),
                        sharesWithdrawn: new bignumber_js_1.default(a.sharesWithdrawn),
                        interestWithdrawn: new bignumber_js_1.default(a.interestWithdrawn),
                        lenderBalance: new bignumber_js_1.default(a.lenderBalance),
                        borrowerInterestSharesWithdrawn: new bignumber_js_1.default(a.borrowerInterestSharesWithdrawn),
                        yieldInterestWithdrawnShares: new bignumber_js_1.default(a.yieldInterestWithdrawnShares),
                    };
                });
                lenderPerPoolData.push(requiredLenderPerPool);
            }
            const emulatorResult = yield this.transformToPooledCreditLineEmulator(pclData, lpData, lenderPerPoolData, prices, collateralPerStrategyToken);
            const lines = this.transformToPooledCreditLine(emulatorResult, prices);
            return lines.sort((a, b) => new bignumber_js_1.default(b.id).minus(a.id).toNumber());
        });
    }
    /**
     * @description Returns all the PCL of borrower with state in
     * @param borrower
     * @param status
     * @param count
     * @param skip
     * @returns
     */
    getAllPooledCreditLinesOfBorrowerWithStateIn(borrower, status, count = 13, skip = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getPooledCreditLinesOfBorrowerWithState)(this.subgraphUrl, borrower, count, skip, status.map((a) => a.toString()));
            const lpData = result.map((a) => a.lenderPool);
            const pclData = result;
            const prices = yield this.refreshTokenData(pclData);
            const collateralPerStrategyToken = yield this.refreshStrategyDataForPcl(pclData);
            const lendersPerPoolData = yield (0, queries_1.getAllLendersPerPool)(this.subgraphUrl, pclData.map((a) => a.id));
            const allPoolIds = pclData.map((a) => a.id);
            const lenderPerPoolData = [];
            for (let index = 0; index < allPoolIds.length; index++) {
                const poolId = allPoolIds[index];
                const requiredLendersPerPool = lendersPerPoolData.filter((a) => a.lenderPool.id == poolId);
                const requiredLenderPerPool = requiredLendersPerPool.map((a) => {
                    return {
                        lenderAddress: a.lenderAddress,
                        amountLent: new bignumber_js_1.default(a.amountLent),
                        amountWithdrawn: new bignumber_js_1.default(a.amountWithdrawn),
                        sharesWithdrawn: new bignumber_js_1.default(a.sharesWithdrawn),
                        interestWithdrawn: new bignumber_js_1.default(a.interestWithdrawn),
                        lenderBalance: new bignumber_js_1.default(a.lenderBalance),
                        borrowerInterestSharesWithdrawn: new bignumber_js_1.default(a.borrowerInterestSharesWithdrawn),
                        yieldInterestWithdrawnShares: new bignumber_js_1.default(a.yieldInterestWithdrawnShares),
                    };
                });
                lenderPerPoolData.push(requiredLenderPerPool);
            }
            const emulatorResult = yield this.transformToPooledCreditLineEmulator(pclData, lpData, lenderPerPoolData, prices, collateralPerStrategyToken);
            return this.transformToPooledCreditLine(emulatorResult, prices);
        });
    }
    /**
     * @description Returns all the PCL of borrower with state not in
     * @param borrower
     * @param status
     * @param count
     * @param skip
     * @returns
     */
    getAllPooledCreditLinesOfBorrowerWithStateNotIn(borrower, status, count = 13, skip = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getPooledCreditLinesOfBorrowerWithNotState)(this.subgraphUrl, borrower, count, skip, status.map((a) => a.toString()));
            const lpData = result.map((a) => a.lenderPool);
            const pclData = result;
            const prices = yield this.refreshTokenData(pclData);
            const collateralPerStrategyToken = yield this.refreshStrategyDataForPcl(pclData);
            const lendersPerPoolData = yield (0, queries_1.getAllLendersPerPool)(this.subgraphUrl, pclData.map((a) => a.id));
            const allPoolIds = pclData.map((a) => a.id);
            const lenderPerPoolData = [];
            for (let index = 0; index < allPoolIds.length; index++) {
                const poolId = allPoolIds[index];
                const requiredLendersPerPool = lendersPerPoolData.filter((a) => a.lenderPool.id == poolId);
                const requiredLenderPerPool = requiredLendersPerPool.map((a) => {
                    return {
                        lenderAddress: a.lenderAddress,
                        amountLent: new bignumber_js_1.default(a.amountLent),
                        amountWithdrawn: new bignumber_js_1.default(a.amountWithdrawn),
                        sharesWithdrawn: new bignumber_js_1.default(a.sharesWithdrawn),
                        interestWithdrawn: new bignumber_js_1.default(a.interestWithdrawn),
                        lenderBalance: new bignumber_js_1.default(a.lenderBalance),
                        borrowerInterestSharesWithdrawn: new bignumber_js_1.default(a.borrowerInterestSharesWithdrawn),
                        yieldInterestWithdrawnShares: new bignumber_js_1.default(a.yieldInterestWithdrawnShares),
                    };
                });
                lenderPerPoolData.push(requiredLenderPerPool);
            }
            const emulatorResult = yield this.transformToPooledCreditLineEmulator(pclData, lpData, lenderPerPoolData, prices, collateralPerStrategyToken);
            return this.transformToPooledCreditLine(emulatorResult, prices);
        });
    }
    /**
     * @description Returns all the PCL which a lender can lend to
     * @param lender
     * @param count
     * @param skip
     * @returns
     */
    getAllPooledCreditLinesLenderCanLendTo(lender, count = 13, skip = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getPooledCreditLinesOfLenderCanLendTo)(this.subgraphUrl, lender, count, skip);
            const lpData = result.map((a) => a.lenderPool);
            const pclData = result;
            const prices = yield this.refreshTokenData(pclData);
            const collateralPerStrategyToken = yield this.refreshStrategyDataForPcl(pclData);
            const lendersPerPoolData = yield (0, queries_1.getAllLendersPerPool)(this.subgraphUrl, pclData.map((a) => a.id));
            const allPoolIds = pclData.map((a) => a.id);
            const lenderPerPoolData = [];
            for (let index = 0; index < allPoolIds.length; index++) {
                const poolId = allPoolIds[index];
                const requiredLendersPerPool = lendersPerPoolData.filter((a) => a.lenderPool.id == poolId);
                const requiredLenderPerPool = requiredLendersPerPool.map((a) => {
                    return {
                        lenderAddress: a.lenderAddress,
                        amountLent: new bignumber_js_1.default(a.amountLent),
                        amountWithdrawn: new bignumber_js_1.default(a.amountWithdrawn),
                        sharesWithdrawn: new bignumber_js_1.default(a.sharesWithdrawn),
                        interestWithdrawn: new bignumber_js_1.default(a.interestWithdrawn),
                        lenderBalance: new bignumber_js_1.default(a.lenderBalance),
                        borrowerInterestSharesWithdrawn: new bignumber_js_1.default(a.borrowerInterestSharesWithdrawn),
                        yieldInterestWithdrawnShares: new bignumber_js_1.default(a.yieldInterestWithdrawnShares),
                    };
                });
                lenderPerPoolData.push(requiredLenderPerPool);
            }
            const emulatorResult = yield this.transformToPooledCreditLineEmulator(pclData, lpData, lenderPerPoolData, prices, collateralPerStrategyToken);
            return this.transformToPooledCreditLine(emulatorResult, prices);
        });
    }
    /**
     * @description Get All pooled credit lines of lender
     * @param lender
     * @param count
     * @param skip
     * @returns
     */
    getAllPooledCreditLinesOfLender(lender, count = 13, skip = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            const [result, contributionsData] = yield (0, queries_1.getPooledCreditLinesOfLender)(this.subgraphUrl, lender, count, skip);
            const lpData = result.map((a) => a.lenderPool);
            const pclData = result;
            const prices = yield this.refreshTokenData(pclData);
            const collateralPerStrategyToken = yield this.refreshStrategyDataForPcl(pclData);
            const lendersPerPoolData = yield (0, queries_1.getAllLendersPerPool)(this.subgraphUrl, pclData.map((a) => a.id));
            const allPoolIds = pclData.map((a) => a.id);
            const lenderPerPoolData = [];
            for (let index = 0; index < allPoolIds.length; index++) {
                const poolId = allPoolIds[index];
                const requiredLendersPerPool = lendersPerPoolData.filter((a) => a.lenderPool.id == poolId);
                const requiredLenderPerPool = requiredLendersPerPool.map((a) => {
                    return {
                        lenderAddress: a.lenderAddress,
                        amountLent: new bignumber_js_1.default(a.amountLent),
                        amountWithdrawn: new bignumber_js_1.default(a.amountWithdrawn),
                        sharesWithdrawn: new bignumber_js_1.default(a.sharesWithdrawn),
                        interestWithdrawn: new bignumber_js_1.default(a.interestWithdrawn),
                        lenderBalance: new bignumber_js_1.default(a.lenderBalance),
                        borrowerInterestSharesWithdrawn: new bignumber_js_1.default(a.borrowerInterestSharesWithdrawn),
                        yieldInterestWithdrawnShares: new bignumber_js_1.default(a.yieldInterestWithdrawnShares),
                    };
                });
                lenderPerPoolData.push(requiredLenderPerPool);
            }
            const emulatorResult = yield this.transformToPooledCreditLineEmulator(pclData, lpData, lenderPerPoolData, prices, collateralPerStrategyToken);
            let pooledCreditLines = this.transformToPooledCreditLine(emulatorResult, prices);
            let contributions = yield this.transformToLenderContributionToPooledCreditLines(pooledCreditLines, contributionsData);
            pooledCreditLines = pooledCreditLines.sort((a, b) => new bignumber_js_1.default(b.id).minus(a.id).toNumber());
            contributions = contributions.sort((a, b) => new bignumber_js_1.default(b.id).minus(a.id).toNumber());
            return [pooledCreditLines, contributions];
        });
    }
    getSpecificPooledCreditLineOfLender(lender, id) {
        return __awaiter(this, void 0, void 0, function* () {
            const [poolData, contributionData] = yield (0, queries_1.getPooledCreditLinesForLenderById)(this.subgraphUrl, lender, id);
            if (poolData.length == 0) {
                return [undefined, undefined];
            }
            const lpData = poolData.map((a) => a.lenderPool);
            const pclData = poolData;
            const prices = yield this.refreshTokenData(pclData);
            const collateralPerStrategyToken = yield this.refreshStrategyDataForPcl(pclData);
            const lendersPerPoolData = yield (0, queries_1.getAllLendersPerPool)(this.subgraphUrl, pclData.map((a) => a.id));
            const allPoolIds = pclData.map((a) => a.id);
            const lenderPerPoolData = [];
            for (let index = 0; index < allPoolIds.length; index++) {
                const poolId = allPoolIds[index];
                const requiredLendersPerPool = lendersPerPoolData.filter((a) => a.lenderPool.id == poolId);
                const requiredLenderPerPool = requiredLendersPerPool.map((a) => {
                    return {
                        lenderAddress: a.lenderAddress,
                        amountLent: new bignumber_js_1.default(a.amountLent),
                        amountWithdrawn: new bignumber_js_1.default(a.amountWithdrawn),
                        sharesWithdrawn: new bignumber_js_1.default(a.sharesWithdrawn),
                        interestWithdrawn: new bignumber_js_1.default(a.interestWithdrawn),
                        lenderBalance: new bignumber_js_1.default(a.lenderBalance),
                        borrowerInterestSharesWithdrawn: new bignumber_js_1.default(a.borrowerInterestSharesWithdrawn),
                        yieldInterestWithdrawnShares: new bignumber_js_1.default(a.yieldInterestWithdrawnShares),
                    };
                });
                lenderPerPoolData.push(requiredLenderPerPool);
            }
            const emulatorResult = yield this.transformToPooledCreditLineEmulator(pclData, lpData, lenderPerPoolData, prices, collateralPerStrategyToken);
            const poolDataDetail = this.transformToPooledCreditLine(emulatorResult, prices);
            const lenderContribution = yield this.transformToLenderContributionToPooledCreditLines(poolDataDetail, contributionData);
            return [poolDataDetail[0], lenderContribution[0]];
        });
    }
    /**
     * @deprecated Will be removed in future
     * @param user
     * @returns
     */
    getAllPooledCreditLinesOfUser(user) {
        return __awaiter(this, void 0, void 0, function* () {
            const borrowerLines = yield this.getAllPooledCreditLinesOfBorrower(user);
            const [lenderLines] = yield this.getAllPooledCreditLinesOfLender(user);
            let lines = [...lenderLines, ...borrowerLines];
            lines = lines.sort((a, b) => new bignumber_js_1.default(b.id).minus(a.id).toNumber());
            return lines;
        });
    }
    getAllLendersPerPool(pclIds) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getAllLendersPerPool)(this.subgraphUrl, pclIds);
            return result;
        });
    }
    /**
     * @description Fetch PCL timeline
     * @param pooledCreditLineId
     * @param count
     * @param skip
     * @returns
     */
    getPooledCreditLineTimeline(pooledCreditLineId, count = 13, skip = 0) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getPooledCreditLineTimeline)(this.subgraphUrl, pooledCreditLineId, count, skip);
            const cl = yield this.getPooledCreditLineById(parseInt(pooledCreditLineId));
            if (cl.length > 0) {
                const borrowTokenDecimal = this.tokenManager.getTokenDecimals(cl[0].borrowAsset.address);
                const collateralTokenDecimal = this.tokenManager.getTokenDecimals(cl[0].collateralAsset.address);
                return this.transformToPooledCreditLineOperation(result, borrowTokenDecimal, collateralTokenDecimal);
            }
            else {
                return [];
            }
        });
    }
    transformToPooledCreditLineOperation(data, borrowTokenDecimal, collateralTokenDecimal) {
        return __awaiter(this, void 0, void 0, function* () {
            return data.map((a) => {
                // NOT_CREATED
                // REQUESTED
                // ACTIVE
                // DEPOSIT_COLLATERAL
                // BORROW
                // REPAY
                // WITHDRAW_COLLATERAL
                // CLOSED
                // EXPIRED
                // CANCELLED
                // RESET
                // LIQUIDATED
                // LEND
                // WITHDRAW_INTEREST
                // WITHDRAW_LIQUIDATY
                // WITHDRAW_COLLATERAL_LIQUIDITY
                let amount = { value: '0', decimals: 18 };
                if (['DEPOSIT_COLLATERAL', 'WITHDRAW_COLLATERAL', 'WITHDRAW_INTEREST', 'WITHDRAW_COLLATERAL_LIQUIDITY'].includes(a.pooledCreditLineOperation)) {
                    amount = { value: a.amount, decimals: collateralTokenDecimal };
                }
                if (['BORROW', 'REPAY', 'LEND', 'WITHDRAW_LIQUIDATY'].includes(a.pooledCreditLineOperation)) {
                    amount = { value: a.amount, decimals: borrowTokenDecimal };
                }
                return {
                    id: a.id,
                    transactionHash: String(a.id).split('#')[0],
                    eventIndex: String(a.id).split('#')[1],
                    pooledCreditLineOperation: a.pooledCreditLineOperation,
                    timestamp: a.timestamp,
                    amount,
                    strategy: a.strategy
                        ? {
                            type: this.yieldApi.getStrategy(a.strategy),
                            address: a.strategy,
                            displayName: this.yieldApi.getStrategyDisplayName(a.strategy),
                            logo: this.yieldApi.getStrategyLogo(a.strategy),
                        }
                        : undefined,
                };
            });
        });
    }
    /**
     * @description Get Lenders Per Pool
     * @todo Add pagination to LendersPerPool query
     * @param id
     * @returns
     */
    getLendersPerPool(id) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getLenderPerPool)(this.subgraphUrl, id);
            if (result.length == 0) {
                throw new Error('No such pooled credit line');
            }
            const returnData = yield this.transformToLenderPoolDetail(result);
            return returnData[0];
        });
    }
    transformToLenderPoolDetail(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const borrowTokens = data.map((a) => a.collateralAsset);
            const collateralTokens = data.map((a) => a.borrowAsset);
            const allTokens = [...borrowTokens, ...collateralTokens].filter((value, index, array) => array.indexOf(value) === index);
            for (let index = 0; index < allTokens.length; index++) {
                const element = allTokens[index];
                yield this.tokenManager.updateAll(element);
            }
            const prices = {};
            for (let index = 0; index < allTokens.length; index++) {
                prices[allTokens[index]] = yield this.tokenManager.getPricePerAsset(allTokens[index]);
            }
            return data.map((a) => {
                return {
                    id: a.id,
                    startTime: a.startTime,
                    borrowAsset: {
                        name: this.tokenManager.getTokenName(a.borrowAsset),
                        address: a.borrowAsset,
                        logo: this.tokenManager.getLogo(a.borrowAsset),
                        pricePerAssetInUSD: prices[a.borrowAsset],
                        decimals: this.tokenManager.getTokenDecimals(a.borrowAsset),
                    },
                    collateralAsset: {
                        name: this.tokenManager.getTokenName(a.collateralAsset),
                        address: a.collateralAsset,
                        logo: this.tokenManager.getLogo(a.collateralAsset),
                        pricePerAssetInUSD: prices[a.collateralAsset],
                        decimals: this.tokenManager.getTokenDecimals(a.collateralAsset),
                    },
                    borrowLimit: { value: a.borrowLimit, decimals: this.tokenManager.getTokenDecimals(a.borrowAsset) },
                    minBorrowAmount: { value: a.minBorrowAmount, decimals: this.tokenManager.getTokenDecimals(a.borrowAsset) },
                    borrowAssetStrategy: {
                        address: a.borrowAssetStrategy,
                        type: this.yieldApi.getStrategy(a.borrowAssetStrategy),
                        displayName: this.yieldApi.getStrategyDisplayName(a.borrowAssetStrategy),
                        logo: this.yieldApi.getStrategyLogo(a.borrowAssetStrategy),
                    },
                    sharesHeld: { value: a.sharesHeld, decimals: this.tokenManager.getTokenDecimals(a.collateralAsset) },
                    borrowerInterestShares: { value: a.borrowerInterestShares, decimals: this.tokenManager.getTokenDecimals(a.collateralAsset) },
                    yieldInterestWithdrawnShares: {
                        value: a.yieldInterestWithdrawnShares,
                        decimals: this.tokenManager.getTokenDecimals(a.collateralAsset),
                    },
                    collateralHeld: { value: a.collateralHeld, decimals: this.tokenManager.getTokenDecimals(a.collateralAsset) },
                    areTokensTransferable: a.areTokensTransferable,
                    verifier: {
                        type: this.verificationApi.getVerifierType(a.verifier.id),
                        address: a.verifier.id,
                        displayName: this.verificationApi.getVerifierDisplayName(a.verifier.id),
                    },
                    lenders: this.transformToLenderPerPoolDetail(a.lender, this.tokenManager.getTokenDecimals(a.borrowAsset)),
                };
            });
        });
    }
    transformToLenderPerPoolDetail(data, collateralDecimal) {
        return data.map((a) => {
            return {
                lenderAddress: a.lenderAddress,
                amountLent: { value: a.lenderBalance, decimals: collateralDecimal },
                amountWithdrawn: { value: a.amountWithdrawn, decimals: collateralDecimal },
                sharesWithdrawn: { value: a.sharesWithdrawn, decimals: collateralDecimal },
                strategy: {
                    address: a.strategy,
                    type: this.yieldApi.getStrategy(a.strategy),
                    displayName: this.yieldApi.getStrategy(a.strategy),
                    logo: this.yieldApi.getStrategyLogo(a.strategy),
                },
            };
        });
    }
    /**
     * @description Count all PCL with states in
     * @param state
     * @returns
     */
    countAllPooledCreditLinesWithStates(state) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getAllPooledCreditLinesForCountWithState)(this.subgraphUrl, state.map((a) => a.toString()));
            return result.length;
        });
    }
    /**
     * @description Count all PCL with states not in
     * @param state
     * @returns
     */
    countAllPooledCreditLinesWithStatesNotIn(state) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getAllPooledCreditLinesForCountWithStateNotIn)(this.subgraphUrl, state.map((a) => a.toString()));
            return result.length;
        });
    }
    /**
     * @description Count all PCL of borrower with state in
     * @param borrower
     * @param state
     * @returns
     */
    countAllPooledCreditLinesOfBorrowerWithState(borrower, state) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getAllPooledCreditLinesOfBorrowerWithState)(this.subgraphUrl, borrower, state.map((a) => a.toString()));
            return result.length;
        });
    }
    /**
     * @description Count all PCL of borrower with state not in
     * @param borrower
     * @param state
     * @returns
     */
    countAllPooledCreditLinesOfBorrowerWithStateNotIn(borrower, state) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getAllPooledCreditLinesOfBorrowerWithStateNotIn)(this.subgraphUrl, borrower, state.map((a) => a.toString()));
            return result.length;
        });
    }
    /**
     * @description Count all PCL of lender with state in
     * @param borrower
     * @param state
     * @returns
     */
    countAllPooledCreditLinesOfLenderWithState(lender, state) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getAllPooledCreditLinesOfLenderWithState)(this.subgraphUrl, lender, state.map((a) => a.toString()));
            return result.length;
        });
    }
    /**
     * @description Count all PCL of lender with state not in
     * @param borrower
     * @param state
     * @returns
     */
    countAllPooledCreditLinesOfLenderWithStateNotIn(lender, state) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getAllPooledCreditLinesOfLenderWithStateNotIn)(this.subgraphUrl, lender, state.map((a) => a.toString()));
            return result.length;
        });
    }
    /**
     * @description Count all Lenders of PooledCreditLine
     * @param id
     * @returns
     */
    countAllLendersOfPooledCreditLine(id) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getLendersOfPooledCreditLines)(this.subgraphUrl, id.toString());
            return result.length;
        });
    }
    transformToPooledCreditLineEmulator(pclData, lenderPoolData, lendersPerPool, prices, collateralPerStrategyToken) {
        return pclData.map((a, index) => {
            const aLenderPool = lenderPoolData[index];
            const lenderPerPool = lendersPerPool[index];
            return new PooledCreditLines_1.PooledCreditLineEmulator({
                id: a.id,
                pooledCreditLineStatus: this.convertToPooledCreditLineStatusEnum(a.status),
                endsAt: new bignumber_js_1.default(a.endsAt),
                principal: new bignumber_js_1.default(a.principal),
                idealCollateralRatio: new bignumber_js_1.default(a.idealCollateralRatio),
                totalInterestRepaid: new bignumber_js_1.default(a.totalInterestRepaid),
                lastPrincipalUpdateTime: new bignumber_js_1.default(a.lastPrincipalUpdateTime),
                gracePenaltyRate: new bignumber_js_1.default(a.gracePenaltyRate),
                borrowRate: new bignumber_js_1.default(a.borrowRate),
                interestAccruedTillLastPrincipalUpdate: new bignumber_js_1.default(a.interestAccruedTillLastPrincipalUpdate),
                depositedCollateralInShares: new bignumber_js_1.default(a.depositedCollateralInShares),
                borrowLimit: new bignumber_js_1.default(a.borrowLimit),
                defaultsAt: new bignumber_js_1.default(a.defaultsAt),
                borrowerAddress: a.borrowerAddress,
                borrowAsset: a.borrowAsset,
                collateralAsset: a.collateralAsset,
                createdAt: new bignumber_js_1.default(a.createdAt),
                startsAt: new bignumber_js_1.default(a.startsAt),
                lenderStrategy: a.lenderStrategy,
                collateralStrategy: a.collateralStrategy,
                totalLentAmount: new bignumber_js_1.default(a.totalLentAmount),
                minBorrowAmount: new bignumber_js_1.default(a.minBorrowAmount),
                lenderVerifier: a.lenderVerifier.id,
                borrowerVerifier: a.borrowerVerifier.id,
            }, {
                collateralPerStrategyToken: new bignumber_js_1.default(collateralPerStrategyToken[a.lenderStrategy][a.collateralAsset]),
                ratioOfPrices: new bignumber_js_1.default(prices[a.collateralAsset]).dividedBy(prices[a.borrowAsset]),
                ratioOfPricesDecimals: 0,
            }, {
                id: aLenderPool.id,
                sharesHeld: new bignumber_js_1.default(aLenderPool.sharesHeld),
                borrowerInterestShares: new bignumber_js_1.default(aLenderPool.borrowerInterestShares),
                borrowerInterestSharesWithdrawn: new bignumber_js_1.default(aLenderPool.borrowerInterestShares),
                yieldInterestWithdrawnShares: new bignumber_js_1.default(aLenderPool.yieldInterestWithdrawnShares),
                borrowLimit: new bignumber_js_1.default(aLenderPool.borrowLimit),
                startTime: new bignumber_js_1.default(aLenderPool.startTime),
                minBorrowAmount: new bignumber_js_1.default(aLenderPool.minBorrowAmount),
            }, lenderPerPool);
        });
    }
    transformToPooledCreditLine(emulatorResult, prices) {
        return emulatorResult.map((aNew) => {
            return {
                id: aNew.getId(),
                borrowerAddress: aNew.borrowerAddress(),
                borrowLimit: { value: aNew.getBorrowLimit().toString(), decimals: this.tokenManager.getTokenDecimals(aNew.borrowAsset()) },
                borrowRate: { value: aNew.borrowRate().toString(), decimals: 18 },
                idealCollateralRatio: { value: aNew.idealCollateralratio().toString(), decimals: 18 },
                collateralTokens: {
                    value: aNew.calculateTotalCollateralTokens().toString(),
                    decimals: this.tokenManager.getTokenDecimals(aNew.collateralAsset()),
                },
                borrowAsset: {
                    name: this.tokenManager.getTokenName(aNew.borrowAsset()),
                    address: aNew.borrowAsset(),
                    logo: this.tokenManager.getLogo(aNew.borrowAsset()),
                    pricePerAssetInUSD: prices[aNew.borrowAsset()].toString(),
                    decimals: this.tokenManager.getTokenDecimals(aNew.borrowAsset()),
                },
                collateralAsset: {
                    name: this.tokenManager.getTokenName(aNew.collateralAsset()),
                    address: aNew.collateralAsset(),
                    logo: this.tokenManager.getLogo(aNew.collateralAsset()),
                    pricePerAssetInUSD: prices[aNew.collateralAsset()].toString(),
                    decimals: this.tokenManager.getTokenDecimals(aNew.collateralAsset()),
                },
                createdAt: aNew.createdAt().toString(),
                startsAt: aNew.startsAt().toString(),
                endsAt: aNew.endsAt().toString(),
                defaultsAt: aNew.defaultsAt().toString(),
                lenderStrategy: {
                    type: this.yieldApi.getStrategy(aNew.lenderStrategy()),
                    address: aNew.lenderStrategy(),
                    displayName: this.yieldApi.getStrategyDisplayName(aNew.lenderStrategy()),
                    logo: this.yieldApi.getStrategyLogo(aNew.lenderStrategy()),
                },
                collateralStrategy: {
                    type: this.yieldApi.getStrategy(aNew.collateralStrategy()),
                    address: aNew.collateralStrategy(),
                    displayName: this.yieldApi.getStrategyDisplayName(aNew.collateralStrategy()),
                    logo: this.yieldApi.getStrategyLogo(aNew.collateralStrategy()),
                },
                gracePenaltyRate: { value: aNew.gracePenaltyRate().toString(), decimals: 18 },
                status: aNew.getStatus(),
                principal: { value: aNew.getPrincipal().toString(), decimals: this.tokenManager.getTokenDecimals(aNew.borrowAsset()) },
                totalInterestRepaid: {
                    value: aNew.totalInterestRepaid().toString(),
                    decimals: this.tokenManager.getTokenDecimals(aNew.borrowAsset()),
                },
                lastPrincipalUpdateTime: aNew.lastPrincipalUpdateTime().toString(),
                interestAccruedTillLastPrincipalUpdate: {
                    value: aNew.interestAccruedTillLastPrincipalUpdate().toString(),
                    decimals: this.tokenManager.getTokenDecimals(aNew.borrowAsset()),
                },
                totalLentAmount: { value: aNew.totalAmountLent().toString(), decimals: this.tokenManager.getTokenDecimals(aNew.borrowAsset()) },
                currentCollateralRatio: { value: aNew.calculateCurrentCollateralRatio().toString(), decimals: 18 },
                currentDebt: { value: aNew.calculateCurrentDebt().toString(), decimals: this.tokenManager.getTokenDecimals(aNew.borrowAsset()) },
                minBorrowAmount: { value: aNew.minBorrowAmount().toString(), decimals: this.tokenManager.getTokenDecimals(aNew.borrowAsset()) },
                emulator: aNew,
            };
        });
    }
    transformToLenderContributionToPooledCreditLines(pooledCreditLines, data) {
        return __awaiter(this, void 0, void 0, function* () {
            return data.map((a, index) => {
                const borrowToken = pooledCreditLines[index].borrowAsset;
                const collateralToken = pooledCreditLines[index].collateralAsset;
                const borrowDecimals = this.tokenManager.getTokenDecimals(borrowToken.address);
                const collateralDecimals = this.tokenManager.getTokenDecimals(collateralToken.address);
                return {
                    id: pooledCreditLines[index].id,
                    amountLent: { value: a.lenderBalance || '0', decimals: borrowDecimals },
                    amountWithdrawn: { value: a.amountWithdrawn || '0', decimals: borrowDecimals },
                    sharesWithdrawn: { value: a.sharesWithdrawn || '0', decimals: collateralDecimals },
                    interestWithdrawn: { value: a.interestWithdrawn || '0', decimals: borrowDecimals },
                };
            });
        });
    }
    convertToPooledCreditLineStatusEnum(stateInSubgraph) {
        //   enum PooledCreditLineStatus {
        //     NOT_CREATED,
        //     REQUESTED,
        //     ACTIVE,
        //     CLOSED,
        //     EXPIRED,
        //     LIQUIDATED,
        //     CANCELLED
        // }
        if (stateInSubgraph == 'NOT_CREATED') {
            return Types_1.CreditLineStatus.NOT_CREATED;
        }
        else if (stateInSubgraph == 'REQUESTED') {
            return Types_1.CreditLineStatus.REQUESTED;
        }
        else if (stateInSubgraph == 'ACTIVE') {
            return Types_1.CreditLineStatus.ACTIVE;
        }
        else if (stateInSubgraph == 'CLOSED') {
            return Types_1.CreditLineStatus.CLOSED;
        }
        else if (stateInSubgraph == 'EXPIRED') {
            return Types_1.CreditLineStatus.EXPIRED;
        }
        else if (stateInSubgraph == 'LIQUIDATED') {
            return Types_1.CreditLineStatus.LIQUIDATED;
        }
        else if (stateInSubgraph == 'CANCELLED') {
            return Types_1.CreditLineStatus.CANCELLED;
        }
        else {
            return Types_1.CreditLineStatus.NOT_CREATED;
        }
    }
}
exports.PooledCreditLineCalls = PooledCreditLineCalls;
//# sourceMappingURL=PooledCreditLine.js.map