"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PooledCreditLinesBorrowerOverviewCall = void 0;
const CreditLineOverview_1 = require("./CreditLineOverview");
const queries_1 = require("../queries");
const Types_1 = require("../types/Types");
const bignumber_js_1 = __importDefault(require("bignumber.js"));
class PooledCreditLinesBorrowerOverviewCall extends CreditLineOverview_1.CreditLinesOverviewCall {
    constructor(url, signer, tokenManager, config) {
        super(url, signer, tokenManager, config);
    }
    getPooledCreditLineCollectiveOfBorrower(borrower) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getBorrowerPooledCreditLineCollective)(this.subgraphUrl, borrower);
            return this.transformToPooledCreditLineBorrowerData(result);
        });
    }
    getLatestActionablePooledCreditLinesOfBorrower(borrower) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getAllPooledCreditLinesOfBorrowerWithState)(this.subgraphUrl, borrower, [
                Types_1.CreditLineStatus.ACTIVE,
                Types_1.CreditLineStatus.REQUESTED,
            ]);
            const now = new bignumber_js_1.default(new Date().valueOf()).div(1000);
            const gtEndedAtAndLtDefaultsAt = result
                .filter((a) => now.gte(a.endsAt) && now.lt(a.defaultsAt))
                .map((a) => {
                return Object.assign(Object.assign({}, a), { upcomingAction: Types_1.PclUpcomingState.ABOUT_TO_DEFAULT, timeRemaining: new bignumber_js_1.default(a.defaultsAt).minus(now).toFixed() });
            });
            const ltEndedAt = result
                .filter((a) => now.lt(a.endsAt))
                .map((a) => {
                return Object.assign(Object.assign({}, a), { upcomingAction: Types_1.PclUpcomingState.ABOUT_TO_EXPIRE, timeRemaining: new bignumber_js_1.default(a.endsAt).minus(now).toFixed() });
            });
            const actionablePcls = [...gtEndedAtAndLtDefaultsAt, ...ltEndedAt].sort((a, b) => new bignumber_js_1.default(a.timeRemaining).minus(b.timeRemaining).toNumber());
            return actionablePcls;
        });
    }
    transformToPooledCreditLineBorrowerData(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const allTokens = data.map((a) => a.token);
            const uniqueTokens = allTokens.filter((value, index, self) => self.indexOf(value) === index);
            for (let index = 0; index < uniqueTokens.length; index++) {
                const element = uniqueTokens[index];
                yield this.tokenManager.updateAll(element);
            }
            const pooledCreditLineBorrowerCollectivePerTokenPerStrategy = yield this.transformToPooledCreditLineBorrowerCollectivePerTokenPerStrategy(data);
            const pooledCreditLineBorrowerCollectivePerToken = yield this.transformToPooledCreditLineBorrowerCollectivePerToken(pooledCreditLineBorrowerCollectivePerTokenPerStrategy);
            const pooledCreditLineBorrowerCollectivePerStrategy = yield this.transformToPooledCreditLineBorrowerCollectivePerStrategy(pooledCreditLineBorrowerCollectivePerTokenPerStrategy);
            return {
                pooledCreditLineBorrowerCollective: yield this.transformToPooledCreditLineBorrowerCollective(pooledCreditLineBorrowerCollectivePerTokenPerStrategy),
                pooledCreditLineBorrowerCollectivePerStrategy,
                pooledCreditLineBorrowerCollectivePerToken,
                pooledCreditLineBorrowerCollectivePerTokenPerStrategy,
            };
        });
    }
    transformToPooledCreditLineBorrowerCollectivePerStrategy(pooledCreditLineBorrowerCollectivePerTokenPerStrategy) {
        return __awaiter(this, void 0, void 0, function* () {
            const prices = {};
            const allTokens = pooledCreditLineBorrowerCollectivePerTokenPerStrategy
                .map((a) => a.token.address)
                .filter((value, index, self) => self.indexOf(value) === index);
            for (let index = 0; index < allTokens.length; index++) {
                prices[allTokens[index]] = yield this.tokenManager.getPricePerAsset(allTokens[index]);
            }
            const allStrategies = pooledCreditLineBorrowerCollectivePerTokenPerStrategy
                .map((a) => a.strategy.type)
                .filter((value, index, self) => self.indexOf(value) === index);
            const pooledCreditLineBorrowerCollectivePerStrategy = [];
            for (let index = 0; index < allStrategies.length; index++) {
                const element = allStrategies[index];
                const requiredElements = pooledCreditLineBorrowerCollectivePerTokenPerStrategy.filter((a) => a.strategy.type === element);
                if (requiredElements.length > 0) {
                    const totalCollteralDeposited = pooledCreditLineBorrowerCollectivePerTokenPerStrategy.reduce((total, current) => total.plus(new bignumber_js_1.default(current.collateralDeposited.value.toString())
                        .multipliedBy(prices[current.token.address])
                        .dividedBy(new bignumber_js_1.default(10).pow(this.tokenManager.getTokenDecimals(current.token.address)))), new bignumber_js_1.default(0));
                    const totalAmountRepaid = pooledCreditLineBorrowerCollectivePerTokenPerStrategy.reduce((total, current) => total.plus(new bignumber_js_1.default(current.amountRepaid.value.toString())
                        .multipliedBy(prices[current.token.address])
                        .dividedBy(new bignumber_js_1.default(10).pow(this.tokenManager.getTokenDecimals(current.token.address)))), new bignumber_js_1.default(0));
                    const totalAmountBorrowed = pooledCreditLineBorrowerCollectivePerTokenPerStrategy.reduce((total, current) => total.plus(new bignumber_js_1.default(current.amountBorrowed.value.toString())
                        .multipliedBy(prices[current.token.address])
                        .dividedBy(new bignumber_js_1.default(10).pow(this.tokenManager.getTokenDecimals(current.token.address)))), new bignumber_js_1.default(0));
                    pooledCreditLineBorrowerCollectivePerStrategy.push({
                        id: 'to-do',
                        strategy: requiredElements[0].strategy,
                        collateralDeposited: { value: totalCollteralDeposited.toString(), decimals: 0 },
                        amountBorrowed: { value: totalAmountBorrowed.toString(), decimals: 0 },
                        amountRepaid: { value: totalAmountRepaid.toString(), decimals: 0 },
                    });
                }
            }
            return pooledCreditLineBorrowerCollectivePerStrategy;
        });
    }
    transformToPooledCreditLineBorrowerCollectivePerToken(pooledCreditLineBorrowerCollectivePerTokenPerStrategy) {
        return __awaiter(this, void 0, void 0, function* () {
            const allTokens = pooledCreditLineBorrowerCollectivePerTokenPerStrategy.map((a) => a.token.address);
            const uniqueTokens = allTokens.filter((value, index, self) => self.indexOf(value) === index);
            const result = [];
            for (let index = 0; index < uniqueTokens.length; index++) {
                const element = uniqueTokens[index];
                const requiredElements = pooledCreditLineBorrowerCollectivePerTokenPerStrategy.filter((a) => a.token.address === element);
                if (requiredElements.length > 0) {
                    const totalCollteralDeposited = requiredElements.reduce((total, current) => total.plus(new bignumber_js_1.default(current.collateralDeposited.value.toString())), new bignumber_js_1.default(0));
                    const totalAmountRepaid = requiredElements.reduce((total, current) => total.plus(new bignumber_js_1.default(current.amountRepaid.value.toString())), new bignumber_js_1.default(0));
                    const totalAmountBorrowed = requiredElements.reduce((total, current) => total.plus(new bignumber_js_1.default(current.amountBorrowed.value.toString())), new bignumber_js_1.default(0));
                    result.push({
                        id: 'to-do',
                        token: requiredElements[0].token,
                        collateralDeposited: {
                            value: totalCollteralDeposited.toString(),
                            decimals: this.tokenManager.getTokenDecimals(requiredElements[0].token.address),
                        },
                        amountBorrowed: {
                            value: totalAmountBorrowed.toString(),
                            decimals: this.tokenManager.getTokenDecimals(requiredElements[0].token.address),
                        },
                        amountRepaid: {
                            value: totalAmountRepaid.toString(),
                            decimals: this.tokenManager.getTokenDecimals(requiredElements[0].token.address),
                        },
                    });
                }
            }
            return result;
        });
    }
    transformToPooledCreditLineBorrowerCollective(pooledCreditLineBorrowerCollectivePerTokenPerStrategy) {
        return __awaiter(this, void 0, void 0, function* () {
            const prices = {};
            const allTokens = pooledCreditLineBorrowerCollectivePerTokenPerStrategy.map((a) => a.token.address);
            for (let index = 0; index < allTokens.length; index++) {
                prices[allTokens[index]] = yield this.tokenManager.getPricePerAsset(allTokens[index]);
            }
            const totalCollteralDeposited = pooledCreditLineBorrowerCollectivePerTokenPerStrategy.reduce((total, current) => total.plus(new bignumber_js_1.default(current.collateralDeposited.value.toString())
                .multipliedBy(prices[current.token.address])
                .dividedBy(new bignumber_js_1.default(10).pow(this.tokenManager.getTokenDecimals(current.token.address)))), new bignumber_js_1.default(0));
            const totalAmountRepaid = pooledCreditLineBorrowerCollectivePerTokenPerStrategy.reduce((total, current) => total.plus(new bignumber_js_1.default(current.amountRepaid.value.toString())
                .multipliedBy(prices[current.token.address])
                .dividedBy(new bignumber_js_1.default(10).pow(this.tokenManager.getTokenDecimals(current.token.address)))), new bignumber_js_1.default(0));
            const totalAmountBorrowed = pooledCreditLineBorrowerCollectivePerTokenPerStrategy.reduce((total, current) => total.plus(new bignumber_js_1.default(current.amountBorrowed.value.toString())
                .multipliedBy(prices[current.token.address])
                .dividedBy(new bignumber_js_1.default(10).pow(this.tokenManager.getTokenDecimals(current.token.address)))), new bignumber_js_1.default(0));
            return {
                collateralDeposited: { value: totalCollteralDeposited.toString(), decimals: 0 },
                amountBorrowed: { value: totalAmountBorrowed.toString(), decimals: 0 },
                amountRepaid: { value: totalAmountRepaid.toString(), decimals: 0 },
            };
        });
    }
    transformToPooledCreditLineBorrowerCollectivePerTokenPerStrategy(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const all = data.map((a) => __awaiter(this, void 0, void 0, function* () {
                return {
                    id: a.id,
                    token: {
                        name: this.tokenManager.getTokenName(a.token),
                        address: a.token,
                        logo: this.tokenManager.getLogo(a.token),
                        pricePerAssetInUSD: (yield this.tokenManager.getPricePerAsset(a.token)).toString(),
                        decimals: this.tokenManager.getTokenDecimals(a.token),
                    },
                    strategy: {
                        type: this.yieldApi.getStrategy(a.strategy),
                        address: a.strategy,
                        displayName: this.yieldApi.getStrategyDisplayName(a.strategy),
                        logo: this.yieldApi.getStrategyLogo(a.strategy),
                    },
                    collateralDeposited: { value: a.collateralDeposited, decimals: this.tokenManager.getTokenDecimals(a.token) },
                    amountRepaid: { value: a.amountRepaid, decimals: this.tokenManager.getTokenDecimals(a.token) },
                    amountBorrowed: { value: a.amountBorrowed, decimals: this.tokenManager.getTokenDecimals(a.token) },
                };
            }));
            return Promise.all(all);
        });
    }
}
exports.PooledCreditLinesBorrowerOverviewCall = PooledCreditLinesBorrowerOverviewCall;
//# sourceMappingURL=PooledCreditLineBorrowerOverview.js.map