"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PooledCreditLineInterestCalculator = void 0;
const CreditLineInterestCalculator_1 = require("./CreditLineInterestCalculator");
const getPooledCreditLines_1 = require("../queries/getPooledCreditLines");
const Types_1 = require("../types/Types");
const bignumber_js_1 = __importDefault(require("bignumber.js"));
const exponent = new bignumber_js_1.default(10).pow(18);
const oneYear = new bignumber_js_1.default(365).multipliedBy(86400);
class PooledCreditLineInterestCalculator extends CreditLineInterestCalculator_1.CreditLineInterestCalculator {
    constructor(url, signer, tokenManager, config) {
        super(url, signer, tokenManager, config);
    }
    getBorrowerInterestDueForAllPooledCreditLines(borrower) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = [];
            const count = 999;
            let skip = 0;
            for (;;) {
                const _temp = yield (0, getPooledCreditLines_1.getPooledCreditLinesOfBorrowerWithState)(this.subgraphUrl, borrower, count, skip, [Types_1.CreditLineStatus.ACTIVE, Types_1.CreditLineStatus.REQUESTED].map((a) => a.toString()));
                skip += count;
                if (_temp.length == 0) {
                    break;
                }
                else {
                    result.push(..._temp);
                }
            }
            return this.transformToInterestDueDetailsForPcl(result);
        });
    }
    transformToInterestDueDetailsForPcl(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const borrowTokens = data.map((a) => a.borrowAsset);
            const collateralTokens = data.map((a) => a.collateralAsset);
            const allTokens = [...borrowTokens, ...collateralTokens].filter((value, index, self) => self.indexOf(value) === index);
            const prices = {};
            for (let index = 0; index < allTokens.length; index++) {
                const element = allTokens[index];
                yield this.tokenManager.updateAll(element);
                prices[element] = yield this.tokenManager.getPricePerAsset(element);
            }
            const interestDuePerTokenPerStrategy = yield this.transformToInterestDuePerTokenPerStrategyForPcl(data, prices);
            const interestDuePerStrategy = yield this.transformToInterestDuePerStrategyForPcl(interestDuePerTokenPerStrategy, prices);
            const interestDuePerToken = yield this.transformToInterestDuePerTokenPcl(interestDuePerTokenPerStrategy);
            const interestDue = yield this.transformToInterestDueForPcl(interestDuePerTokenPerStrategy, prices);
            return { interestDue, interestDuePerStrategy, interestDuePerToken, interestDuePerTokenPerStrategy };
        });
    }
    transformToInterestDuePerStrategyForPcl(interestDuePerTokenPerStrategy, prices) {
        return __awaiter(this, void 0, void 0, function* () {
            const interestDuePerStrategy = [];
            const strategies = interestDuePerTokenPerStrategy
                .map((a) => a.strategy.address)
                .filter((value, index, self) => self.indexOf(value) === index);
            for (let index = 0; index < strategies.length; index++) {
                const element = strategies[index];
                const requiredElements = interestDuePerTokenPerStrategy.filter((a) => a.strategy.address === element);
                if (requiredElements.length > 0) {
                    const totalInterest = requiredElements.reduce((total, current) => total.plus(new bignumber_js_1.default(current.interest.value.toString())
                        .multipliedBy(prices[current.token.address])
                        .dividedBy(new bignumber_js_1.default(10).pow(this.tokenManager.getTokenDecimals(current.token.address)))), new bignumber_js_1.default(0));
                    interestDuePerStrategy.push({
                        id: 'to-do',
                        interest: { value: totalInterest.toString(), decimals: 0 },
                        strategy: requiredElements[0].strategy,
                    });
                }
            }
            return interestDuePerStrategy;
        });
    }
    transformToInterestDuePerTokenPcl(interestDuePerTokenPerStrategy) {
        return __awaiter(this, void 0, void 0, function* () {
            const interestDuePerToken = [];
            const uniqueTokens = interestDuePerTokenPerStrategy
                .map((a) => a.token.address)
                .filter((value, index, self) => self.indexOf(value) === index);
            for (let index = 0; index < uniqueTokens.length; index++) {
                const element = uniqueTokens[index];
                const requiredElements = interestDuePerTokenPerStrategy.filter((a) => a.token.address === element);
                if (requiredElements.length > 0) {
                    const totalInterest = requiredElements.reduce((total, current) => total.plus(current.interest.value.toString()), new bignumber_js_1.default(0));
                    interestDuePerToken.push({
                        interest: { value: totalInterest.toString(), decimals: this.tokenManager.getTokenDecimals(requiredElements[0].token.address) },
                        token: requiredElements[0].token,
                        id: 'to-do',
                    });
                }
            }
            return interestDuePerToken;
        });
    }
    transformToInterestDueForPcl(interestDuePerTokenPerStrategy, prices) {
        return __awaiter(this, void 0, void 0, function* () {
            const totalInterest = interestDuePerTokenPerStrategy.reduce((total, current) => total.plus(new bignumber_js_1.default(current.interest.value.toString())
                .multipliedBy(prices[current.token.address])
                .dividedBy(new bignumber_js_1.default(10).pow(this.tokenManager.getTokenDecimals(current.token.address)))), new bignumber_js_1.default(0));
            return { id: 'to-do', interest: { value: totalInterest.toString(), decimals: 0 } };
        });
    }
    transformToInterestDuePerTokenPerStrategyForPcl(data, prices) {
        return __awaiter(this, void 0, void 0, function* () {
            return data.map((a) => {
                return {
                    id: a.id,
                    token: {
                        address: a.borrowAsset,
                        name: this.tokenManager.getTokenName(a.borrowAsset),
                        logo: this.tokenManager.getLogo(a.borrowAsset),
                        pricePerAssetInUSD: prices[a.borrowAsset].toString(),
                        decimals: this.tokenManager.getTokenDecimals(a.borrowAsset),
                    },
                    strategy: {
                        address: a.strategy,
                        type: this.yieldApi.getStrategy(a.strategy),
                        displayName: this.yieldApi.getStrategyDisplayName(a.strategy),
                        logo: this.yieldApi.getStrategyLogo(a.strategy),
                    },
                    interest: {
                        value: this.calculateInterestForPcl(a.principal, a.borrowRate, a.lastPrincipalUpdateTime).toString(),
                        decimals: this.tokenManager.getTokenDecimals(a.borrowAsset),
                    },
                };
            });
        });
    }
    calculateInterestForPcl(principal, borrowRate, lastPrincipalUpdateTime) {
        const now = new bignumber_js_1.default(new Date().valueOf()).div(1000);
        if (now.gt(lastPrincipalUpdateTime)) {
            return new bignumber_js_1.default(principal)
                .multipliedBy(now.minus(lastPrincipalUpdateTime))
                .multipliedBy(borrowRate)
                .dividedBy(exponent)
                .dividedBy(oneYear);
        }
        else {
            return new bignumber_js_1.default(0);
        }
    }
}
exports.PooledCreditLineInterestCalculator = PooledCreditLineInterestCalculator;
//# sourceMappingURL=PooledCreditLineInterestCalculator.js.map