"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PooledCreditLinesLenderOverview = void 0;
const PooledCreditLineBorrowerOverview_1 = require("./PooledCreditLineBorrowerOverview");
const queries_1 = require("../queries");
const bignumber_js_1 = __importDefault(require("bignumber.js"));
class PooledCreditLinesLenderOverview extends PooledCreditLineBorrowerOverview_1.PooledCreditLinesBorrowerOverviewCall {
    constructor(url, signer, tokenManager, config) {
        super(url, signer, tokenManager, config);
    }
    /**
     * @description TotalsharesWithdrawn is pending
     * @param lender
     * @returns
     */
    getPooledCreditLineCollectiveOfLender(lender) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getLenderPooledCreditLineCollective)(this.subgraphUrl, lender);
            return this.transformToPooledCreditLineLenderData(result);
        });
    }
    transformToPooledCreditLineLenderData(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const allTokens = data.map((a) => a.token).filter((value, index, self) => self.indexOf(value) === index);
            for (let index = 0; index < allTokens.length; index++) {
                const element = allTokens[index];
                yield this.tokenManager.updateAll(element);
            }
            const pooledCreditLineLenderCollectivePerTokenPerStrategy = yield this.transformToPooledCreditLineLenderCollectivePerTokenPerStrategy(data);
            const pooledCreditLineLenderCollectivePerToken = yield this.transformToPooledCreditLineLenderCollectivePerToken(pooledCreditLineLenderCollectivePerTokenPerStrategy);
            const pooledCreditLineLenderCollectivePerStrategy = yield this.transformToPooledCreditLineLenderCollectivePerStrategy(pooledCreditLineLenderCollectivePerTokenPerStrategy);
            const pooledCreditLineCollective = yield this.transformToPooledCreditLineCollective(pooledCreditLineLenderCollectivePerTokenPerStrategy);
            return {
                pooledCreditLineCollective,
                pooledCreditLineLenderCollectivePerToken,
                pooledCreditLineLenderCollectivePerStrategy,
                pooledCreditLineLenderCollectivePerTokenPerStrategy,
            };
        });
    }
    transformToPooledCreditLineLenderCollectivePerStrategy(pooledCreditLineLenderCollectivePerTokenPerStrategy) {
        return __awaiter(this, void 0, void 0, function* () {
            const allStrategies = pooledCreditLineLenderCollectivePerTokenPerStrategy
                .map((a) => a.strategy.type)
                .filter((value, index, self) => self.indexOf(value) === index);
            const prices = {};
            const allTokens = pooledCreditLineLenderCollectivePerTokenPerStrategy
                .map((a) => a.token.address)
                .filter((value, index, self) => self.indexOf(value) === index);
            for (let index = 0; index < allTokens.length; index++) {
                prices[allTokens[index]] = yield this.tokenManager.getPricePerAsset(allTokens[index]);
            }
            const pooledCreditLineLenderCollectivePerStrategy = [];
            for (let index = 0; index < allStrategies.length; index++) {
                const element = allStrategies[index];
                const requiredElements = pooledCreditLineLenderCollectivePerTokenPerStrategy.filter((a) => a.strategy.type === element);
                if (requiredElements.length > 0) {
                    const totalAmountLent = requiredElements.reduce((total, current) => total.plus(new bignumber_js_1.default(current.amountLent.value.toString())
                        .multipliedBy(prices[current.token.address])
                        .dividedBy(new bignumber_js_1.default(10).pow(this.tokenManager.getTokenDecimals(current.token.address)))), new bignumber_js_1.default(0));
                    const totalAmountWithdrawn = requiredElements.reduce((total, current) => total.plus(new bignumber_js_1.default(current.amountWithdrawn.value.toString())
                        .multipliedBy(prices[current.token.address])
                        .dividedBy(new bignumber_js_1.default(10).pow(this.tokenManager.getTokenDecimals(current.token.address)))), new bignumber_js_1.default(0));
                    const totalInterestWithdrawn = requiredElements.reduce((total, current) => total.plus(new bignumber_js_1.default(current.interestWithdrawn.value.toString())
                        .multipliedBy(prices[current.token.address])
                        .dividedBy(new bignumber_js_1.default(10).pow(this.tokenManager.getTokenDecimals(current.token.address)))), new bignumber_js_1.default(0));
                    const totalInterestEarned = requiredElements.reduce((total, current) => total.plus(new bignumber_js_1.default(current.interestEarned.value.toString())
                        .multipliedBy(prices[current.token.address])
                        .dividedBy(new bignumber_js_1.default(10).pow(this.tokenManager.getTokenDecimals(current.token.address)))), new bignumber_js_1.default(0));
                    const totalSharesWithdrawn = requiredElements.reduce((total, current) => total.plus(new bignumber_js_1.default(current.sharesWithdrawn.value.toString())), new bignumber_js_1.default(0));
                    pooledCreditLineLenderCollectivePerStrategy.push({
                        id: 'to-do',
                        strategy: requiredElements[0].strategy,
                        amountLent: { value: totalAmountLent.toString(), decimals: 0 },
                        amountWithdrawn: { value: totalAmountWithdrawn.toString(), decimals: 0 },
                        interestWithdrawn: { value: totalInterestWithdrawn.toString(), decimals: 0 },
                        sharesWithdrawn: { value: totalSharesWithdrawn.toString(), decimals: 18 },
                        interestEarned: { value: totalInterestEarned.toString(), decimals: 0 },
                    });
                }
            }
            return pooledCreditLineLenderCollectivePerStrategy;
        });
    }
    transformToPooledCreditLineLenderCollectivePerToken(pooledCreditLineLenderCollectivePerTokenPerStrategy) {
        return __awaiter(this, void 0, void 0, function* () {
            const allTokens = pooledCreditLineLenderCollectivePerTokenPerStrategy
                .map((a) => a.token.address)
                .filter((value, index, self) => self.indexOf(value) === index);
            const pooledCreditLineLenderCollectivePerToken = [];
            for (let index = 0; index < allTokens.length; index++) {
                const element = allTokens[index];
                const requiredEntries = pooledCreditLineLenderCollectivePerTokenPerStrategy.filter((a) => a.token.address === element);
                if (requiredEntries.length > 0) {
                    const totalAmountLent = requiredEntries.reduce((total, current) => total.plus(new bignumber_js_1.default(current.amountLent.value.toString())), new bignumber_js_1.default(0));
                    const totalAmountWithdrawn = requiredEntries.reduce((total, current) => total.plus(new bignumber_js_1.default(current.amountWithdrawn.value.toString())), new bignumber_js_1.default(0));
                    const totalInterestWithdrawn = requiredEntries.reduce((total, current) => total.plus(new bignumber_js_1.default(current.interestWithdrawn.value.toString())), new bignumber_js_1.default(0));
                    const totalSharesWithdrawn = requiredEntries.reduce((total, current) => total.plus(new bignumber_js_1.default(current.sharesWithdrawn.value.toString())), new bignumber_js_1.default(0));
                    const totalInterestEarned = requiredEntries.reduce((total, current) => total.plus(current.interestEarned.value.toString()), new bignumber_js_1.default(0));
                    pooledCreditLineLenderCollectivePerToken.push({
                        amountLent: { value: totalAmountLent.toString(), decimals: this.tokenManager.getTokenDecimals(requiredEntries[0].token.address) },
                        amountWithdrawn: { value: totalAmountWithdrawn.toString(), decimals: 0 },
                        interestWithdrawn: { value: totalInterestWithdrawn.toString(), decimals: 0 },
                        sharesWithdrawn: { value: totalSharesWithdrawn.toString(), decimals: 18 },
                        id: 'to=do',
                        token: requiredEntries[0].token,
                        interestEarned: {
                            value: totalInterestEarned.toString(),
                            decimals: this.tokenManager.getTokenDecimals(requiredEntries[0].token.address),
                        },
                    });
                }
            }
            return pooledCreditLineLenderCollectivePerToken;
        });
    }
    transformToPooledCreditLineCollective(pooledCreditLineLenderCollectivePerTokenPerStrategy) {
        return __awaiter(this, void 0, void 0, function* () {
            const prices = {};
            const allTokens = pooledCreditLineLenderCollectivePerTokenPerStrategy.map((a) => a.token.address);
            for (let index = 0; index < allTokens.length; index++) {
                prices[allTokens[index]] = yield this.tokenManager.getPricePerAsset(allTokens[index]);
            }
            const totalAmountLent = pooledCreditLineLenderCollectivePerTokenPerStrategy.reduce((total, current) => total.plus(new bignumber_js_1.default(current.amountLent.value.toString())
                .multipliedBy(prices[current.token.address])
                .dividedBy(new bignumber_js_1.default(10).pow(this.tokenManager.getTokenDecimals(current.token.address)))), new bignumber_js_1.default(0));
            const totalAmountWithdrawn = pooledCreditLineLenderCollectivePerTokenPerStrategy.reduce((total, current) => total.plus(new bignumber_js_1.default(current.amountWithdrawn.value.toString())
                .multipliedBy(prices[current.token.address])
                .dividedBy(new bignumber_js_1.default(10).pow(this.tokenManager.getTokenDecimals(current.token.address)))), new bignumber_js_1.default(0));
            const totalInterestWithdrawn = pooledCreditLineLenderCollectivePerTokenPerStrategy.reduce((total, current) => total.plus(new bignumber_js_1.default(current.interestWithdrawn.value.toString())
                .multipliedBy(prices[current.token.address])
                .dividedBy(new bignumber_js_1.default(10).pow(this.tokenManager.getTokenDecimals(current.token.address)))), new bignumber_js_1.default(0));
            const totalSharesWithdrawn = pooledCreditLineLenderCollectivePerTokenPerStrategy.reduce((total, current) => total.plus(new bignumber_js_1.default(current.sharesWithdrawn.value.toString())), new bignumber_js_1.default(0));
            const totalInterestEarned = pooledCreditLineLenderCollectivePerTokenPerStrategy.reduce((total, current) => total.plus(new bignumber_js_1.default(current.interestEarned.value.toString())
                .multipliedBy(prices[current.token.address])
                .dividedBy(new bignumber_js_1.default(10).pow(this.tokenManager.getTokenDecimals(current.token.address)))), new bignumber_js_1.default(0));
            return {
                amountLent: { value: totalAmountLent.toString(), decimals: 0 },
                amountWithdrawn: { value: totalAmountWithdrawn.toString(), decimals: 0 },
                interestWithdrawn: { value: totalInterestWithdrawn.toString(), decimals: 0 },
                sharesWithdrawn: { value: totalSharesWithdrawn.toString(), decimals: 18 },
                interestEarned: { value: totalInterestEarned.toString(), decimals: 0 },
            };
        });
    }
    transformToPooledCreditLineLenderCollectivePerTokenPerStrategy(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const all = data.map((a) => __awaiter(this, void 0, void 0, function* () {
                const pclData = a.pooledCreditLines;
                const emulators = yield this.getPclEmulators(pclData);
                const totalInterest = emulators.reduce((total, pcl) => total.plus(pcl.getLenderPoolEmulator().getLenderInterest(a.user)), new bignumber_js_1.default(0));
                return {
                    id: a.id,
                    token: {
                        address: a.token,
                        name: this.tokenManager.getTokenName(a.token),
                        logo: this.tokenManager.getLogo(a.token),
                        pricePerAssetInUSD: yield (yield this.tokenManager.getPricePerAsset(a.token)).toString(),
                        decimals: this.tokenManager.getTokenDecimals(a.token),
                    },
                    strategy: {
                        type: this.yieldApi.getStrategy(a.strategy),
                        address: a.strategy,
                        displayName: this.yieldApi.getStrategyDisplayName(a.strategy),
                        logo: this.yieldApi.getStrategyLogo(a.strategy),
                    },
                    amountLent: { value: a.amountLent, decimals: this.tokenManager.getTokenDecimals(a.token) },
                    amountWithdrawn: { value: a.amountWithdrawn, decimals: this.tokenManager.getTokenDecimals(a.token) },
                    sharesWithdrawn: { value: a.sharesWithdrawn, decimals: 0 },
                    interestWithdrawn: { value: a.interestWithdrawn, decimals: 0 },
                    interestEarned: { value: totalInterest.toNumber(), decimals: this.tokenManager.getTokenDecimals(a.token) },
                };
            }));
            return Promise.all(all);
        });
    }
    getPclEmulators(result) {
        return __awaiter(this, void 0, void 0, function* () {
            const lpData = result.map((a) => a.lenderPool);
            const pclData = result;
            const prices = yield this.refreshTokenData(pclData);
            const collateralPerStrategyToken = yield this.refreshStrategyDataForPcl(pclData);
            const lendersPerPoolData = yield (0, queries_1.getAllLendersPerPool)(this.subgraphUrl, pclData.map((a) => a.id));
            const allPoolIds = pclData.map((a) => a.id);
            const lenderPerPoolData = [];
            for (let index = 0; index < allPoolIds.length; index++) {
                const poolId = allPoolIds[index];
                const requiredLendersPerPool = lendersPerPoolData.filter((a) => a.lenderPool.id == poolId);
                const requiredLenderPerPool = requiredLendersPerPool.map((a) => {
                    return {
                        lenderAddress: a.lenderAddress,
                        amountLent: new bignumber_js_1.default(a.amountLent),
                        amountWithdrawn: new bignumber_js_1.default(a.amountWithdrawn),
                        sharesWithdrawn: new bignumber_js_1.default(a.sharesWithdrawn),
                        interestWithdrawn: new bignumber_js_1.default(a.interestWithdrawn),
                        lenderBalance: new bignumber_js_1.default(a.lenderBalance),
                        borrowerInterestSharesWithdrawn: new bignumber_js_1.default(a.borrowerInterestSharesWithdrawn),
                        yieldInterestWithdrawnShares: new bignumber_js_1.default(a.yieldInterestWithdrawnShares),
                    };
                });
                lenderPerPoolData.push(requiredLenderPerPool);
            }
            const emulatorResult = yield this.transformToPooledCreditLineEmulator(pclData, lpData, lenderPerPoolData, prices, collateralPerStrategyToken);
            return emulatorResult;
        });
    }
}
exports.PooledCreditLinesLenderOverview = PooledCreditLinesLenderOverview;
//# sourceMappingURL=PooledCreditLinesLenderOverview.js.map