import { Signer } from '@ethersproject/abstract-signer';
import { TokenManager } from '../tokenManager';
import { SublimeConfig } from '../types/sublimeConfig';
import { PooledCreditLineCalls } from './PooledCreditLine';
import { PoolDetail, PoolLender, ProfileOverview } from '../types/Types';
export declare class PoolCalls extends PooledCreditLineCalls {
    constructor(url: string, signer: Signer, tokenManager: TokenManager, config: SublimeConfig);
    /**
     *
     * @returns Array of all pools created on sublime
     */
    getPools(): Promise<PoolDetail[]>;
    /**
     *
     * @param poolType (possible types of pools include: ACTIVE, CLOSED, REQUESTED, LIQUIDATED)
     * @returns Array of specific pool types
     */
    getAllPoolsByPoolType(poolType: string): Promise<PoolDetail[]>;
    /**
     * @param poolId
     * @returns pool data if the pool exists, else null
     */
    getPool(poolId: string): Promise<PoolDetail>;
    /**
     * @param borrower: address of the borrower
     * @returns Array of pools for the corresponding borrower
     */
    getPoolByBorrower(borrower: string): Promise<PoolDetail[]>;
    /**
     * @param lender: address of the lender
     * @returns Array of pool for the lender
     */
    getPoolByLender(lender: string): Promise<PoolDetail[]>;
    /**
     * @param lender: address of the lender
     * @param poolType: type of the pool
     * @returns Array of pool
     */
    getPoolByLenderByType(lender: string, poolType: string): Promise<PoolDetail[]>;
    /**
     * @param borrower: address of the borrower
     * @param poolType: type of the pool
     * @returns Array of pool
     */
    getPoolByBorrowerByType(borrower: string, poolType: string): Promise<PoolDetail[]>;
    /**
     * @param poolAddress
     * @description To-Do
     * @returns All the lenders of a given pool
     */
    getAllLendersOfPool(): Promise<PoolLender[]>;
    /**
     * @param data
     * @@description transforms the data received from the subgraph to pool detail
     */
    private transformToPoolDetail;
    /**
     * @param address: address of the user to query
     * @description Return the profiel overview of the user
     */
    getProfileOverview(address: string): Promise<ProfileOverview>;
}
