"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PoolCalls = void 0;
const PooledCreditLine_1 = require("./PooledCreditLine");
const bignumber_js_1 = require("bignumber.js");
const sha2_1 = require("@ethersproject/sha2");
const queries_1 = require("../queries");
class PoolCalls extends PooledCreditLine_1.PooledCreditLineCalls {
    constructor(url, signer, tokenManager, config) {
        super(url, signer, tokenManager, config);
    }
    /**
     *
     * @returns Array of all pools created on sublime
     */
    getPools() {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getAllPools)(this.subgraphUrl);
            return this.transformToPoolDetail(result);
        });
    }
    /**
     *
     * @param poolType (possible types of pools include: ACTIVE, CLOSED, REQUESTED, LIQUIDATED)
     * @returns Array of specific pool types
     */
    getAllPoolsByPoolType(poolType) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getAllPoolsByPoolType)(this.subgraphUrl, poolType);
            return this.transformToPoolDetail(result);
        });
    }
    /**
     * @param poolId
     * @returns pool data if the pool exists, else null
     */
    getPool(poolId) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getPool)(this.subgraphUrl, poolId);
            const poolDetails = yield this.transformToPoolDetail(result);
            return poolDetails[0];
        });
    }
    /**
     * @param borrower: address of the borrower
     * @returns Array of pools for the corresponding borrower
     */
    getPoolByBorrower(borrower) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getAllPoolsByBorrower)(this.subgraphUrl, borrower);
            return this.transformToPoolDetail(result);
        });
    }
    /**
     * @param lender: address of the lender
     * @returns Array of pool for the lender
     */
    getPoolByLender(lender) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getAllPoolsByLender)(this.subgraphUrl, lender);
            return this.transformToPoolDetail(result);
        });
    }
    /**
     * @param lender: address of the lender
     * @param poolType: type of the pool
     * @returns Array of pool
     */
    getPoolByLenderByType(lender, poolType) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getAllPoolsByLenderByType)(this.subgraphUrl, lender, poolType);
            return this.transformToPoolDetail(result);
        });
    }
    /**
     * @param borrower: address of the borrower
     * @param poolType: type of the pool
     * @returns Array of pool
     */
    getPoolByBorrowerByType(borrower, poolType) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getAllPoolsByBorrowerByType)(this.subgraphUrl, borrower, poolType);
            return this.transformToPoolDetail(result);
        });
    }
    /**
     * @param poolAddress
     * @description To-Do
     * @returns All the lenders of a given pool
     */
    getAllLendersOfPool() {
        return __awaiter(this, void 0, void 0, function* () {
            const totalNumberOfLenders = 1 + this.getRandomInt(100);
            const lenders = [];
            const date = new Date().valueOf();
            const poolToken = (0, sha2_1.sha256)(Buffer.from(date + 'poolToken')).slice(0, 40);
            const suppliedToken = (0, sha2_1.sha256)(Buffer.from(date + 'suppliedToken')).slice(0, 40);
            for (let index = 0; index < totalNumberOfLenders; index++) {
                lenders.push({
                    address: (0, sha2_1.sha256)(Buffer.from(date + '' + index)).slice(0, 40),
                    shareInPool: new bignumber_js_1.BigNumber(this.getRandomInt(3000)).div(100).toFixed(2),
                    suppliedAmount: { value: new bignumber_js_1.BigNumber(this.getRandomInt(100000000)).div(100).toFixed(2), decimals: 2 },
                    poolToken: {
                        address: poolToken,
                        name: 'Pending...',
                        pricePerAssetInUSD: '00.00',
                        logo: 'Logo pending ...',
                        decimals: 18,
                    },
                    suppliedToken: {
                        address: suppliedToken,
                        name: 'Pending...',
                        pricePerAssetInUSD: '00.00',
                        logo: 'Logo pending ...',
                        decimals: 18,
                    },
                });
            }
            return lenders;
        });
    }
    /**
     * @param data
     * @@description transforms the data received from the subgraph to pool detail
     */
    transformToPoolDetail(data) {
        return __awaiter(this, void 0, void 0, function* () {
            const borrowTokens = data.map((a) => a.collateralAsset);
            const collateralTokens = data.map((a) => a.borrowAsset);
            const allTokens = [...borrowTokens, ...collateralTokens].filter((value, index, array) => array.indexOf(value) === index);
            for (let index = 0; index < allTokens.length; index++) {
                const element = allTokens[index];
                yield this.tokenManager.updateAll(element);
            }
            const transformedData = data.map((a) => __awaiter(this, void 0, void 0, function* () {
                return {
                    address: a.id,
                    poolType: a.loanStatus,
                    borrowedAmount: {
                        value: new bignumber_js_1.BigNumber(a.borrowAmountRequested).toFixed(0),
                        decimals: this.tokenManager.getTokenDecimals(a.borrowAsset),
                    },
                    lentAmount: { value: new bignumber_js_1.BigNumber(a.amountBorrowed).toFixed(0), decimals: this.tokenManager.getTokenDecimals(a.collateralAsset) },
                    borrowRate: new bignumber_js_1.BigNumber(a.borrowRate).div(new bignumber_js_1.BigNumber(10).pow(16)).toFixed(2),
                    nextPayment: new bignumber_js_1.BigNumber(a.nextPaymentDeadline).toString(),
                    repaymentProgress: new bignumber_js_1.BigNumber(this.getRandomInt(10000)).div(100).toFixed(2),
                    borrowAsset: {
                        address: a.borrowAsset,
                        name: this.tokenManager.getTokenName(a.borrowAsset),
                        pricePerAssetInUSD: (yield this.tokenManager.getPricePerAsset(a.borrowAsset)).toString(),
                        logo: this.tokenManager.getLogo(a.borrowAsset),
                        decimals: this.tokenManager.getTokenDecimals(a.borrowAsset),
                    },
                    collateralAsset: {
                        address: a.collateralAsset,
                        name: this.tokenManager.getTokenName(a.collateralAsset),
                        pricePerAssetInUSD: (yield this.tokenManager.getPricePerAsset(a.collateralAsset)).toString(),
                        logo: this.tokenManager.getLogo(a.collateralAsset),
                        decimals: this.tokenManager.getTokenDecimals(a.collateralAsset),
                    },
                    estimatedEndDate: new bignumber_js_1.BigNumber(this.getRandomInt(1000000)).multipliedBy(new bignumber_js_1.BigNumber(10).pow(4)).toString(),
                    lockedCollateral: new bignumber_js_1.BigNumber(this.getRandomInt(10000)).div(100).toFixed(2),
                    collectionProgress: new bignumber_js_1.BigNumber(this.getRandomInt(100)).toFixed(2),
                    lent: { value: new bignumber_js_1.BigNumber(this.getRandomInt(1000000)).toFixed(0), decimals: 2 },
                    profit: { value: new bignumber_js_1.BigNumber(this.getRandomInt(100000)).toFixed(2), decimals: 2 },
                    endedOn: new bignumber_js_1.BigNumber(this.getRandomInt(1000000)).multipliedBy(1000).toString(),
                };
            }));
            return Promise.all(transformedData);
        });
    }
    /**
     * @param address: address of the user to query
     * @description Return the profiel overview of the user
     */
    getProfileOverview(address) {
        return __awaiter(this, void 0, void 0, function* () {
            const pools = yield this.getPoolByBorrower(address);
            const poolsCreated = pools.length;
            const activePools = pools.filter((a) => a.poolType === 'Active').length;
            const timesDefaulted = pools.filter((a) => a.poolType === 'Defaulted').length;
            const totalAmountInBorrow = new bignumber_js_1.BigNumber(this.getRandomInt(100000000)).div(100).toFixed(2);
            return {
                poolsCreated: poolsCreated.toString(),
                activePools: activePools.toString(),
                timesDefaulted: timesDefaulted.toString(),
                totalAmountInBorrow,
            };
        });
    }
}
exports.PoolCalls = PoolCalls;
//# sourceMappingURL=Pools.js.map