"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.UserMetaCalls = void 0;
const SavingsAccount_1 = require("./SavingsAccount");
const queries_1 = require("../queries");
class UserMetaCalls extends SavingsAccount_1.SavingsAccountCalls {
    constructor(url, signer, tokenManager, config) {
        super(url, signer, tokenManager, config);
    }
    getUserMetadata(user) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getUserMetadata)(this.subgraphUrl, user);
            return this.transformToUserMetaData(result);
        });
    }
    transformToUserMetaData(data) {
        const _data = data.map((a) => __awaiter(this, void 0, void 0, function* () {
            let userDetails;
            if (a.verifier == 'Twitter Verifier') {
                userDetails = yield this.getTwitterDetails(a.metadata);
            }
            else {
                userDetails = yield this.getKycDetails(a.metadata);
            }
            return {
                id: a.id,
                verifier: {
                    type: this.verificationApi.getVerifierType(a.verifiedBy.id),
                    address: a.verifiedBy.id,
                    displayName: this.verificationApi.getVerifierDisplayName(a.verifiedBy.id),
                },
                details: userDetails,
                verifiedBy: a.verifiedBy,
            };
        }));
        return Promise.all(_data);
    }
    getTwitterDetails(twitterMetaData) {
        return __awaiter(this, void 0, void 0, function* () {
            const result = yield (0, queries_1.getTwitterDetails)(twitterMetaData);
            return result;
        });
    }
    getKycDetails(kycMetaData) {
        return __awaiter(this, void 0, void 0, function* () {
            return { name: kycMetaData };
        });
    }
    /**
     *
     * @param address Address of the user to fetch
     * @returns
     */
    getVerifiedTwitterId(address) {
        return __awaiter(this, void 0, void 0, function* () {
            const addr = address.toLocaleLowerCase();
            const result = yield (0, queries_1.getTwitterId)(this.subgraphUrl, addr);
            return [...result].map((a) => {
                return {
                    id: a.id,
                    metadata: a.metadata,
                };
            });
        });
    }
}
exports.UserMetaCalls = UserMetaCalls;
//# sourceMappingURL=UserMeta.js.map