import { Provider, TransactionReceipt } from '@ethersproject/providers';
import { Signer } from '@ethersproject/abstract-signer';
import { SublimeConfig } from './types/sublimeConfig';
import { PoolApi } from './api/pool';
import { CreditLineApi } from './api/creditLines';
import { YieldAndStrategyApi } from './api/yieldAndStrategy';
import { SavingsAccountApi } from './api/savingsAccount';
import { RepaymentApi } from './api/repayments';
import { VerificationAPI } from './api/verification';
import { TokenApi } from './api/tokenApi';
import { PriceOracleApi } from './api/priceOracle';
import { CreditLineEthUtils } from './api/utils/creditLineEthUtils';
import { PoolEthUtils } from './api/utils/poolEthUtils';
import { SavingAccountsEthUtils } from './api/utils/savingsAccountEthUtils';
import { WrappedEth } from './api/utils/wrappedEth';
import { PooledCreditLineApi } from './api/pooledCreditLines';
import { TokenManager } from './tokenManager';
import { ERC1155 } from './api/erc1155';
/**
 * @class SublimeConnector
 */
export declare class SublimeConnector {
    private provider;
    private signer;
    private config;
    private tokenManager;
    constructor(provider: Provider, signer: Signer, config: SublimeConfig, tokenManager: TokenManager);
    PoolApi(): PoolApi;
    CreditLinesApi(): CreditLineApi;
    SavingsAccountApi(): SavingsAccountApi;
    /**
     *
     * @returns The current address of the signer
     */
    Address(): Promise<string>;
    RepaymentApi(): RepaymentApi;
    VerificationApi(): VerificationAPI;
    YieldAndStrategyApi(): YieldAndStrategyApi;
    TokenApi(tokenAddress: string): TokenApi;
    PriceOracleApi(): PriceOracleApi;
    PooledCreditlineApi(): PooledCreditLineApi;
    CreditLineEthUtils(): CreditLineEthUtils;
    PoolEthUtils(): PoolEthUtils;
    SavingAccountsEthUtils(): SavingAccountsEthUtils;
    WrappedEth(): WrappedEth;
    ERC1155(): ERC1155;
    getChainId(): Promise<number>;
    /**
     *
     * @param address
     * @returns balance in ethers (decimals included)
     */
    getBalance(address: string): Promise<string>;
    /**
     *
     * @param transactionHash
     * @param confirmations
     * @param timeout
     * @returns transaction receipt after waiting for the specified number oif blocks
     */
    waitForTransaction(transactionHash: string, confirmations?: number, timeout?: number): Promise<TransactionReceipt>;
}
