"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SublimeConnector = void 0;
const pool_1 = require("./api/pool");
const creditLines_1 = require("./api/creditLines");
const yieldAndStrategy_1 = require("./api/yieldAndStrategy");
const savingsAccount_1 = require("./api/savingsAccount");
const repayments_1 = require("./api/repayments");
const verification_1 = require("./api/verification");
const tokenApi_1 = require("./api/tokenApi");
const priceOracle_1 = require("./api/priceOracle");
const creditLineEthUtils_1 = require("./api/utils/creditLineEthUtils");
const poolEthUtils_1 = require("./api/utils/poolEthUtils");
const savingsAccountEthUtils_1 = require("./api/utils/savingsAccountEthUtils");
const wrappedEth_1 = require("./api/utils/wrappedEth");
const pooledCreditLines_1 = require("./api/pooledCreditLines");
const ethers_1 = require("ethers");
const erc1155_1 = require("./api/erc1155");
/**
 * @class SublimeConnector
 */
class SublimeConnector {
    constructor(provider, signer, config, tokenManager) {
        this.provider = provider;
        this.signer = signer;
        this.config = config;
        this.tokenManager = tokenManager;
    }
    PoolApi() {
        return new pool_1.PoolApi(this.signer, this.config, this.tokenManager);
    }
    CreditLinesApi() {
        return new creditLines_1.CreditLineApi(this.signer, this.config, this.tokenManager);
    }
    SavingsAccountApi() {
        return new savingsAccount_1.SavingsAccountApi(this.signer, this.config, this.tokenManager);
    }
    /**
     *
     * @returns The current address of the signer
     */
    Address() {
        return this.signer.getAddress();
    }
    RepaymentApi() {
        return new repayments_1.RepaymentApi(this.signer, this.config, this.tokenManager);
    }
    VerificationApi() {
        return new verification_1.VerificationAPI(this.signer, this.config);
    }
    YieldAndStrategyApi() {
        return new yieldAndStrategy_1.YieldAndStrategyApi(this.signer, this.config, this.tokenManager);
    }
    TokenApi(tokenAddress) {
        return new tokenApi_1.TokenApi(this.signer, tokenAddress, this.tokenManager);
    }
    PriceOracleApi() {
        return new priceOracle_1.PriceOracleApi(this.signer, this.config);
    }
    PooledCreditlineApi() {
        return new pooledCreditLines_1.PooledCreditLineApi(this.signer, this.config, this.tokenManager);
    }
    CreditLineEthUtils() {
        return new creditLineEthUtils_1.CreditLineEthUtils(this.signer, this.config, this.tokenManager);
    }
    PoolEthUtils() {
        return new poolEthUtils_1.PoolEthUtils(this.signer, this.config, this.tokenManager);
    }
    SavingAccountsEthUtils() {
        return new savingsAccountEthUtils_1.SavingAccountsEthUtils(this.signer, this.config, this.tokenManager);
    }
    WrappedEth() {
        return new wrappedEth_1.WrappedEth(this.signer);
    }
    ERC1155() {
        return new erc1155_1.ERC1155(this.signer, this.config);
    }
    getChainId() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.signer.getChainId();
        });
    }
    /**
     *
     * @param address
     * @returns balance in ethers (decimals included)
     */
    getBalance(address) {
        return __awaiter(this, void 0, void 0, function* () {
            let balance = ethers_1.ethers.BigNumber.from(0);
            try {
                balance = yield this.provider.getBalance(address);
            }
            catch (ex) { }
            return ethers_1.ethers.utils.formatEther(balance);
        });
    }
    /**
     *
     * @param transactionHash
     * @param confirmations
     * @param timeout
     * @returns transaction receipt after waiting for the specified number oif blocks
     */
    waitForTransaction(transactionHash, confirmations, timeout) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.provider.waitForTransaction(transactionHash, confirmations, timeout);
        });
    }
}
exports.SublimeConnector = SublimeConnector;
//# sourceMappingURL=sublime.js.map