import { Signer } from 'ethers';
import { BigNumber } from 'bignumber.js';
import { Asset } from './types/Types';
/**
 * @class TokenManager
 * @description An token manager stores, retrieves and caches the data like token name, decimals, logos and prices.
 */
export declare class TokenManager {
    /**
     * @description caches number of decimals for a token address
     */
    private decimals;
    /**
     * @description caches name of token
     */
    private names;
    /**
     * @description cache the price of token, Updates every 1 min
     */
    private prices;
    /**
     * @description stores when last time when the token price was updated
     */
    private priceLastUpdatedAt;
    /**
     * @description caches the token logo
     */
    private logos;
    /**
     * @description used for mapping tokens logos of testnet
     */
    private addressMapper;
    /**
     * @description Base path for logos
     */
    private logoUrlTemplate;
    /**
     * @description subgraph URL that fetches prices
     */
    private priceSubgraphUrl;
    /**
     * @description interval at which prices are refreshed in the SDK
     */
    private priceRefreshInterval;
    private chainlinkFeedsAddress;
    /**
     * @description web3 signer
     */
    private signer;
    private subgraph;
    constructor(signer: Signer, priceSubgraphUrl: string);
    /**
     * @description Updates the token meta data
     * @param tokenAddress
     */
    updateAll(tokenAddress: string): Promise<void>;
    /**
     * @description Updated the token decimals
     * @param tokenAddress
     * @returns
     */
    updateTokenDecimals(tokenAddress: string): Promise<void>;
    /**
     * @description Updates token name
     * @param tokenAddress
     * @returns
     */
    updateTokenName(tokenAddress: string): Promise<void>;
    /**
     * @description Updates token price
     * @param tokenAddress
     * @returns
     */
    updatePricePerAsset(tokenAddress: string): Promise<void>;
    private _updatePricePerAsset;
    /**
     * @description Updates token Logo
     * @param tokenAddress
     * @returns
     */
    updateLogo(tokenAddress: string): Promise<void>;
    /**
     * @param tokenAddress
     * @returns number of token decimals
     */
    getTokenDecimals(tokenAddress: string): number;
    /**
     * @param tokenAddress
     * @returns token name
     */
    getTokenName(tokenAddress: string): string;
    /**
     * @param tokenAddress
     * @returns price of asset
     */
    getPricePerAsset(tokenAddress: string): Promise<BigNumber>;
    /**
     * @param tokenAddress
     * @returns return url which renders the logo/pic of the token
     */
    getLogo(tokenAddress: string): string;
    /**
     * @param tokenAddress
     * @returns
     */
    getAssetMeta(tokenAddress: string): Promise<Asset>;
}
